SUBROUTINE Decisions

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER		:: iWl,iz,ie,iul,iu,izthis(2),iy,io,nlocopt,ibestopt(1),ibp,ib,ibestded(1),iWul
REAL(8)		:: lxN,ltot,lass,lmuc,lcon,lval,lhfn,lxa,lxb,lvalded(ngpd)
REAL(8)		:: clocopt(2*nbrackets+4),alocopt(2*nbrackets+4),slocopt(2*nbrackets+4),vlocopt(2*nbrackets+4),llocopt(2*nbrackets+4),hlocopt(2*nbrackets+4),lpythis(2)
REAL(8), EXTERNAL	:: FnBondPrice, FnBondPriceD, FnTax,FnValAssChoice, FnNoAdjEE, FnEE,FnInvQA, FnPortfolio
REAL(8)				:: xb1(nbrackets),xb2(nbrackets),fb1(nbrackets),fb2(nbrackets)
REAL(8)				:: xb1a(nbracketsa),xb2a(nbracketsa),fb1a(nbracketsa),fb2a(nbracketsa)
INTEGER				:: nsoln,is



DO gij = Jwork,1,-1
	if (Display>=1) write(*,*) 'Solving for decision rules at age ', gij			
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!! CHOICE SPECIFIC DECISION RULES !!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


    !$OMP PARALLEL DO PRIVATE(lxa,lxb,lass,lval,lhfn,nlocopt,clocopt,alocopt,slocopt,llocopt,vlocopt,hlocopt,io,ibestopt,ibestded,lvalded,ib,ibp,xb1,xb2,fb1,fb2,xb1a,xb2a,fb1a,fb2a,nsoln,is) COPYIN(gij)
	DO giW = 1,ngpW
	DO giu = 1,ngpu !could combine grid to parallelize
		gif = fWind(giW)
		giz = zWind(giW)	!note this, and gif are not needed in last working period, unless they are correlated with medical shocks
		gip = (giz-1)*ngpfy+gif			!except that pension point is final period z point
		gyps = ypsgrid(gij,gif,giz)


		!!!! ADJUST ILLIQUID ASSETS !!!!
		gadjcost = fixcost + varcost*gyps*(1.0-ugrid(giu)*(1.0-uemprepl))
		gutilcost = -log(1-utilcost)	!with - it is a positive number, so we subtract it below
	
		DO gix = 1,ngpxA !could combine with iW and parallelize
		DO gid = 1,ngpd	!this is the choice variable for next period deduction
		
			gcoh = xAgrid(gij,gix,gif,giz) - gadjcost
			gsavmax = min(gcoh - cmin*(1.0+ctax),lmax+amax)
			IF (gij<Jwork) gliqmin = abl(gij+1,gif,giz)			
			IF (gij==Jwork) gliqmin = 0.0
			gsavmin = FnBondPrice(gliqmin,0.0_8)*gliqmin
					
			IF(gcoh<=gsavmin) THEN !fixed cost might make it infeasible
				gcon = cmin*(1.0+ctax)
				lval  = 1.0e-20
				conA(gij,gix,gid,giW,giu) = gcon
				valA(gij,gix,gid,giW,giu) = EZAGG(gcon,lval) - gutilcost
				GO TO 20
			END IF
			
			nlocopt = 0
			
			!1. check for local min at constrained liquid 
			IF (FnEE(gsavmin) .ge. 0.0) THEN	!both constrained
				nlocopt  = nlocopt+1
				clocopt(nlocopt) = (gcoh-gsavmin)/(1.0+ctax)
				alocopt(nlocopt) = 0.0
				slocopt(nlocopt) = gsavmin
				llocopt(nlocopt) = gliqmin				
			END IF
			
			
			!2. check for solution with sav<0.0
			IF (gsavmin<0.0) THEN
				lxa = gsavmin
				lxb = min(-1.0e-10,gsavmax)
				IF (FnEE(lxb)>0.0 .and. FnEE(lxa)<0.0) THEN
					gass = 0.0
					CALL rtbis(FnEE,lxa,lxb,bisxtol,bisftol,gsav)
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					alocopt(nlocopt) = 0.0
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq				
				ELSE IF(gsavmax<0.0) THEN	!constrained at gsavmax: so consume cmin
					gcon = cmin/(1.0+ctax)
					gsav = gsavmax
					gliq = FnInvQA(gsav,0.0_8)
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					alocopt(nlocopt) = 0.0
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq				
				END IF
			END IF			
			
			
			!3. check for solution with sav=0.0
5			IF (gsavmin<0.0 .and. gsavmax>0.0) THEN !if gsavmin==0 then we have already checked it above
				IF ((FnEE(-1.0e-10_8) .le. 0.0_8) .and. (FnEE(0.0d0) .ge. 0.0)) THEN	!local solution here
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcoh/(1.0+ctax)
					alocopt(nlocopt) = 0.0
					slocopt(nlocopt) = 0.0
					llocopt(nlocopt) = 0.0
				END IF
			END IF			
			
			
			!4. check for solution with sav>0.0 and positive liquid
			IF (gsavmax>0.0) THEN
				lxa = 1.0e-10
				lxb = gsavmax
				IF (FnEE(lxb)<=0.0) THEN	!upper bounds binds
					IF (gliq>10.0*bisxtol) THEN
						nlocopt  = nlocopt+1
						gsav = gsavmax
						clocopt(nlocopt) = gcon
						alocopt(nlocopt) = gass
						slocopt(nlocopt) = gsav
						llocopt(nlocopt) = gliq
					END IF
				ELSE IF (FnEE(lxa)<0.0) THEN				
					CALL zbrak(FnEE,lxa,lxb,nbrackets,xb1,xb2,fb1,fb2,nsoln)
					IF(nsoln>=1) THEN
						DO is = 1,nsoln
						IF(fb2(is)>fb1(is)) THEN 	!SOC holds, possible solution: 2nd derivative positive in a'						
							lxa = xb1(is)
							lxb = xb2(is)
							CALL rtbis(FnEE,lxa,lxb,bisxtol,bisftol,gsav)
							IF (gliq>10.0*bisxtol) THEN
								nlocopt = nlocopt +1				
								clocopt(nlocopt) = gcon
								alocopt(nlocopt) = gass
								slocopt(nlocopt) = gsav
								llocopt(nlocopt) = gliq
							END IF 	 
						END IF
						END DO
					END IF
				END IF
			END IF
			
			
			!5. check for solution with sav>0.0 and zero liquid
			IF (gsavmax>0.0) THEN
				lxa = 0.0
				lxb = gsavmax/qill
				gUsePortfolioAsEE = 1
				IF (FnPortfolio(lxb)<=0.0) THEN			
					gsav = gsavmax	
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					alocopt(nlocopt) = gass
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq
				ELSE  ! use zbrak to search for regions containing multiple local optima and compare values
					CALL zbrak(FnPortfolio,lxa,lxb,nbracketsa,xb1a,xb2a,fb1a,fb2a,nsoln)
					IF(nsoln>=1) THEN
						DO is = 1,nsoln
! 							IF(fb2a(is)>fb1a(is)) THEN 	!SOC holds, possible solution: 2nd derivative positive in a'
								nlocopt = nlocopt +1
								lxa = xb1a(is)
								lxb = xb2a(is)
								CALL rtbis(FnPortfolio,lxa,lxb,bisxtol,bisftol,gass)
								clocopt(nlocopt) = gcon
								alocopt(nlocopt) = gass
								slocopt(nlocopt) = gsav
								llocopt(nlocopt) = gliq
! 							END IF 
						END DO
					END IF
				END IF				
				gUsePortfolioAsEE = 0						
			END IF

			!6. explicitly check for soution with sav>0.0 and zero illiquid
			IF (gsavmax>0.0) THEN
				lxa = 1.0e-10_8
				lxb = gsavmax
				gImposeA0inEE = 1
				IF (FnEE(lxa)<0.0 .and. FnEE(lxb)>0.0) THEN		! use zbrak to search for regions containing multiple local optima and compare values
					CALL zbrak(FnEE,lxa,lxb,nbrackets,xb1,xb2,fb1,fb2,nsoln)
					IF(nsoln>=1) THEN
						DO is = 1,nsoln
							IF(fb2(is)>fb1(is)) THEN 	!SOC holds, possible solution: 2nd derivative positive in a'
								nlocopt = nlocopt +1
								lxa = xb1(is)
								lxb = xb2(is)
								CALL rtbis(FnEE,lxa,lxb,bisxtol,bisftol,gsav)
								clocopt(nlocopt) = gcon
								alocopt(nlocopt) = gass
								slocopt(nlocopt) = gsav
								llocopt(nlocopt) = gliq
 							END IF 
						END DO
					END IF
				END IF				
				gImposeA0inEE = 0
			END IF
			


			IF (nlocopt==0) THEN
								
				if (Display>=1) write(*,*) 'no solutions to FOC found'				
! 				pause
				
			ELSE IF (nlocopt>=1) THEN
				!calculate value at each local optimum
! 				write(*,*) 'number of solutions to FOCs is: ',nlocopt
				DO io = 1,nlocopt
					gass = alocopt(io)
					gliq = llocopt(io) + rebateamt*rebatenext																		

					IF (gij<Jwork) THEN
						IF(Display>=1 .and. (gass<agrid(gij+1,1,gif,giz) .or. gliq <lgrid(gij+1,1,gif,giz))) write(*,*) 'trying to extrapolate out of range (adjust)'
						CALL Bi45DegInterp1(ngpl,lgrid(gij+1,:,gif,giz),ngpa,agrid(gij+1,:,gif,giz),evalnext(gij+1,:,:,gid,giW,giu),gliq,gass,lval,1)							
						
					ELSE  IF (gij==Jwork) THEN	!nb - use grid for lowest medical fixed effect
						IF(Display>=1 .and. (gass<agridret(1,1,gip) .or. gliq <lgridret(1,1,gip))) write(*,*) 'trying to extrapolate out of range (adjust)'
						CALL Bi45DegInterp1(ngplret,lgridret(1,:,gip),ngparet,agridret(1,:,gip),evalretJ1next(:,:,gid,gip),gliq,gass,lval,1)							
											
					END IF				
				
					vlocopt(io) = EZAGG(clocopt(io)+servflow*gass+liqflow*max(llocopt(io),0.0),lval)
				
				END DO !io
				ibestopt = maxloc(vlocopt(1:nlocopt))
	
				conA(gij,gix,gid,giW,giu) = clocopt(ibestopt(1))
				savA(gij,gix,gid,giW,giu) = slocopt(ibestopt(1))
				assA(gij,gix,gid,giW,giu) = alocopt(ibestopt(1))
				liqA(gij,gix,gid,giW,giu) = llocopt(ibestopt(1))
				valA(gij,gix,gid,giW,giu) = vlocopt(ibestopt(1)) - gutilcost
				
				!adjust for tolerance level in  bisection
				IF (abs(liqA(gij,gix,gid,giW,giu))<bisxtol*10.0) liqA(gij,gix,gid,giW,giu) = 0.0
				IF (abs(assA(gij,gix,gid,giW,giu))<bisxtol*10.0) assA(gij,gix,gid,giW,giu) = 0.0
				IF (abs(savA(gij,gix,gid,giW,giu))<bisxtol*10.0) savA(gij,gix,gid,giW,giu) = 0.0
				

			END IF
			
20			lval = lval !marker for GO TO
		END DO !gid
		END DO !gix
! 		
! 		
		
		!!!! DO NOT ADJUST ILLIQUID ASSETS !!!!

		!find corresponding value of savings for current cash on hand
		DO gix = 1,ngpxN
		DO gia = 1,ngpa
		DO gid = 1,ngpd	!this is the state variable carried over from last period
			gded = dgrid(gid)
			
			gass = (agrid(gij,gia,gif,giz) + gded*(gyps-FnTax(taxsysN,gyps,1,0,taxincrN)))/qill 	!next period asset holding, need to add on current period deduction
			gcoh = xNgrid(gij,gix,gia,gif,giz,gid)

			IF (gij<Jwork) gliqmin = abl(gij+1,gif,giz)			
			IF (gij==Jwork) gliqmin = 0.0
			gsavmin = FnBondPrice(gliqmin,0.0_8)*gliqmin
			gsavmax = min(gcoh - cmin*(1.0+ctax),lmax)
			nlocopt = 0

			IF(gsavmin>=gsavmax) THEN !might be infeasible due to taxes on illiquid assets: we will make sure he never goes there
				gsav = gsavmin
				gcon = cmin*(1.0+ctax)
				gliq = FnInvQA(gsav,0.0_8)
				nlocopt  = nlocopt+1
				clocopt(nlocopt) = gcon
				slocopt(nlocopt) = gsav
				llocopt(nlocopt) = gliq
				GO TO 60			
			END IF
						
			!1. check for binding borrowing constraint
			IF(FnNoAdjEE(gsavmin)>=0.0) THEN	
				gsav = gsavmin
				gcon = (gcoh-gsav)/(1.0+ctax)
				gliq = FnInvQA(gsav,0.0_8)
				nlocopt  = nlocopt+1
				clocopt(nlocopt) = gcon
				slocopt(nlocopt) = gsav
				llocopt(nlocopt) = gliq				
				
			END IF

			!2. not borrowng constrained, upper bound binds
			IF(FnNoAdjEE(gsavmax)<=0.0) THEN
				gsav = gsavmax
				gcon = (gcoh-gsav)/(1.0+ctax)
				gliq = FnInvQA(gsav,0.0_8)
				nlocopt  = nlocopt+1
				clocopt(nlocopt) = gcon
				slocopt(nlocopt) = gsav
				llocopt(nlocopt) = gliq				
				
			END IF
			
			!3. positive interior solution 
			IF (gsavmax>0.0) THEN
				lxa = 0.0_8
				lxb = gsavmax
				CALL zbrak(FnNoAdjEE,lxa,lxb,nbrackets,xb1,xb2,fb1,fb2,nsoln)
				IF(nsoln>=1) THEN
					DO is = 1,nsoln
						nlocopt = nlocopt +1
						lxa = xb1(is)
						lxb = xb2(is)
						CALL rtbis(FnNoAdjEE,lxa,lxb,bisxtol,bisftol,gsav)
						clocopt(nlocopt) = gcon
						slocopt(nlocopt) = gsav
						llocopt(nlocopt) = gliq
					END DO
				END IF
			END IF
									
			!4. check for solution at 0 savings, in discontinuity range
			IF (gsavmin<0.0 .and. gsavmax>0.0) THEN
				IF(FnNoAdjEE(0.0_8)>0.0 .and. FnNoAdjEE(-1.0e-8_8)<0.0) THEN
					gsav = 0.0_8
					gcon = (gcoh-gsav)/(1.0+ctax)
					gliq = 0.0_8
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq				
					
				END IF
			END IF

			!5. negative interior solution 
			IF (gsavmin<-1.0e-8_8) THEN
				lxa = gsavmin
				lxb = min(-1.0e-8,gsavmax)
				IF (FnNoAdjEE(lxb)>0.0 .and. FnNoAdjEE(lxa)<0.0) THEN
					CALL rtbis(FnNoAdjEE,lxa,lxb,bisxtol,bisftol,gsav)
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq				
				ELSE IF(gsavmax<0.0) THEN	!constrained at gsavmax: so consume cmin
					gcon = cmin/(1.0+ctax)
					gsav = gsavmax
					gliq = FnInvQA(gsav,0.0_8)
					nlocopt  = nlocopt+1
					clocopt(nlocopt) = gcon
					slocopt(nlocopt) = gsav
					llocopt(nlocopt) = gliq				
				END IF
						
			END IF

60			IF (nlocopt==0) THEN
								
				if (Display>=1) write(*,*) 'no solutions to FOC found (no adjust)'				
			ELSE IF (nlocopt>=1) THEN
				DO io = 1,nlocopt
					gliq = llocopt(io) + rebateamt*rebatenext

					IF(gij<Jwork) THEN
						IF(Display>=1 .and. (gass<agrid(gij+1,1,gif,giz) .or. gliq <lgrid(gij+1,1,gif,giz)-1.0e-6)) write(*,*) 'trying to extrapolate out of range (no adjust)'
						CALL Bi45DegInterp1(ngpl,lgrid(gij+1,:,gif,giz),ngpa,agrid(gij+1,:,gif,giz),evalnext(gij+1,:,:,gid,giW,giu),gliq,gass,lval,1)
						CALL Bi45DegInterp1(ngpl,lgrid(gij+1,:,gif,giz),ngpa,agrid(gij+1,:,gif,giz),hfnnext(gij+1,:,:,gid,giW,giu),gliq,gass,lhfn,0)							

					ELSE IF (gij==Jwork) THEN		!use grid for lowest medical fixed effect
						!start retirement with no deductions as default, unless they adjust in last working period
						IF(Display>=1 .and. (gass<agridret(1,1,gip) .or. gliq <lgridret(1,1,gip))) write(*,*) 'trying to extrapolate out of range (no adjust)'
						CALL Bi45DegInterp1(ngplret,lgridret(1,:,gip),ngparet,agridret(1,:,gip),evalretJ1next(:,:,1,gip),gliq,gass,lval,1)							
						CALL Bi45DegInterp1(ngplret,lgridret(1,:,gip),ngparet,agridret(1,:,gip),hfnretJ1next(:,:,1,gip),gliq,gass,lhfn,0)											

					END IF
					vlocopt(io) = EZAGG(clocopt(io)+servflow*gass+liqflow*max(llocopt(io),0.0),lval)
					hlocopt(io) = lhfn
					
				END DO !io
				
				ibestopt = maxloc(vlocopt(1:nlocopt))
				conN(gij,gix,gia,gid,giW,giu) = clocopt(ibestopt(1))
				savN(gij,gix,gia,gid,giW,giu) = slocopt(ibestopt(1))
				liqN(gij,gix,gia,gid,giW,giu) = llocopt(ibestopt(1))
				valN(gij,gix,gia,gid,giW,giu) = vlocopt(ibestopt(1))
				mucN(gij,gix,gia,gid,giW,giu) = (clocopt(ibestopt(1)) + servflow*gass + liqflow*max(llocopt(ibestopt(1)),0.0))**(-sigC)
				gfn(gij,gix,gia,gid,giW,giu)  = (bet/qill)*hlocopt(ibestopt(1)) + servflow*mucN(gij,gix,gia,gid,giW,giu)/qill

			END IF

			!adjust for tolerance level in  bisection
			IF (abs(liqN(gij,gix,gia,gid,giW,giu))<bisxtol*10.0) liqN(gij,gix,gia,gid,giW,giu) = 0.0
			IF (abs(savN(gij,gix,gia,gid,giW,giu))<bisxtol*10.0) savN(gij,gix,gia,gid,giW,giu) = 0.0
		
		END DO	!gid
		END DO	!gia
		END DO	!gix


		!work out adjustment
		DO gix = 1,ngpxN
		DO gia = 1,ngpa
		DO gid = 1,ngpd	!state carried over from last period
			!choose best deduction if adjust
			DO gidn = 1,ngpd	!choice of next period
				lxA = xNgrid(gij,gix,gia,gif,giz,gid) + agrid(gij,gia,gif,giz) + dgrid(gid)*(gyps-FnTax(taxsys,gyps,1,0,taxincr))
				CALL LinInterp1(ngpxA,xAgrid(gij,:,gif,giz),valA(gij,:,gidn,giW,giu),lxA,lvalded(gidn))
			END DO
			ibestded = maxloc(lvalded(:))
			dval(gij,gix,gia,gid,giW,giu) = lvalded(ibestded(1)) - valN(gij,gix,gia,gid,giW,giu)
		END DO
		END DO
		END DO

	END DO !giu
	END DO !giW
	!$OMP END PARALLEL DO



	!!!!!!!!!!!!!!!!!!!!
	!! EMAX FUNCTIONS !!
	!!!!!!!!!!!!!!!!!!!!

	IF(gij>1) THEN !no point calculating in first period

	!$OMP PARALLEL DO PRIVATE(iWl,iul,iz,ie,iu,iy,lxN,lxA,ltot,lass,lmuc,lcon,lval,lhfn,izthis,lpythis) COPYIN(gij)
	DO iWul = 1,ngpW*ngpu*ngpl
		iWl = WfromWul(iWul)
		iul = ufromWul(iWul)  !employment last period
		gif = fWind(iWl)
		giz = zWind(iWl)
		gil = lfromWul(iWul)
		
		DO gia = 1,ngpa
		DO gid = 1,ngpd	!state carred over from last period
			gded = dgrid(gid)

			eval(gij,gil,gia,gid,iWl,iul) = 0.0
			emuc(gij,gil,gia,gid,iWl,iul) = 0.0
			hfn(gij,gil,gia,gid,iWl,iul) = 0.0

			lass = agrid(gij,gia,gif,giz)
			
			DO iu = 1,ngpu	!employment this period
				IF(iul==1 .and. iu==1) THEN !emp both			
			
					DO iz = 1,ngpzy
					DO ie = 1,ngpey
					
!						lass = agrid(gij,gia,gif,giz) + gded*ypsgrid(gij,gif,iz)
						lxN = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie) -FnTax(taxsysN,ygrid(gij,gif,iz,ie),1,0,taxincrN) - mturcg*lass - gded*(ypsgrid(gij,gif,iz)-FnTax(taxsysN,ypsgrid(gij,gif,iz),1,0,taxincrN))
! 						lxA = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie) -FnTax(taxsysN,ygrid(gij,gif,iz,ie),1,0,taxincrN) - mturcg*lass + lass
						lxA = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie) -FnTax(taxsysN,ygrid(gij,gif,iz,ie),1,0,taxincrN) - mturcg*lass + lass
						giW = Wind(gif,iz)		
		
						CALL DecEmaxInterps(gij,gif,gid,giW,lass,lxA,lxN,iu,iz,lval,lcon,lhfn,lmuc)
		
						
						IF(gamC .ne. 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + (lval**(1.0-gamC))*zytrans(gij-1,giz,iz)*eydist(ie)*(1.0-jobloss(gij-1))
						IF(gamC == 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + log(lval)*zytrans(gij-1,giz,iz)*eydist(ie)*(1.0-jobloss(gij-1))
						
						emuc(gij,gil,gia,gid,iWl,iul) = emuc(gij,gil,gia,gid,iWl,iul) + lmuc*zytrans(gij-1,giz,iz)*eydist(ie)*(1.0-jobloss(gij-1))
						hfn(gij,gil,gia,gid,iWl,iul) = hfn(gij,gil,gia,gid,iWl,iul) + lhfn*zytrans(gij-1,giz,iz)*eydist(ie)*(1.0-jobloss(gij-1))
					END DO	!iz
					END DO !ie

 				ELSE IF(iul==1 .and. iu==2) THEN !emp, then unemp
 					iz = giz
					!NOTE UNEMPLOYMENT WITH DEDUCTIONS IS NOT CODED PROPERLY
					lxN = lgrid(gij,gil,gif,giz) + uemprepl*ypsgrid(gij,gif,iz) -FnTax(taxsysN,uemprepl*ypsgrid(gij,gif,iz),1,0,taxincrN) - mturcg*lass
					lxA = lgrid(gij,gil,gif,giz) + uemprepl*ypsgrid(gij,gif,iz) -FnTax(taxsysN,uemprepl*ypsgrid(gij,gif,iz),1,0,taxincrN) - mturcg*lass + lass
 					giW = Wind(gif,iz)
 					
 					CALL DecEmaxInterps(gij,gif,gid,giW,lass,lxA,lxN,iu,iz,lval,lcon,lhfn,lmuc)
		 					
 					IF(gamC .ne. 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + (lval**(1.0-gamC))*jobloss(gij-1)
 					IF(gamC == 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + log(lval)*jobloss(gij-1)
					emuc(gij,gil,gia,gid,iWl,iul) = emuc(gij,gil,gia,gid,iWl,iul) + lmuc*jobloss(gij-1)
					hfn(gij,gil,gia,gid,iWl,iul) = hfn(gij,gil,gia,gid,iWl,iul) + lhfn*jobloss(gij-1)
				
 				ELSE IF(iul==2 .and. iu==1) THEN !unemp, then emp
					!NOTE UNEMPLOYMENT WITH DEDUCTIONS IS NOT CODED PROPERLY
 					iz = giz
 					IF (ngpzy>1) THEN
 						CALL FindLinProb1 ((1.0-uemplosspers)*ypsgrid(gij,gif,iz),ypsgrid(gij,gif,:),izthis,lpythis)
 					ELSE
 						izthis(:) = 1
 						lpythis(:) = 0.5
 					END IF
 					
 					DO iy = 1,2
 					    iz = izthis(iy)
						lxN = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans)  -FnTax(taxsysN,ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans) ,1,0,taxincrN) - mturcg*lass - gded*ypsgrid(gij,gif,iz)*(1.0-uemplosstrans) 
! 						lxA = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans)  -FnTax(taxsysN,ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans) ,1,0,taxincrN) - mturcg*lass + lass
						lxA = lgrid(gij,gil,gif,giz) + ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans)  -FnTax(taxsysN,ygrid(gij,gif,iz,ie)*(1.0-uemplosstrans) ,1,0,taxincrN) - mturcg*lass + lass
						giW = Wind(gif,iz)		
		
						CALL DecEmaxInterps(gij,gif,gid,giW,lass,lxA,lxN,iu,iz,lval,lcon,lhfn,lmuc)
 						
 						IF(gamC .ne. 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + (lval**(1.0-gamC))*jobfind(gij-1)*lpythis(iy)
 						IF(gamC == 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + log(lval)*jobfind(gij-1)*lpythis(iy)
						emuc(gij,gil,gia,gid,iWl,iul) = emuc(gij,gil,gia,gid,iWl,iul) + lmuc*jobfind(gij-1)*lpythis(iy)
						hfn(gij,gil,gia,gid,iWl,iul) = hfn(gij,gil,gia,gid,iWl,iul) + lhfn*jobfind(gij-1)*lpythis(iy)
					
 					END DO !iy
 
 				ELSE IF(iul==2 .and. iu==2) THEN !unemp both
 					iz = giz
					!NOTE UNEMPLOYMENT WITH DEDUCTIONS IS NOT CODED PROPERLY
					lxN = lgrid(gij,gil,gif,giz) + uemprepl*ypsgrid(gij,gif,iz) -FnTax(taxsysN,uemprepl*ypsgrid(gij,gif,iz),1,0,taxincrN) - mturcg*lass
					lxA = lgrid(gij,gil,gif,giz) + uemprepl*ypsgrid(gij,gif,iz) -FnTax(taxsysN,uemprepl*ypsgrid(gij,gif,iz),1,0,taxincrN) - mturcg*lass + lass
 					giW = Wind(gif,iz)


					IF(gamC .ne. 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + (lval**(1.0-gamC))*(1.0d0-jobfind(gij-1))
 					IF(gamC == 1.0) eval(gij,gil,gia,gid,iWl,iul) = eval(gij,gil,gia,gid,iWl,iul) + log(lval)*(1.0d0-jobfind(gij-1))
					emuc(gij,gil,gia,gid,iWl,iul) = emuc(gij,gil,gia,gid,iWl,iul) + lmuc*(1.0d0-jobfind(gij-1))
					hfn(gij,gil,gia,gid,iWl,iul) = hfn(gij,gil,gia,gid,iWl,iul) + lhfn*(1.0d0-jobfind(gij-1))

				END IF	!employment cases
			END DO !iu

		END DO !gid
		END DO !gia
	END DO !iWl*iul*gil
	!$OMP END PARALLEL DO
	
	IF(gamC .ne. 1.0) eval(gij,:,:,:,:,:) = eval(gij,:,:,:,:,:)**(1.0/(1.0-gamC))
	IF(gamC == 1.0) eval(gij,:,:,:,:,:) = exp(eval(gij,:,:,:,:,:))
	emuc(gij,:,:,:,:,:) = emuc(gij,:,:,:,:,:) * (eval(gij,:,:,:,:,:)**(gamC-sigC))
	hfn(gij,:,:,:,:,:) = hfn(gij,:,:,:,:,:) * (eval(gij,:,:,:,:,:)**(gamC-sigC))
	
	
	IF(TransitionPath == .false.) THEN
		emucnext(gij,:,:,:,:,:) = emuc(gij,:,:,:,:,:)
		evalnext(gij,:,:,:,:,:) = eval(gij,:,:,:,:,:)
		hfnnext(gij,:,:,:,:,:) = hfn(gij,:,:,:,:,:)
	END IF
	
	
	END IF !gij>1

	
END DO	!j: age




END SUBROUTINE Decisions
