REAL(8) FUNCTION FnEE(lsav)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

REAL(8), INTENT(IN) :: lsav
REAL(8)             :: lemuc,lqpldq,llxa,llxb,lassL,lassH,lerr,lval,lnliq
REAL(8), EXTERNAL	:: FnBondPrice,FnBondPriceD,FnInvQA,FnPortfolio
REAL(8)				:: xb1(nbrackets),xb2(nbrackets),fb1(nbrackets),fb2(nbrackets),xlocopt(nbrackets),vlocopt(nbrackets)
INTEGER				:: nsoln,is,nlopt,ibestopt(1)

gsav = lsav
gcon =  (gcoh-gsav)/(1.0+ctax)


!if evaluating at savings<=0, then cannot invest in illiquid assets
IF (gsav <= 0.0 .or. gImposeA0inEE==1) THEN
	gass = 0.0
	gliq = FnInvQA(gsav,0.0_8)
	IF(abs(gliq)<1.0e-10_8) gliq = 0.0
	
	lnliq = gliq + rebateamt*rebatenext																		
	lqpldq = FnBondPrice(gliq,0.0_8) + gliq*FnBondPriceD(gliq,0.0_8)	!this is where discontinuity at zero is introduced.

	IF(gij<Jwork) CALL TransformBi45DegInterp1(-1.0/sigC,ngpl,lgrid(gij+1,:,gif,giz),ngpa,agrid(gij+1,:,gif,giz),emucnext(gij+1,:,:,gid,giW,giu),lnliq,gass,lemuc)							
	IF (gij==Jwork)	CALL TransformBi45DegInterp1(-1.0/sigC,ngplret,lgridret(1,:,gip),ngparet,agridret(1,:,gip),emucretJ1next(:,:,gid,gip),lnliq,gass,lemuc)							
	
	gfoc = lemuc/lqpldq + (1.0+ctax)*liqflow*((gcon+servflow*gass+liqflow*gliq)**(-sigC))/(lqpldq*bet)
	GO TO 10
END IF

!evaluating at savings>0: so m>=0, a>=0


lassL = 0.0
lassH = gsav/qill

!first check for corners
nlopt = 0
IF (FnPortfolio(lassL)>0.0) THEN
	nlopt = nlopt+1
	xlocopt(nlopt) = lassL
END IF
IF (FnPortfolio(lassH)<0.0) THEN
	nlopt = nlopt+1
	xlocopt(nlopt) = lassH
END IF

! use zbrak to search for regions local optima and then check soc
CALL zbrak(FnPortfolio,lassL,lassH,nbrackets,xb1,xb2,fb1,fb2,nsoln)


! write(*,*) 'FnEE: no. of possible solution brackets for FnPortfortfolio', nsoln

IF(nsoln>0) THEN
	DO is = 1,nsoln
		IF(fb2(is)>fb1(is)) THEN 	!SOC holds, possible solution
			nlopt = nlopt +1
			llxa = xb1(is)
			llxb = xb2(is)
			CALL rtbis(FnPortfolio,llxa,llxb,bisxtol,bisftol,xlocopt(nlopt))
		END IF 
	END DO


END IF

! write(*,*) 'FnEE: no. of local optima for FnPortfortfolio', nlopt

IF(nlopt==0) THEN
	write(*,*) 'FnPortfolio: no solutions found'
	write(*,*) gij,giW,giu
	write(*,*) gif,giz,gip
	write(*,*) nsoln,fb1(1),fb2(1)
	write(*,*) lassL,lassH
	write(*,*) FnPortfolio(lassL)
	write(*,*) FnPortfolio(lassH)
	
	pause
ELSE IF (nlopt==1) THEN	!only one local optimum
	gass = xlocopt(1)
	lerr = FnPortfolio(gass) 
	GO TO 10
ELSE IF (nlopt>1) THEN !evalute value functions for possible local optima
	DO is = 1,nlopt
		gass =  xlocopt(is)
		gliq = FnInvQA(gsav-qill*gass,0.0_8)
		IF(abs(gliq)<1.0e-10_8) gliq = 0.0_8
		
		lnliq = gliq + rebateamt*rebatenext																			

		IF (gij<Jwork) THEN
			CALL Bi45DegInterp1(ngpl,lgrid(gij+1,:,gif,giz),ngpa,agrid(gij+1,:,gif,giz),evalnext(gij+1,:,:,gid,giW,giu),lnliq,gass,lval,1)							

		ELSE  IF (gij==Jwork) THEN
			CALL Bi45DegInterp1(ngplret,lgridret(1,:,gip),ngparet,agridret(1,:,gip),evalretJ1next(:,:,gid,gip),lnliq,gass,lval,1)							
		END IF	
	
		vlocopt(is) = EZAGG(gcon+servflow*gass+liqflow*max(gliq,0.0),lval)
	END DO
	
	ibestopt = maxloc(vlocopt(1:nlopt))
	gass = xlocopt(ibestopt(1))
	lerr = FnPortfolio(gass) 
	GO TO 10
	
END IF



10 FnEE = (gcon+servflow*gass+liqflow*max(gliq,0.0))**(-sigC) - bet*gfoc	!gfoc is defined inside FnPortfolio and is different depending on whether m=0 or a=0 or neither.

END FUNCTION FnEE





