REAL(8) FUNCTION FnEERet(lsav)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

REAL(8), INTENT(IN) :: lsav
REAL(8)             :: lemuc,lqpldq,lxa,lxb,lassL,lassH,lerr,lval,lnliq
REAL(8), EXTERNAL	:: FnBondPrice,FnBondPriceD,FnInvQA,FnPortfolioRet
REAL(8)				:: xb1(nbrackets),xb2(nbrackets),fb1(nbrackets),fb2(nbrackets),xlocopt(nbrackets),vlocopt(nbrackets)
INTEGER				:: nsoln,is,nlopt,ibestopt(1)

gsav = lsav
gcon =  (gcoh-gsav)/(1.0+ctax)


!if evaluating at lowest possible savings, then cannot invest in illiquid assets
IF (gsav<=0.0  .or. gImposeA0inEE==1) THEN
	gass = 0.0
	gliq = FnInvQA(gsav,0.0_8)
	IF(abs(gliq)<1.0e-10_8) gliq = 0.0
	lqpldq = FnBondPrice(gliq,0.0_8) + gliq*FnBondPriceD(gliq,0.0_8)
	lnliq = gliq + rebateamt*rebatenext	
	CALL TransformBi45DegInterp1(-1.0/sigC,ngplret,lgridret(gij+1,:,gip),ngparet,agridret(gij+1,:,gip),emucretnext(gij+1,:,:,gid,giR),lnliq,gass,lemuc)							
	gfoc = lemuc/lqpldq + (1.0+ctax)*liqflow*((gcon+servflow*gass+liqflow*gliq)**(-sigC))/(lqpldq*bet)
	GO TO 10
END IF

! use zbrak to search for regions containing multiple local optima and compare values: only need to look for positive liquid
lassL = 0.0
lassH = gsav/qill

!first check for corners
nlopt = 0
IF (FnPortfolioRet(lassL)>0.0) THEN
	nlopt = nlopt+1
	xlocopt(nlopt) = lassL
END IF
IF (FnPortfolioRet(lassH)<0.0) THEN
	nlopt = nlopt+1
	xlocopt(nlopt) = lassH
END IF

! use zbrak to search for regions local optima and then check soc
CALL zbrak(FnPortfolioRet,lassL,lassH,nbrackets,xb1,xb2,fb1,fb2,nsoln)


!write(*,*) 'FnEERet: no. of possible solution brackets for FnPortfortfolioRet', nsoln

IF(nsoln>0) THEN
	DO is = 1,nsoln
 		IF(fb2(is)>fb1(is)) THEN 	!SOC holds, possible solution
			nlopt = nlopt +1
			lxa = xb1(is)
			lxb = xb2(is)
			CALL rtbis(FnPortfolioRet,lxa,lxb,bisxtol,bisftol,xlocopt(nlopt))
 		END IF 
	END DO
END IF



! write(*,*) 'FnEERet: no. of local optima for FnPortfortfolioRet', nlopt
IF(nlopt==0) THEN
	write(*,*) 'FnPortfolioRet: no solutions found'
	write(*,*) nsoln,fb1(1),fb2(1)
	write(*,*) lassL,lassH
	write(*,*) FnPortfolioRet(lassL)
	write(*,*) FnPortfolioRet(lassH)
	
	pause
ELSE IF (nlopt==1) THEN	!only one local optimum
	gass = xlocopt(1)
	lerr = FnPortfolioRet(gass) 
	GO TO 10
ELSE IF  (nlopt>1) THEN !evalute value functions for possible local optima
	DO is = 1,nlopt
		gass =  xlocopt(is)
		gliq = FnInvQA(gsav-qill*gass,0.0_8)
		IF(abs(gliq)<1.0e-10_8) gliq = 0.0
		lnliq = gliq + rebateamt*rebatenext	
		CALL Bi45DegInterp1(ngplret,lgridret(gij+1,:,gip),ngparet,agridret(gij+1,:,gip),evalretnext(gij+1,:,:,gid,giR),lnliq,gass,lval,1)							
		vlocopt(is) = EZAGG(gcon+servflow*gass+liqflow*gliq,lval)
	
	END DO

	ibestopt = maxloc(vlocopt(1:nlopt))
	gass = xlocopt(ibestopt(1))
	lerr = FnPortfolioRet(gass) 
	GO TO 10
END IF

10 FnEERet = (gcon+servflow*gass+liqflow*gliq)**(-sigC) - bet*gfoc	!gfoc is defined inside FnPortfolioRet and is the (equal) sides of the first-order condition









END FUNCTION FnEERet
