REAL(8) FUNCTION  FnGridPers(lx)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

REAL(8), INTENT(IN)         :: lx
REAL(8),DIMENSION(Jwork)    :: lwidth,lvar
REAL(8)                     :: ltemp1,ltemp2,ltemp3,ltemp4,lvetacond
INTEGER             :: iz1,iz2,ij

lvetacond = Vetay/lambday	!variance of innovation conditional on a change

! get boundaries and fill in wijh equally spaced points
DO ij = 1,Jwork
    zygrid(ij,1)    = -lx*sqrt(Vzy(ij))
    zygrid(ij,ngpzy) = lx*sqrt(Vzy(ij))
    lwidth(ij) = (zygrid(ij,ngpzy)-zygrid(ij,1))/real(ngpzy-1)
    DO iz1 = 2, ngpzy-1
        zygrid(ij,iz1) = zygrid(ij,1) + lwidth(ij)*(iz1-1)
    END DO        
END DO

! fill in transijion matrix using normal distribution
DO ij = 1,Jwork-1
    DO iz1 = 1,ngpzy
        CALL cumnor ((zygrid(ij+1,1)+0.5*lwidth(ij+1)-rhoy*zygrid(ij,iz1))/sqrt(lvetacond), zytrans(ij,iz1,1), ltemp2)    
		zytrans(ij,iz1,1) = lambday*zytrans(ij,iz1,1)
		IF(iz1==1) zytrans(ij,iz1,1) = zytrans(ij,iz1,1) + 1.0-lambday

	    DO iz2 = 2,ngpzy-1
	        CALL cumnor ((zygrid(ij+1,iz2)+0.5*lwidth(ij+1)-rhoy*zygrid(ij,iz1))/sqrt(lvetacond), ltemp1, ltemp2)
	        CALL cumnor ((zygrid(ij+1,iz2)-0.5*lwidth(ij+1)-rhoy*zygrid(ij,iz1))/sqrt(lvetacond), ltemp3, ltemp4)
            zytrans(ij,iz1,iz2) = ltemp1 - ltemp3
			zytrans(ij,iz1,iz2) = lambday*zytrans(ij,iz1,iz2)
			IF(iz1==iz2) zytrans(ij,iz1,iz2) = zytrans(ij,iz1,iz2) + 1.0-lambday
        END DO

        CALL cumnor ((zygrid(ij+1,ngpzy)-0.5*lwidth(ij+1)-rhoy*zygrid(ij,iz1))/sqrt(lvetacond), ltemp3,zytrans(ij,iz1,ngpzy))
		zytrans(ij,iz1,ngpzy) = lambday*zytrans(ij,iz1,ngpzy)
		IF(iz1==ngpzy) zytrans(ij,iz1,ngpzy) = zytrans(ij,iz1,ngpzy) + 1.0-lambday

        zytrans(ij,iz1,:) = zytrans(ij,iz1,:)/sum(zytrans(ij,iz1,:))
    END DO
END DO    


!find distribution at first period
CALL cumnor( (zygrid(1,1)+0.5*lwidth(1))/sqrt(Vzy(1)), zydist(1,1), ltemp2)	
DO iz1 = 2,ngpzy-1
    CALL cumnor ((zygrid(1,iz1)+0.5*lwidth(1))/sqrt(Vzy(1)), ltemp1, ltemp2)
    CALL cumnor ((zygrid(1,iz1)-0.5*lwidth(1))/sqrt(Vzy(1)), ltemp3, ltemp4)
    zydist(1,iz1) = ltemp1 - ltemp3
END DO
CALL cumnor ((zygrid(1,ngpzy)-0.5*lwidth(1))/sqrt(Vzy(1)), ltemp3, zydist(1,ngpzy))
zydist(1,:) = zydist(1,:)/sum(zydist(1,:))

!find uncondijional distributions
DO ij = 2,Jwork
    zydist(ij,:) = MATMUL(zydist(ij-1,:),zytrans(ij-1,:,:))
    zydist(ij,:) = zydist(ij,:) /sum(zydist(ij,:) )
END DO


!find variance
DO ij = 1,Jwork
    lvar(ij) = DOT_PRODUCT(zygrid(ij,:)**2,zydist(ij,:)) - DOT_PRODUCT(zygrid(ij,:),zydist(ij,:))**2
END DO

!moment
FnGridPers  = SUM((Vzy -lvar)**2)


END FUNCTION FnGridPers
