MODULE Globals
USE Parameters

IMPLICIT NONE

!GLOBALS FOR DIRECTORIES
character(len=100)  OutputDir
character(len=100)  OutputDirSims
character(len=100)  InputParamFile

integer::allquarters,InfiniteFixedCostModel,rebtype

!GLOBALS FOR GRIDS

real(8), allocatable              	:: fylevgrid(:)        !fixed effect component levels, earnings
real(8), allocatable              	:: fyslogrid(:)        !fixed effect component slopes, earnings


real(8), dimension(ngpey)              		:: eygrid        !transitory component, earnings
real(8), dimension(Jwork,ngpzy)             :: zygrid        !permanent component, earnings
real(8), dimension(ngpu)             		:: ugrid        !indicator for unemployment

REAL(8)                 ::      BushDebt

real(8), dimension(Jwork,ngpfy,ngpzy,ngpey) :: ygrid,holdygrid 		!earnings
real(8), dimension(Jwork,ngpfy,ngpzy) 		:: ypsgrid,holdypsgrid 		!persistent component of earnings

real(8), dimension(Jwork,ngpxA,ngpfy,ngpzy)  		:: xAgrid        !cash on hand
real(8), dimension(Jwork,ngpxN,ngpa,ngpfy,ngpzy,ngpd)  	:: xNgrid        !cash on hand
real(8), dimension(Jwork,ngpl,ngpfy,ngpzy)  		:: lgrid        !liquid asset grid
real(8), dimension(Jwork,ngpa,ngpfy,ngpzy)  		:: agrid        !illiquid asset grid

real(8), dimension(Jret,ngpxAret,ngpp)  			:: xAgridret     !cash on hand
real(8), dimension(Jret,ngpxNret,ngparet,ngpp,ngpd) :: xNgridret     !cash on hand
real(8), dimension(Jret,ngplret,ngpp)  			:: lgridret     !liquid asset grid
real(8), dimension(Jret,ngparet,ngpp)  			:: agridret     !illiquid asset grid

real(8), dimension(Jtot,ngpp)       	:: pgrid,holdpgrid        !pensions grid

real(8), dimension(ngpd)					:: dgrid		!deductions grid
real(8), dimension(ngpdret)					:: dgridret		!deductions grid

real(8), dimension(Jwork,ngpfy)	        :: nbl		! natural borrowing limits (liquid assets)
real(8), dimension(Jwork,ngpfy,ngpzy)	    :: abl		! actual borrowing limits
real(8), dimension(Jret,ngpp)	        :: nblret,ablret		! borrowing limits in retirement (liquid assets)

!GLOBALS FOR INDICATORS
integer, dimension(ngpW)			:: zWind,fWind
integer, dimension(ngpR)			:: pRind
integer, dimension(ngpfy,ngpzy)			:: Wind
integer, dimension(ngpp)			:: Rind
integer, dimension(ngpR*ngplret)			:: RfromRl,lfromRl
integer, dimension(ngpp*ngplret)			:: pfromplret,lfromplret 
integer, dimension(ngpW*ngpu*ngpl)			:: WfromWul,ufromWul,lfromWul
integer, dimension(ngpfy)					:: fysloind,fylevind
integer, dimension(ngpfylev,ngpfyslo)		:: fyind

! integer  :: UseEZValFn

!GLOBALS FOR DISTRIBUTIONS AND TRANSITION MAPRICES
real(8), allocatable		:: initliq(:)
real(8), allocatable		:: initill(:)
real(8), allocatable		:: initnw(:)
real(8), dimension(:,:), allocatable		:: initliqdist
real(8), dimension(:,:), allocatable		:: initnwdist
real(8), dimension(:,:), allocatable		:: initilldist	
real(8), allocatable		    :: fylevdist(:)	
real(8), allocatable		    :: fyslodist(:)	

real(8), dimension(ngpey)		    :: eydist	
real(8), dimension(Jwork-1,ngpzy,ngpzy)	:: zytrans		
real(8), dimension(Jwork,ngpzy)			:: zydist
real(8), dimension(Jwork,ngpu)		:: udist

real(8), dimension(Jwork)		    :: kappay
real(8), dimension(Jwork)			:: Vzy		

!GLOBALS TO STORE DECISION RULES 
real(8), dimension(Jret,ngpxAret,ngpd,ngpR)	        :: valAret,liqAret,conAret,assAret,savAret
real(8), dimension(Jret,ngpxNret,ngparet,ngpd,ngpR)	        :: valNret,conNret,liqNret,mucNret,savNret,gfnret,dvalret
real(8), dimension(Jret,ngplret,ngparet,ngpd,ngpR)	        :: evalret,emucret,hfnret
real(8), dimension(ngplret,ngparet,ngpd,ngpp)	        :: evalretJ1,emucretJ1,hfnretJ1

real(8), dimension(Jwork,ngpxA,ngpd,ngpW,ngpu)	    	:: valA,liqA,conA,assA,savA
real(8), dimension(Jwork,ngpxN,ngpa,ngpd,ngpW,ngpu)	    :: valN,conN,liqN,mucN,savN,gfn,dval
real(8), dimension(Jwork,ngpl,ngpa,ngpd,ngpW,ngpu)	    :: eval,emuc,hfn,conN1a



!PARAMETER GLOBALS
real(8)     :: Vfey,Vepsy,Vetay,Vzy0,rhoy,lambday,Vfem,Vepsm,Vetam,rhom,agem,age2m,targetUsualKY,Vphy,phvarygrowth,targetMeanIlliquid,targetMedianIlliquid,targetP75Illiquid,rholevslo
real(8)		:: inflation,nomretliq,nomretill,nomborrow,qfree,qill,qborr,taxurcg,mturcg,nomretgov,qgov,taxbase,phaseout,phaseoutamt,servflow,ndweight,liqflow
real(8)     :: gamC,sigC,borrowlim,bondparam,ratematch,debtmatch,maxdeduction,maxdedret
real(8)     :: bet,fixcost,varcost,utilcost,recdepth,recdepthcond,recpclim,recylim
real(8)		:: ctax,liqtax,illtax,meanhealth,uemprepl,uemplosstrans,uemplosspers,jobfind(Jwork),jobloss(Jwork),initialunemp,rebateamt,pcrebatespend,mpcamount,mpcincdropamount
real(8)		:: surprob(Jret),popsize(Jtot),popdist(Jtot)
integer		:: posinitwealth,sameinitwealth,sameborrowlim
integer		:: nrebgrid
real(8), allocatable	:: rebgrid(:),rebdist(:)
real(8)         :: rebatepct,RebatePctWork,RebatePctRet

!TAX FUNCTION APPROXIMATION
real(8), dimension (n_gridtax) :: gridtax
real(8), dimension (n_taxyears, n_gridtax-1, n_coeff) :: aginc_coeff

!OTHER
real(8)						:: yavall,mavall

!GLOBALS TO STORE SIMULATION RESULTS
real(8), dimension(nsim, Jwork)	    :: ysim,yavsim,yavsimforpen,ypssim,eysim,zysim,usim,ablsim
real(8), dimension(nsim, Jtot)	    :: csim,ctotsim,flowsim,ssim,xsim,psim,taxsim,Vsim,dVsim,adjsim,penrand,incsim,mpcsim,mpadjsim,lplsim,lpl3sim,lpl6sim,mpctotsim,aveadjcost
real(8), dimension(nsim, Jtot+1)	:: asim,lsim
real(8), dimension(nsimforpercJ,Jtot/4)	:: annasim,annlsim,annincsim,annlplsim
real(8), dimension(nsimforperc*Jtot/4)	:: ratiovec
real(8), dimension(nsimforpercJ*10)		:: ratiovecJ
real(8), dimension(nsimforperc*Jtot)    :: assvecJtot

integer, dimension(nsim, Jtot)	    :: psimI
integer, dimension(nsim, Jtot+1)	:: dedsimI,mpdedsimI
integer, dimension(nsim, Jwork)	    :: eysimI,zysimI,usimI
integer, dimension(nsim)	    	:: fysimI,fyslosimI,fylevsimI

!LIFECYCLE PROFILES
real(8), dimension(Jwork)	    :: Einc,Eyav,Vloginc,Vloglabinc,Eunemp,Elabinc,Euiben
real(8), dimension(Jtot)	    :: Econ,Etotcon,Eflow,Vlogtotcon,Vlogflow,Etax,Vlogcon,Eadj,Empc,Empadj,Enetlabinc,Eliqpl,CORRillliq,CORRillliqpl,Elpl3,Elpl6,Eliq0negnoadj,Eadjcost
real(8), dimension(Jtot+1)		:: Eass,Eliq,Easspos,Eliqpos,Eliqneg,Eded,Esavpos,Enoanoliq,Eaposliq0,Eliqposapos
real(8), dimension(Jtot+1)		:: Eliq0,Eliqlim,Eliqlimapos
real(8), dimension(Jret)		:: Epen,Vlogpen
real(8), dimension(Jwork/4)		:: annEinc,annElabinc,annEuiben,annEincfe
real(8), dimension((Jwork+Jret)/4)		:: annEcon
real(8), dimension(Jtot+1,ngpd)	:: PRded
real(8), dimension(Jtot,5)	    :: DISTcon,DISTass,DISTliq,DISTlpliq,DISTlpl3,DISTlpl6
real(8), dimension(Jtot,4)	    :: MedLiqbyAssAge,MeanLiqbyAssAge,MedLiqplbyAssAge,MeanLiqplbyAssAge,MedLiqbyIncAge,MedLiqplbyIncAge,MeanLiqbyIncAge,MeanLiqplbyIncAge
real(8), dimension(5)	    	:: MEDliq10yr,MEDass10yr,MEDtot10yr,MEDliq2inc10yr,MEDass2inc10yr,MEDtot2inc10yr,Eadj10yr,Empc10yr
integer, dimension(nsim)        :: spelllength,numspells
real(8), dimension(nsim)	:: avespell
real(8), dimension(5)		:: DISTALLass,DISTALLliq,DISTALLlpl3,DISTALLlpl6,DISTALLcon
real(8), dimension(Jtot-1)	    :: Ecstr1,Ecstr2,Ecstr3,Ecstr4,Ecstr5,Ecstr1noill,Ecstr2noill,Ecstr3noill,Ecstr4noill,Ecstr5noill,Empccstr1,Empccstr2,Empccstr3,Empccstr4,Empccstr5,Empccstr5noill,Empccstr4noill

!INSURANCE COEFFICIENTS
REAL(8), DIMENSION(nsim,Jwork)		:: logc,logy
REAL(8), DIMENSION(nsim,Jwork-1)	:: eps,eta,Dc,Dy
REAL(8), DIMENSION(Jwork-1)			:: covepsDc,vareps,covetaDc,vareta,phieps,phieta

!THREADPRIVATE GLOBALS
integer							:: gij,gix,gip,gif,giz,gid,gidn,giu,giR,giW,gia,gil,gUsePortfolioAsEE,gImposeA0inEE
real(8)							:: gamax,gamin,gyps,gcoh,gcon,gliq,gass,gsav,gded,gtot,gfoc,gsavmin,gsavmax,gliqmin,gadjcost,gutilcost
!$OMP THREADPRIVATE(gij,gix,gip,gif,giz,gid,gidn,giu,giR,giW,gia,gil,gamax,gamin,gyps,gcoh,gcon,gliq,gass,gsav,gded,gtot,gfoc,gsavmin,gsavmax,gliqmin,gUsePortfolioAsEE,gImposeA0inEE,gadjcost,gutilcost)

!TRANSITION GLOBALS
logical							:: InitialSteadyState,FinalSteadyState,TransitionPath
integer							:: time,taxsys,taxsysN,rebatenext
real(8)							:: taxincr,taxincrN
real(8), dimension(Jret,ngplret,ngparet,ngpd,ngpR)	        :: evalretnext,emucretnext,hfnretnext
real(8), dimension(Jwork,ngpl,ngpa,ngpd,ngpW,ngpu)	    :: evalnext,emucnext,hfnnext
real(8), dimension(ngplret,ngparet,ngpd,ngpp)	        :: evalretJ1next,emucretJ1next,hfnretJ1next




TYPE DecisionType
	real(8), dimension(Jret,ngpxAret,ngpd,ngpR)	        :: valAret,assAret,savAret
	real(8), dimension(Jret,ngpxNret,ngparet,ngpd,ngpR)	    :: valNret,savNret,dvalret
	real(8), dimension(Jret,ngplret,ngparet,ngpd,ngpR)	    :: evalret,emucret,hfnret,htfnret
	real(8), dimension(ngplret,ngparet,ngpd,ngpp)	        :: evalretJ1,emucretJ1,hfnretJ1,htfnretJ1

	real(8), dimension(Jwork,ngpxA,ngpd,ngpW,ngpu)	    	:: valA,assA,savA
	real(8), dimension(Jwork,ngpxN,ngpa,ngpd,ngpW,ngpu)	    :: valN,savN,dval
	real(8), dimension(Jwork,ngpl,ngpa,ngpd,ngpW,ngpu)	    :: eval,emuc,hfn,htfn

END TYPE DecisionType
type(DecisionType), allocatable, target				:: decFinalSS(:),decInitialSS(:),dectrans(:),decrebateA(:,:),decrebateB(:,:)
type(DecisionType), pointer				:: gdecsim
!$OMP THREADPRIVATE(gdecsim)

TYPE SimulationType
	integer, dimension(nsimtrans)	:: jsim,psimI,eysimI,zysimI,usimI,diesimI,dedsimI
	real(8), dimension(nsimtrans)	:: ysim,ypssim,csim,ctotsim,xsim,psim,lsim,taxsim,asim,ssim,usim,rebsim,Vsim,dVsim,adjsim,flowsim
END TYPE SimulationType
type(SimulationType)			:: simtrans(-1:Ttransition+Tadditionalsim+1)	!-1 is previous steady state, Ttransition+1 is to store ass and dur decisions in simulation

integer, dimension(nsimtrans)		:: idsimI,fysimtransI,rebsimI,Simcstr5,Simcstr5noill
integer, dimension(nsimtrans)		:: rebatetypeI	!0 = no rebate, 1 = early group, 2 = late group

real(8), dimension(:,:), allocatable	:: uerand,zyrand,eyrand,dierand,initliqrand,initillrand
real(8), dimension(:), allocatable		:: initss1,initss2

real(8), dimension(-1:Ttransition+Tadditionalsim,3)	:: pathEage,pathEinc,pathEcon,pathEass,pathEliq,pathEtax,pathEasspos,pathEliqpos,pathEadjust,pathEcstr5,pathEcstr5noill,pathEliqlim,pathEliqlimapos,path0liq,path0liqposill,pathposliqposill
real(8), dimension(-1:Ttransition+Tadditionalsim,3)	:: pathEtranpos,pathEpen,pathEunemp,pathEdie,pathErebate,pathElabinc,pathEuiben,pathEavrebsize
real(8), dimension(-1:Ttransition+Tadditionalsim,12)	:: agepathEage,agepathEinc,agepathEcon,agepathEsav,agepathEass,agepathEliq,agepathEtax,agepathEasspos,agepathEliqpos
real(8), dimension(-1:Ttransition+Tadditionalsim,12)	:: agepathEtranpos,agepathEpen,agepathEunemp,agepathEdie,agepathErebate,agepathElabinc,agepathEuiben,agepathEadjust,agepathEcash

!REGRESSION RESULTS AND INPUT
INTEGER, dimension(3*(nsimrebateA+nsimrebateB))::sortord
real(8), dimension(5,4)		:: table2coef
real(8), dimension(5,4)		:: table3coef,table4coef,table5coef,table6coef
real(8), dimension(4,4,Jtot):: tableagecoef
real(8), dimension(5,4,3)   :: tablecoefbyliq,tablecoefbyage,tablecoefbylpl,tablecoefbyinc
real(8), dimension(4,12,Jtot)   :: tableliqagecoef,tablelplagecoef
!real(8), dimension((nsimrebateA+nsimrebateB)*3)	:: dCtp1,dlogCtp1,age_reg,rebamt_reg,rebind_reg,liq_reg,inc_reg,lpl_reg,tempvect
!real(8), dimension((nsimrebateA+nsimrebateB)*3,3)	:: quarter_reg
!real(8), dimension((nsimrebateA+nsimrebateB)*3,5)	:: X_reg,Z_reg
real(8), dimension(:), allocatable	                :: dCtp1,dlogCtp1,age_reg,rebamt_reg,rebind_reg,liq_reg,inc_reg,lpl_reg,tempvect
real(8), dimension(:,:), allocatable     	        :: quarter_reg
real(8), dimension(:,:), allocatable     	        :: X_reg,Z_reg

!real(8), dimension(3*(nsimrebateA+nsimrebateB))		:: tempvec,Y1,logY1
!real(8), dimension(3*(nsimrebateA+nsimrebateB),4)	:: X1,Z1

END MODULE Globals
