MODULE Parameters
IMPLICIT NONE
SAVE

integer,parameter :: long = 8

!DIRECTORIES
character(len=*), parameter	:: ParamOutputDir = "Output/"
character(len=*), parameter	:: ParamOutputDirSims = "Output/sims/"
character(len=*), parameter	:: InputDir = "Input/"			!to read input files

!OPTIONS
integer,parameter  ::  Numeraire           		= 1	!1 for annual av earns, 4 for quarterly av earns
integer,parameter  ::  Display              	= 0
integer,parameter  ::  ReadParameters           = 1
integer,parameter  ::  MatchKYRatio         	= 1
integer,parameter  ::  SolveTransition			= 1
integer,parameter  ::  DoRebateCalcs			= 1
integer,parameter  ::  GovtBudgetConstraint		= 1 !must have SolveTransition==1
integer,parameter  ::  ModelRecession			= 0	!must have SolveTransition==1
integer,parameter  ::  NoChangeTaxSystem		= 1
integer, parameter	:: Sunset					= 0 !1 if sunsets in 2011
integer,parameter  ::  BushInc					= 0 !must have sunset==1, No Change=0, GovtBu=1
integer, parameter	:: announcequ				= 3	!2 or 3: quarter of 2001 in which announcement
integer, parameter	:: rebatequ					= 3	!quarter of 2001 in which first rebate checks are sent
integer,parameter  ::  BothRebateSurprises		= 0
integer,parameter  ::  WhichTaxSystem			= 1	!1-6, only operative if NoChangeTaxSystem==1
integer,parameter  ::  Borrowing				= 1
integer,parameter  ::  RebateRegressionsInStata = 1
integer,parameter  ::  IncRebate				= 0

integer,parameter  ::  SaveSimulations	    	= 0
integer,parameter  ::  SaveTransitionSims		= 0
integer,parameter  ::  SaveDecisionRules		= 0	
integer,parameter  ::  SaveRebateSims			= 1

integer,parameter  ::  DeterministicModel		= 0	!effects how population is set up when doing rebate experiment in deterministic model with no death in retirement

integer,parameter  ::  WhichStochModel			= 1	!to allow various calibrations	
integer,parameter  ::  NoUnempShocks			= 1	!remember to set ngpu accordingly
integer,parameter  ::  NoTaxes					= 0 
integer,parameter  ::  SmoothInitialUnemp		= 1
integer,parameter  ::  SameRandomNumbersRebate	= 1	!must have same number of sims for control, rebate A and rebate B
integer,parameter  ::  NoDeathInRetirement		= 1 !must be 1 for model with EZ pref
integer,parameter  ::  FlatEarningsProfile		= 0
integer,parameter  ::  NoInitialWealth			= 0

integer,parameter  ::  SimulateAtMeanTrans		= 0
integer,parameter  ::  SimulateContinuousTrans	= 0	 !earnings and medical, only for steady state: should change this.

integer,parameter  ::  FixedPension				= 0		
integer,parameter  ::  LinearPension			= 0		
integer,parameter  ::  ConcavePension			= 1	!cant run without zy or fy risk		

integer,parameter  ::  NoExtrapXA          		= 1 !imposes hard bounds on extrapolations outside of feasible ranges
integer,parameter  ::  NewFYDist                = 0 
integer,parameter  ::  TransformValFnInterp		= 0	
integer,parameter  ::  NoRatioInterp			= 0	
integer,parameter  ::  UsePosNegDvalInterp		= 0

!WEALTH TARGET FOR BETA
integer,parameter  ::  MatchUsualKY         = 1	!this matches agg K / agg Y
integer,parameter  ::  MatchMeanIlliquid    = 0
integer,parameter  ::  MatchMedianIlliquid  = 0	
integer,parameter  ::  MatchP75Illiquid     = 0 


! GRIDS DIMENSION - STATE VARIABLES
integer,parameter :: ngpa		= 30 		    !grid for illiquid asset holdings

integer,parameter :: ngpxAneg       = 16*Borrowing         
integer,parameter :: ngpxNneg       = 16*Borrowing 
integer,parameter :: ngplneg        = 16*Borrowing 

integer,parameter :: ngpxA          = 50+ngpxAneg !cash on hand points when adjusting
integer,parameter :: ngpxN          = 30+ngpxNneg !cash on hand points when not adjusting
integer,parameter :: ngpl           = 30+ngplneg  !grid for liquid asset holdings

integer,parameter :: ngpxAret = ngpxA-ngpxAneg
integer,parameter :: ngpxNret = ngpxN-ngpxNneg
integer,parameter :: ngparet = ngpa
integer,parameter :: ngplret = ngpl-ngplneg

integer,parameter :: ngpd = 1			!grid for deductions
integer,parameter :: ngpdret = 1		!grid for deductions in retirement - must = ngpd because of last working period, when adjusting choice is over ngpdret possible deductions. would have to do last working period separately in policy functions and decisions code.

integer,parameter :: ngpey	= 1  		    !transitory component earnings
integer,parameter :: ngpzy	= 21 			!permanent component earnings
integer,parameter :: ngpfylev	= 1  				    !fixed effect earnings: levels
integer,parameter :: ngpfyslo	= 1 				    !fixed effect earnings: slopes
integer,parameter :: ngpfy = ngpfyslo*ngpfylev
integer,parameter :: ngpu	= 1					!unemployment

integer,parameter :: ngpp	= ngpzy*ngpfy  			    !number of pension points 

integer,parameter :: ngpW = ngpfy*ngpzy		!points in state for workers loop, not including x or a or l
integer,parameter :: ngpR = ngpp		!points in state for retirees loop,  not including x or a or l


!DEMOGRAPHIC PARAMETERS PARAMETERS
integer,parameter	:: Jwork    = 38*4  		!working quarters: must be multiple of 4
integer,parameter   :: Jret     = 20*4        !retirement quarters
integer,parameter   :: Jtot     = Jwork+Jret        !total years

!NUMERAIRE IN ECONOMY
real(8), parameter	:: DataAvAnnualEarns = 53000.0

!TARGET MOMENTS
real(8), parameter   :: targetWelfPerElligible = 2670.0 /(DataAvAnnualEarns/real(Numeraire))

!PENSION PARAMETERS
real(8), parameter   :: fracmainearner = 0.84
real(8), parameter   :: sstaxcap 	= 2.0 *Numeraire/4.0		!cap on earnings for calculation of ss tax as multuple of quarterly earnings
real(8), parameter   :: pencap 		= 19050.0 /(DataAvAnnualEarns/real(Numeraire)) 		!cap on (pre-tax) earnings that contribute to pension index, as fraction of av pre-tax earnigs
real(8), parameter   :: ssbend1		= 2124.0 /(DataAvAnnualEarns/real(Numeraire))		!fraction of average earnings 
real(8), parameter   :: ssbend2		= 12808.0 /(DataAvAnnualEarns/real(Numeraire))
real(8), parameter   :: meanpension		= 7000.0 /(DataAvAnnualEarns/real(Numeraire))	!needs to be positive 
real(8), parameter   :: stdvpension		=  3000.0 /(DataAvAnnualEarns/real(Numeraire))


!PARAMETERS FOR GRID CONSTRUCTION
real(8), parameter   :: xAgridparam = 0.4	!for xA: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: xNgridparam = 0.4	!for xN: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: agridparam = 0.4	!for a: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: lgridparam = 0.4	!for l: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: xAgridretparam = 0.4	!for xA: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: xNgridretparam = 0.4	!for xN: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: agridretparam = 0.4	!for a: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: lgridretparam = 0.4	!for l: approaches linear as goes to 1, approaches L shaped as goes to 0
real(8), parameter   :: amax  = 20.0*Numeraire	!multiple of quarterly/annual earnings: for illiquid asset grid: (or multiple of maximum fixed component of earnings)
real(8), parameter   :: lmax  = 20.0*Numeraire	!multiple of quarterly/annual earnings: for liquid asset grid: (or multiple of maximum fixed component of earnings)
real(8), parameter	 :: cmin = 1.0e-8 		!tolerance parameter for considering consumption is effectively zero: needed?
integer, parameter	 :: nbrackets = 10		!number of regions to look for local minima in portfolio equation
integer, parameter	 :: nbracketsa = 10		!number of regions to look for local minima in illiquid euler equation, with zero liq
real(8), parameter	 :: bisxtol = 1.0e-8	
real(8), parameter	 :: bisftol = 1.0e-10	


!SIMULATION PARAMETERS	
integer, parameter	:: nsim 	= 5000 !50000 !total number of simulations (per age for steady state)
integer, parameter	:: nsimFE	= ngpfy ! X times number of FE
integer, parameter	:: nsimtrans 	= 60000 !600000 !number of simulations for transition (whole population across ages) - must be divisible by 3 (set to 3 times Jtot if doing DeterministicModel)
integer, parameter	:: nsimrebateA	= nsimtrans/3	!to receive rebate A
integer, parameter	:: nsimrebateB 	= nsimtrans/3	!to receive rebate B : NB remainder of nsim receive no rebate
integer, parameter	:: nsimforperc 	= min(nsim,1000) ! number of simulations to use for percentiles of KY<=nsim. should be okay up to 10000
integer, parameter	:: nsimforpercJ = min(nsim,10000) ! number of simulations to use for percentiles of KY<=nsim. should be okay up to 10000

!TRANSITION PARAMETERS
integer, parameter	:: recndrop = 3	!number of peiods for decline
integer, parameter	:: recngrow = 8	!number of periods for recovery
integer, parameter	:: npergovbc = 4 !40	!if no tax change, must be at least 4. if tax change then must be at least 24 if no sunset and 40 if sunset. if model recession, must be at least length of recession

 !number of quarters that need to be solved for during transition, 0 is period of announcement, so Ttransition +1 sets of decision rules
integer, parameter	:: Ttransition = max((1-NoChangeTaxSystem)*(20 + (4-announcequ) + sunset*16), GovtBudgetConstraint*npergovbc,ModelRecession*(recndrop+recngrow-1),SolveTransition*4)
integer, parameter	:: Tadditionalsim = 2 !number of quarters to simluate transition for after transition complete >=2 if No transition


!TAX FUNCTION APPROXIMATION PARAMETERS
integer, parameter :: n_gridtax = 10
integer, parameter :: n_taxyears = 6
integer, parameter :: n_coeff = 4

END MODULE Parameters

