MODULE Procedures

USE Parameters
USE Globals

IMPLICIT NONE
CONTAINS



!-----------------------------------------------

REAL(8) FUNCTION EZAGG(lx,ly)
real(8), INTENT(in)	::  lx,ly                    

IF(sigC==1.0) THEN
	EZAGG = (lx**(1.0-bet)) * (ly**bet)
ELSE
	EZAGG = ((1.0-bet)*(lx**(1.0-sigC)) + bet*(ly**(1.0-sigC))) ** (1.0/(1.0-sigC))
END IF

END FUNCTION EZAGG


!-----------------------------------------------

REAL(8) FUNCTION INVEZ(lx)
real(8), INTENT(in)	::  lx                    

IF(gamC==1.0) THEN
	INVEZ = exp((1-bet)*lx)
ELSE
	INVEZ = (lx*(1.0-gamC)) **(1.0/(1.0-gamC))
END IF

END FUNCTION INVEZ
!-----------------------------------------------

REAL(8) FUNCTION EZ(lx)
real(8), INTENT(in)	::  lx                    

IF(gamC==1.0) THEN
	EZ = log(lx)/(1.0-bet)
ELSE
	EZ = (lx**(1.0-gamC))/ (1.0-gamC) 
END IF

END FUNCTION EZ

!-----------------------------------------------

SUBROUTINE ReverseVector(n,xin,xout)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(8), INTENT(in)		:: xin(n)
REAL(8), INTENT(out)	:: xout(n)
INTEGER					:: i
REAL(8)					:: lxin(n),lxout(n)

lxin = xin
DO i = 1,n
	lxout(i) = lxin(n-i+1)
END DO
xout = lxout

END SUBROUTINE ReverseVector
!-----------------------------------------------

SUBROUTINE LinInterp (n,x,y,ni,xi,yi)
!this does linear interpolation of (x,y) at points xi
!requires x to be sorted in ascending order
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,ni
REAL(long), INTENT(in)	:: x(:),y(:),xi(:)
REAL(long), INTENT(out)	:: yi(:)
REAL(long), DIMENSION(ni)	:: xL,xH,yL,yH
INTEGER					:: i,locL(ni)


DO i = 1,ni

	LocL(i) = MAXLOC(x,1,MASK=xi(i)>x)
	
	IF (xi(i)<=x(1)) THEN
		LocL(i) = 1
	END IF

	IF (LocL(i)>=n) THEN 
		LocL(i) = n-1
	END IF


	xL(i) = x(locL(i))
	xH(i) = x(locL(i)+1)
	yL(i) = y(locL(i))
	yH(i) = y(locL(i)+1)
	
	yi(i) = yL(i) + (xi(i)-xL(i))*((yH(i)-yL(i))/(xH(i)-xL(i)))

END DO


END SUBROUTINE LinInterp

!-----------------------------------------------

SUBROUTINE LinInterp1 (n,x,y,xi,yi)
!this does linear interpolation of (x,y) at points only point,xi
!requires x to be sorted in ascending order
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: xL,xH,yL,yH
INTEGER					:: locL

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	LocL = 1
END IF

IF (LocL>=n) THEN 
	LocL = n-1
END IF

xL  = x(locL)
xH  = x(locL +1)
yL  = y(locL)
yH  = y(locL +1)

yi  = yL  + (xi -xL )*((yH -yL )/(xH -xL ))

END SUBROUTINE LinInterp1

!-----------------------------------------------

SUBROUTINE TransformLinInterp1 (p,n,x,y,xi,yi)
!this does linear interpolation of (x,y) at points only point,xi
!requires x to be sorted in ascending order
!extrapolates out of range
!interpolates the transformed function raised to the power p
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: p,x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: xL,xH,yL,yH
INTEGER					:: locL

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	LocL = 1
END IF

IF (LocL>=n) THEN 
	LocL = n-1
END IF

xL  = x(locL)
xH  = x(locL +1)
yL  = abs(y(locL))**p
yH  = abs(y(locL +1))**p

yi  = yL  + (xi -xL )*((yH -yL )/(xH -xL ))

IF (yi<=0.0) THEN
	yi = yi
END IF
yi = yi**(1.0/p)

IF(y(n)<0.0)yi=-yi

END SUBROUTINE TransformLinInterp1
!-----------------------------------------------

SUBROUTINE InvEZLinInterp1 (n,x,y,xi,yi)
!this does linear interpolation of (x,y) at points only point,xi
!requires x to be sorted in ascending order
!extrapolates out of range
!interpolates the transformed function raised to the power p
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: xL,xH,yL,yH
INTEGER					:: locL

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	LocL = 1
END IF

IF (LocL>=n) THEN 
	LocL = n-1
END IF

xL  = x(locL)
xH  = x(locL +1)
yL  = INVEZ(y(locL))
yH  = INVEZ(y(locL +1))

yi  = yL  + (xi -xL )*((yH -yL )/(xH -xL ))

IF (yi<=0.0) THEN
	if (Display==1) write(*,*) 'extrap prob in InvEZLinInterp1'
	yi = minval(y)
ELSE
	yi = EZ(yi)
END IF


END SUBROUTINE InvEZLinInterp1
!-----------------------------------------------

SUBROUTINE ExpTransformLinInterp1 (n,x,y,xi,yi)
!this does linear interpolation of (x,y) at points only point,xi
!requires x to be sorted in ascending order
!extrapolates out of range
!interpolates the transformed function by taking exponential
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: xL,xH,yL,yH
INTEGER					:: locL

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	LocL = 1
END IF

IF (LocL>=n) THEN 
	LocL = n-1
END IF

xL  = x(locL)
xH  = x(locL +1)
yL  = exp(y(locL))
yH  = exp(y(locL +1))

yi  = yL  + (xi -xL )*((yH -yL )/(xH -xL ))

IF (yi<=0.0) THEN
	if (Display==1) write(*,*) 'extrap prob in ExpTransformLinInterp1'
	yi = minval(y)
ELSE
	yi = log(yi)
END IF


END SUBROUTINE ExpTransformLinInterp1
!-----------------------------------------------

SUBROUTINE PosNegInterp1 (n,x,y,xi,yi)
!this returns a positive number if both neighbors are (weekly) positive and a negative number if both neighbors are negative
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(:),y(:),xi
REAL(long), INTENT(out)	:: yi
REAL(long)	            :: yL,yH
INTEGER					:: locL,locH

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	locL = 1
	locH = 1
ELSE IF (LocL>=n) THEN 
	locL = n
	locH = n
ELSE
	locH = locL+1
END IF

if (y(locL)>=0.0) yL  = 1.0
if (y(locL)<0.0) yL  = 0.0
if(y(locH)>=0.0) yH  = 1.0 
if(y(locH)<0.0) yH  = 0.0

if(yL*yH>0.0) then
	yi = 1.0
else
	yi = -1.0
end if

END SUBROUTINE PosNegInterp1
!-----------------------------------------------

SUBROUTINE BiLinInterp1 (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,dxdy
INTEGER					:: xlocL,ylocL

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
END IF

IF (yLocL>=ny) THEN 
	yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = f(xlocL,ylocL)
fLH = f(xlocL,ylocL+1)
fHL = f(xlocL+1,ylocL)
fHH = f(xlocL+1,ylocL+1)

dxdy = (xH-xL)*(yH-yL)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)


END SUBROUTINE BiLinInterp1
!-----------------------------------------------

SUBROUTINE Bi45DegInterp1 (nx,x,ny,y,f,xi,yi,fi,itrans)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range?
!interpolates along the 45 degree line xi + yi = K
!put itrans=1 if you want to do the EZ transformation
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny,itrans
REAL(long), INTENT(in)	:: x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,K,ficheck
REAL(long)	            :: x1,y1,f1,x2,y2,f2
INTEGER					:: xlocL,ylocL,check

 check = 0
K = xi+yi

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
END IF

IF (yLocL>=ny) THEN 
	yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = f(xlocL,ylocL)
fLH = f(xlocL,ylocL+1)
fHL = f(xlocL+1,ylocL)
fHH = f(xlocL+1,ylocL+1)
!check corners
IF(xi==xL .and. yi==yL) THEN
	fi = fLL
	return
END IF
IF(xi==xH .and. yi==yH) THEN
	fi = fHH
	return
END IF

IF(TransformValFnInterp==1 .and. itrans==1) THEN
	fLL = INVEZ(fLL)
	fLH = INVEZ(fLH)
	fHL = INVEZ(fHL)
	fHH = INVEZ(fHH)
END IF


!DA or AB
10 IF((K-xL<=yH .and. K-xL>=yL) .or. xL+yL>K) THEN !DA
	x1 = xL
	y1 = K-xL
	f1  = fLL  + (y1 -yL )*((fLH -fLL )/(yH -yL ))
ELSE IF ((K-yH<=xH .and. K-yH>=xL) .or. xH+yH<K) THEN!AB
	x1 = K-yH
	y1 = yH
	f1  = fLH  + (x1 -xL )*((fHH -fLH )/(xH -xL ))
ELSE
	IF(Display>=2) write(*,*) 'problem 1 in Bi45DegInterp1'
END IF


!BC or CD
IF((K-xH<=yH .and. K-xH>=yL) .or. xH +yH<K) THEN !BC
	x2 = xH
	y2 = K-xH
	f2  = fHL  + (y2 -yL )*((fHH -fHL )/(yH -yL ))
ELSE IF ((K-yL<=xH .and. K-yL>=xL) .or. xL+yL>K ) THEN!CD
	x2 = K-yL
	y2 = yL
	f2  = fLL  + (x2 -xL )*((fHL -fLL )/(xH -xL ))
ELSE
	IF(Display>=2) write(*,*) 'problem 2 in Bi45DegInterp1'
END IF

!interpolate (x1,y1,f1) and (x2,y2,f2) at (xi,yi)
fi  = f1  + (xi -x1 )*((f2 -f1 )/(x2 -x1 ))

!check
ficheck = f1  + (yi -y1 )*((f2 -f1 )/(y2 -y1 ))

IF (abs(fi-ficheck)>1.0e-10) THEN
	IF(Display>=2) write(*,*) 'problem check in Bi45DegInterp1: ', abs(fi-ficheck)
END IF

IF(TransformValFnInterp==1 .and. itrans==1) THEN
	IF(fi>0.0 .and. check==0) THEN
		fi = EZ(fi)
		return
	ELSE IF (check==1) THEN
		fi = exp(fi)
		fi = EZ(fi)
		return
	ELSE
		!if fi<=0.0 then take log transform first
		check = 1
		fLL = log(fLL)
		fLH = log(fLH)
		fHL = log(fHL)
		fHH = log(fHH)
		GO TO 10
	END IF
END IF

END SUBROUTINE Bi45DegInterp1
!-----------------------------------------------

SUBROUTINE InvEZBiLinInterp1 (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,dxdy
INTEGER					:: xlocL,ylocL

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
END IF

IF (yLocL>=ny) THEN 
	yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = INVEZ(f(xlocL,ylocL))
fLH = INVEZ(f(xlocL,ylocL+1))
fHL = INVEZ(f(xlocL+1,ylocL))
fHH = INVEZ(f(xlocL+1,ylocL+1))

dxdy = (xH-xL)*(yH-yL)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)

IF(fi>0.0) THEN
	fi = EZ(fi)
	return
END IF

!if fi<=0.0 then take log transform first
fLL = log(fLL)
fLH = log(fLH)
fHL = log(fHL)
fHH = log(fHH)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)
fi = exp(fi)
fi = EZ(fi)

END SUBROUTINE InvEZBiLinInterp1
!-----------------------------------------------

SUBROUTINE BiPosNegInterp1 (nx,x,ny,y,f,xi,yi,fi)
!this returns a positive number if both neighbors are (weekly) positive and a negative number if both neighbors are negative
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: fLL,fHH,fLH,fHL
INTEGER					:: xlocL,ylocL,xlocH,ylocH

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
	xlocH = 1
ELSE IF (xLocL>=nx) THEN 
	xLocL = nx
	xlocH = nx
ELSE
	xlocH = xlocL +1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
	ylocH = 1
ELSE IF (yLocL>=ny) THEN 
	yLocL = ny
	ylocH = ny
ELSE
	ylocH = ylocL+1
END IF



fLL = f(xlocL,ylocL)
fLH = f(xlocL,ylocH)
fHL = f(xlocH,ylocL)
fHH = f(xlocH,ylocH)

if (fLL>=0.0) fLL  = 1.0
if (fLL<0.0) fLL  = 0.0

if (fLH>=0.0) fHL  = 1.0
if (fLH<0.0) fLH  = 0.0

if (fHL>=0.0) fHL  = 1.0
if (fHL<0.0) fHL  = 0.0

if (fHH>=0.0) fHH  = 1.0
if (fHH<0.0) fHH  = 0.0

if(fLL*fHH*fLH*fHL>0.0) then
	fi = 1.0
else
	fi = -1.0
end if

END SUBROUTINE BiPosNegInterp1
!-----------------------------------------------

SUBROUTINE TransformBiLinInterp1 (p,nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range
! !interpolates the transformed function raised to the power p
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: p,x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,dxdy
INTEGER					:: xlocL,ylocL

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
END IF

IF (yLocL>=ny) THEN 
	yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = abs(f(xlocL,ylocL))**p
fLH = abs(f(xlocL,ylocL+1))**p
fHL = abs(f(xlocL+1,ylocL))**p
fHH = abs(f(xlocL+1,ylocL+1))**p

dxdy = (xH-xL)*(yH-yL)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)

IF(fi>0.0) THEN
	fi = fi**(1.0/p)
	if(f(nx,ny)<0.0)fi=-fi
	return
END IF

!if fi<=0.0 then take log transform first
fLL = log(fLL)
fLH = log(fLH)
fHL = log(fHL)
fHH = log(fHH)
fi = fLL*(xH-xi)*(yH-yi)/(dxdy) + fHL*(xi-xL)*(yH-yi)/(dxdy) + fLH*(xH-xi)*(yi-yL)/(dxdy) + fHH*(xi-xL)*(yi-yL)/(dxdy)
fi = exp(fi)
fi = fi**(1.0/p)
if(f(nx,ny)<0.0)fi=-fi
return


END SUBROUTINE TransformBiLinInterp1

!-----------------------------------------------

SUBROUTINE TransformBi45DegInterp1 (p,nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!extrapolates out of range?
!interpolates along the 45 degree line xi + yi = K
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: p,x(:),y(:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yL,yH,fLL,fHH,fLH,fHL,K,ficheck
REAL(long)	            :: x1,y1,f1,x2,y2,f2
INTEGER					:: xlocL,ylocL,check

 
 check = 0
K = xi+yi

xlocL = MAXLOC(x,1,MASK=xi>x)
ylocL = MAXLOC(y,1,MASK=yi>y)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF

IF (yi<=y(1)) THEN
	ylocL = 1
END IF

IF (yLocL>=ny) THEN 
	yLocL = ny-1
END IF

xL  = x(xlocL)
xH  = x(xlocL +1)
yL  = y(ylocL)
yH  = y(ylocL +1)
fLL = abs(f(xlocL,ylocL))**p
fLH = abs(f(xlocL,ylocL+1))**p
fHL = abs(f(xlocL+1,ylocL))**p
fHH = abs(f(xlocL+1,ylocL+1))**p

!check corners
IF(xi==xL .and. yi==yL) THEN
	fi = f(xlocL,ylocL)
	return
END IF
IF(xi==xH .and. yi==yH) THEN
	fi = f(xlocL+1,ylocL+1)
	return
END IF

!DA or AB
10 IF((K-xL<=yH .and. K-xL>=yL) .or. xL+yL>K) THEN !DA
	x1 = xL
	y1 = K-xL
	f1  = fLL  + (y1 -yL )*((fLH -fLL )/(yH -yL ))
ELSE IF ((K-yH<=xH .and. K-yH>=xL) .or. xH+yH<K) THEN!AB
	x1 = K-yH
	y1 = yH
	f1  = fLH  + (x1 -xL )*((fHH -fLH )/(xH -xL ))
ELSE
		IF(Display==1) write(*,*) 'problem 1 in TransformBi45DegInterp1'
END IF


!BC or CD
IF((K-xH<=yH .and. K-xH>=yL) .or. xH +yH<K) THEN !BC
	x2 = xH
	y2 = K-xH
	f2  = fHL  + (y2 -yL )*((fHH -fHL )/(yH -yL ))
ELSE IF ((K-yL<=xH .and. K-yL>=xL) .or. xL+yL>K ) THEN!CD
	x2 = K-yL
	y2 = yL
	f2  = fLL  + (x2 -xL )*((fHL -fLL )/(xH -xL ))
ELSE
		IF(Display==1) write(*,*) 'problem 2 in TransformBi45DegInterp1'
END IF

!interpolate (x1,y1,f1) and (x2,y2,f2) at (xi,yi)
fi  = f1  + (xi -x1 )*((f2 -f1 )/(x2 -x1 ))

!check
ficheck = f1  + (yi -y1 )*((f2 -f1 )/(y2 -y1 ))

IF (abs(fi-ficheck)>1.0e-10) THEN
		IF(Display==1) write(*,*) 'problem check in TransformBi45DegInterp1: ', abs(fi-ficheck)
END IF


IF(fi>0.0 .and. check==0) THEN
	fi = fi**(1.0/p)
	if(f(nx,ny)<0.0)fi=-fi
	return
ELSE IF (check==1) THEN
	fi = exp(fi)
	fi = fi**(1.0/p)
	if(f(nx,ny)<0.0)fi=-fi
	return
ELSE
	!if fi<=0.0 then take log transform first
	check = 1
	fLL = log(fLL)
	fLH = log(fLH)
	fHL = log(fHL)
	fHH = log(fHH)
	GO TO 10
END IF

END SUBROUTINE TransformBi45DegInterp1
!-----------------------------------------------
SUBROUTINE BiLinInterp1NR_preservesign (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!allows for non-rectangular grid, by allowing y grid to be function of x
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yA,yB,yC,yD,fLL,fHH,fLH,fHL,y1,y2,f1,f2,xratio
INTEGER					:: xlocL,xlocH,ylocD, ylocC,ylocA,ylocB

xlocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF
xlocH = xlocL+1

ylocD = MAXLOC(y(:,xlocL),1,MASK=yi>y(:,xlocL))
ylocC = MAXLOC(y(:,xlocH),1,MASK=yi>y(:,xlocH))


IF (yi<=y(1,xlocL)) THEN
	ylocD = 1
END IF
IF (yi<=y(1,xlocH)) THEN
	ylocC = 1
END IF

IF (yLocD>=ny) THEN 
	yLocD = ny-1
END IF

IF (yLocC>=ny) THEN 
	yLocC = ny-1
END IF

ylocA = ylocD+1
ylocB = ylocC+1

xL  = x(xlocL)
xH  = x(xlocH)

yD  = y(ylocD,xlocL)
yC  = y(ylocC,xlocH)
yA  = y(ylocA,xlocL)
yB  = y(ylocB,xlocH)

fLL = f(xlocL,ylocD)
fLH = f(xlocL,ylocA)
fHL = f(xlocH,ylocC)
fHH = f(xlocH,ylocB)

! term1 = (fHH/xH)*(xi-xL) + fLH
! term2 = (yB/xH)*(xi-xL) + yA
! term3 = (yi-yD)-(yC/xH)*(xi-xL)
! term4 = (fHL/xH)*(xi-xL) + fLL
! fi = (term1/term2)*term3 + term4

xratio = (xi-xL)/(xH-xL)

y1 = (yC-yD)*xratio + yD
y2 = (yB-yA)*xratio + yA
f1 = (fHL-fLL)*xratio + fLL
f2 = (fHH-fLH)*xratio + fLH

fi = (yi-y1)*(f2-f1)/(y2-y1) + f1
! extrapolating might take to a bad part of range
IF(fi*fHH<0.0 .and. y2<yi) THEN
	!yi is in the right range, but the first linear interp took it out of range. use nearest point on boundary
	fi = f2
END IF
IF(fi*fLL<0.0  .and. y1>yi) THEN
	!yi is in the right range, but the first linear interp took it out of range. use nearest point on boundary
	fi = f1
END IF
! IF(fi*fHH<0.0) THEN
! 	write(*,*)'problem might be here'
! 
! END IF
! 


END SUBROUTINE BiLinInterp1NR_preservesign
!-----------------------------------------------
SUBROUTINE BiLinInterp1Ratio (nx,x,ny,y,f,xi,yi,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)				:: yri,yr(ny),lfi

IF(xi==x(1)) THEN
	write(*,*) 'cant call BiLinInterp1Ratio in closed interval'
	pause
END IF

IF(NoRatioInterp==1) THEN
	CALL BiLinInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
	fi = lfi
	return
END IF

yri = yi/(xi-x(1))
yr = y(:,nx)/(x(nx)-x(1))
CALL BiLinInterp1 (nx,x,ny,yr,f,xi,yri,lfi)
fi = lfi

END SUBROUTINE BiLinInterp1Ratio
!-----------------------------------------------
SUBROUTINE InvEZBiLinInterp1Ratio (nx,x,ny,y,f,xi,yi,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)				:: yri,yr(ny),lfi

IF(xi==x(1)) THEN
	write(*,*) 'cant call InvEZBiLinInterp1Ratio in closed interval'
	pause
END IF

IF(NoRatioInterp==1) THEN
	CALL BiLinInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
	fi = lfi
	return
END IF

yri = yi/(xi-x(1))
yr = y(:,nx)/(x(nx)-x(1))
CALL InvEZBiLinInterp1 (nx,x,ny,yr,f,xi,yri,lfi)
fi = lfi

END SUBROUTINE InvEZBiLinInterp1Ratio

!-----------------------------------------------
! SUBROUTINE BiLinInterp1RatioAB (nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr1(ngpa-ngpaNEG),lfi,yr2(ngpaNEG+1)
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiLinInterp1RatioAB in closed interval'
! 	pause
! END IF
! 
! IF(NoRatioInterp==1) THEN
! 	CALL BiLinInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! IF(ngpbNEG==0 .or. (xi>x(ngpbNEG+1) .and. yi<=xi +1.0e-8)) THEN	!in lower righttriangle
! 	yri = yi/xi
! 	yr1 = y(1:ngpa-ngpaNEG,nx)/x(nx)
! 	CALL BiLinInterp1 (ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpa-ngpaNEG,yr1,f(ngpbNEG+1:ngpb,1:ngpa-ngpaNEG),xi,yri,lfi)
! 	
! ELSE IF(xi<x(ngpbNEG+1)) THEN	!in lower left triangle
! 	yri = yi/(xi-x(1))
! 	yr2 = y(ngpa-ngpaNEG:ngpa,ngpbNEG+1)/(x(ngpbNEG+1)-x(1))
! 	CALL BiLinInterp1 (ngpbNEG+1,x(1:ngpbNEG+1),ngpaNEG+1,yr2,f(1:ngpbNEG+1,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 
! ELSE IF(yi>xi) THEN	!in upper parallelogram
! 	yri = (yi-xi)/(x(ngpbNEG)-x(1))
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,nx)-y(ngpa-ngpaNEG,nx))/(x(ngpbNEG)-x(1))
! 	CALL BiLinInterp1 (ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpaNEG+1,yr2,f(ngpbNEG+1:ngpb,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! END IF
! 
! fi = lfi
! 
! END SUBROUTINE BiLinInterp1RatioAB
! !-----------------------------------------------
! SUBROUTINE InvEZBiLinInterp1RatioAB (nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr1(ngpa-ngpaNEG),lfi,yr2(ngpaNEG+1)
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call InvEZBiLinInterp1RatioAB in closed interval'
! 	pause
! END IF
! 
! IF(NoRatioInterp==1) THEN
! 	CALL BiLinInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! IF(ngpbNEG==0 .or. (xi>x(ngpbNEG+1) .and. yi<=xi +1.0e-8)) THEN	!in lower righttriangle
! 	yri = yi/xi
! 	yr1 = y(1:ngpa-ngpaNEG,nx)/x(nx)
! 	CALL InvEZBiLinInterp1 (ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpa-ngpaNEG,yr1,f(ngpbNEG+1:ngpb,1:ngpa-ngpaNEG),xi,yri,lfi)
! 	
! ELSE IF(xi<x(ngpbNEG+1)) THEN	!in lower left triangle
! 	yri = yi/(xi-x(1))
! 	yr2 = y(ngpa-ngpaNEG:ngpa,ngpbNEG+1)/(x(ngpbNEG+1)-x(1))
! 	CALL InvEZBiLinInterp1 (ngpbNEG+1,x(1:ngpbNEG+1),ngpaNEG+1,yr2,f(1:ngpbNEG+1,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 
! ELSE IF(yi>xi) THEN	!in upper parallelogram
! 	yri = (yi-xi)/(x(ngpbNEG)-x(1))
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,nx)-y(ngpa-ngpaNEG,nx))/(x(ngpbNEG)-x(1))
! 	CALL InvEZBiLinInterp1 (ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpaNEG+1,yr2,f(ngpbNEG+1:ngpb,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! END IF
! 
! fi = lfi
! 
! END SUBROUTINE InvEZBiLinInterp1RatioAB
! !-----------------------------------------------
! SUBROUTINE TransformBiLinInterp1RatioAB (p,nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny
! REAL(long), INTENT(in)	:: p,x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr1(ngpa-ngpaNEG),lfi,yr2(ngpaNEG+1)
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiLinInterp1RatioAB in closed interval'
! 	pause
! END IF
! 
! IF(NoRatioInterp==1) THEN
! 	CALL TransformBiLinInterp1NR(p,nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! IF(ngpbNEG==0 .or. (xi>x(ngpbNEG+1) .and. yi<=xi +1.0e-8)) THEN	!in lower righttriangle
! 	yri = yi/xi
! 	yr1 = y(1:ngpa-ngpaNEG,nx)/x(nx)
! 	CALL TransformBiLinInterp1 (p,ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpa-ngpaNEG,yr1,f(ngpbNEG+1:ngpb,1:ngpa-ngpaNEG),xi,yri,lfi)
! 	
! ELSE IF(xi<x(ngpbNEG+1)) THEN	!in lower left triangle
! 	yri = yi/(xi-x(1))
! 	yr2 = y(ngpa-ngpaNEG:ngpa,ngpbNEG+1)/(x(ngpbNEG+1)-x(1))
! 	CALL TransformBiLinInterp1 (p,ngpbNEG+1,x(1:ngpbNEG+1),ngpaNEG+1,yr2,f(1:ngpbNEG+1,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 
! ELSE IF(yi>xi) THEN	!in upper parallelogram
! 	yri = (yi-xi)/(x(ngpbNEG)-x(1))
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,nx)-y(ngpa-ngpaNEG,nx))/(x(ngpbNEG)-x(1))
! 	CALL TransformBiLinInterp1 (p,ngpb-ngpbNEG,x(ngpbNEG+1:ngpb),ngpaNEG+1,yr2,f(ngpbNEG+1:ngpb,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! END IF
! 
! fi = lfi
! 
! END SUBROUTINE TransformBiLinInterp1RatioAB

!-----------------------------------------------
SUBROUTINE BiLinInterp1RatioXAret (nx,x,na,a,f,xi,ai,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,na
REAL(long), INTENT(in)	:: x(:,:),a(:),f(:,:),xi,ai
REAL(long), INTENT(out)	:: fi
REAL(long)				:: xri,xr(nx),lfi
REAL(long)				:: lxmin,lxmax

lxmin = max(0.0_8, x(1,1)-mturcg*ai)
lxmax = x(nx,1)-mturcg*ai
IF(lxmax<=lxmin) THEN
	write(*,*) 'BiLinInterp1RatioXAret: calling at infeasible point for x,a'
	pause
END IF

xri = (xi-lxmin)/(lxmax -lxmin)
IF(NoExtrapXA==1) xri = max(min(xri,1.0_8),0.0_8)	

xr = (x(:,1)-x(1,1))/(x(nx,1)-x(1,1))
CALL BiLinInterp1 (nx,xr,na,a,f,xri,ai,lfi)
fi = lfi


END SUBROUTINE BiLinInterp1RatioXAret
!-----------------------------------------------
SUBROUTINE BiLinInterp1RatioXA (ij,ify,izy,nx,x,na,a,f,xi,ai,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: ij,ify,izy,nx,na
REAL(long), INTENT(in)	:: x(:,:),a(:),f(:,:),xi,ai
REAL(long), INTENT(out)	:: fi
REAL(long)				:: xri,xr(nx),lfi
REAL(long)				:: lxmin,lxmax

! IF(ngpxNneg==0)	THEN !in positive region

	IF(ngpxNneg==0)	lxmin = max(0.0_8, x(1,1)-mturcg*ai)
	IF(ngpxNneg>0)	lxmin = max(abl(ij,ify,izy), x(1,1)-mturcg*ai)
	lxmax = x(nx,1)-mturcg*ai
	IF(lxmax<=lxmin) THEN
		write(*,*) 'BiLinInterp1RatioXAret: calling at infeasible point for x,a'
		pause
	END IF

	xri = (xi-lxmin)/(lxmax -lxmin)
	IF(NoExtrapXA==1) xri = max(min(xri,1.0_8),0.0_8)	

	xr = (x(1:nx,1)-x(1,1))/(x(nx,1)-x(1,1))

! ELSE IF (ngpXneg>0) THEN !in negative region
! 	lxmin =  max(abl(ij,iff,iz), x(1,1)-mturcg*ai)
! 	lxmax = max(0.0_8, x(ngpxNneg+1,1)-mturcg*ai)
! 	IF(lxmax<=lxmin) THEN
! 		write(*,*) 'BiLinInterp1RatioXAret: calling at infeasible point for x,a (neg)'
! 		pause
! 	END IF
! 
! 	xri = (xi-lxmin)/(lxmax -lxmin)
! 	IF(NoExtrapXA==1) xri = max(min(xri,1.0_8),0.0_8)	
! 
! 	xr = (x(1:ngpxNneg,1)-x(1,1))/(x(ngpxNneg,1)-x(1,1))
! 
! END IF
	
	
CALL BiLinInterp1 (nx,xr,na,a,f,xri,ai,lfi)
fi = lfi


END SUBROUTINE BiLinInterp1RatioXA
! !-----------------------------------------------
! SUBROUTINE BiLinInterp1RatioAX (ij,ify,izy,nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny,ij,ify,izy
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr1(ngpa-ngpaNEG),lfi,yr2(ngpaNEG+1),lytop,lybot
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiLinInterp1RatioAX in closed interval'
! 	pause
! END IF
! 
! IF(NoRatioInterp==1) THEN
! 	CALL BiLinInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! 
! IF(ngpxNEG==0 .or. (xi>=x(ngpxNEG+1) .and. yi<=max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi)) +1.0e-8)) THEN	!in lower righttriangle
! ! 	lytop = max(xi-x(ngpxNEG+1),maxval(dgrid)*(xi))
! 	lytop = max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))
! 	lybot = 0.0
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yri = min(yri,1.0)	!added to stop issues with extrapolation
! 	
! 	yr1 = y(1:ngpa-ngpaNEG,nx) / ((x(nx)-x(ngpxNEG+1))/(1.0-mturcg))
! 	CALL BiLinInterp1 (ngpx-ngpxNEG,x(ngpxNEG+1:ngpx),ngpa-ngpaNEG,yr1,f(ngpxNEG+1:ngpx,1:ngpa-ngpaNEG),xi,yri,lfi)
! 	
! ELSE IF(xi>=x(ngpxNEG+1) .and. yi>max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))) THEN	!in upper parallelogram
! 	lytop = max((xi-x(1))/(1.0-mturcg),maxval(dgrid)*(xi-abl(ij,ify,izy)))
! 	lybot = max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yri = min(yri,1.0)	!added to stop issues with extrapolation
! 	
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,nx)-y(ngpa-ngpaNEG,nx))/((x(ngpxNEG)-x(1))/(1.0-mturcg))
! 	CALL BiLinInterp1 (ngpx-ngpxNEG,x(ngpxNEG+1:ngpx),ngpaNEG+1,yr2,f(ngpxNEG+1:ngpx,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 	
! ELSE IF(xi<x(ngpxNEG+1) .and. yi>maxval(dgrid)*(xi)) THEN	!in top part of lower left triangle
! 	lytop = max((xi-x(1))/(1.0-mturcg),maxval(dgrid)*(xi-abl(ij,ify,izy)))
! 	lybot = max(maxval(dgrid)*(xi),0.0)
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yri = min(yri,1.0)	!added to stop issues with extrapolation
! 	
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,ngpxNEG+1)-max(maxval(dgrid)*x(ngpxNEG+1),0.0))/(max((x(ngpxNEG+1)-x(1))/(1.0-mturcg),maxval(dgrid)*(x(ngpxNEG+1)-abl(ij,ify,izy))) - max(maxval(dgrid)*x(ngpxNEG+1),0.0))
! 	CALL BiLinInterp1 (ngpxNEG+1,x(1:ngpxNEG+1),ngpaNEG+1,yr2,f(1:ngpxNEG+1,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 
! ELSE IF(xi<x(ngpxNEG+1) .and. yi<=maxval(dgrid)*(xi)) THEN	!in bottom part of lower left triangle: only happens with deductions
! 	lytop = maxval(dgrid)*xi
! 	lybot = 0.0
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yri = min(yri,1.0)	!added to stop issues with extrapolation
! 	write(*,*) 'in here but shouldnt be'
! 	yr1 = y(1:ngpa-ngpaNEG,ngpxNEG+1)/(maxval(dgrid)*x(ngpxNEG+1))
! 	CALL BiLinInterp1 (ngpxNEG+1,x(1:ngpxNEG+1),ngpa-ngpaNEG,yr1,f(1:ngpxNEG+1,1:ngpa-ngpaNEG),xi,yri,lfi)
! 
! END IF
! 
! fi = lfi
! 
! END SUBROUTINE BiLinInterp1RatioAX
! !-----------------------------------------------
! SUBROUTINE BiPosNegInterp1Ratio (nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr(ny),lfi
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiPosInterp1Ratio in closed interval'
! 	pause
! END IF
! IF(NoRatioInterp==1) THEN
! 	CALL BiPosNegInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! yri = yi/(xi-x(1))
! yr = y(:,nx)/(x(nx)-x(1))
! CALL BiPosNegInterp1 (nx,x,ny,yr,f,xi,yri,lfi)
! fi = lfi
! 
! END SUBROUTINE BiPosNegInterp1Ratio
! 
! !-----------------------------------------------
! SUBROUTINE BiPosNegInterp1RatioAX (ij,ify,izy,nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny,ij,ify,izy
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr1(ngpa-ngpaNEG),lfi,yr2(ngpaNEG+1),lytop,lybot
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiLinInterp1RatioAX in closed interval'
! 	pause
! END IF
! 
! IF(NoRatioInterp==1) THEN
! 	CALL BiPosNegInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! 
! IF(ngpxNEG==0 .or. (xi>=x(ngpxNEG+1) .and. yi<=max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi)) +1.0e-8)) THEN	!in lower righttriangle
! 	lytop = max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))
! 	lybot = 0.0
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yr1 = y(1:ngpa-ngpaNEG,nx) / ((x(nx)-x(ngpxNEG+1))/(1.0-mturcg))
! 	CALL BiPosNegInterp1 (ngpx-ngpxNEG,x(ngpxNEG+1:ngpx),ngpa-ngpaNEG,yr1,f(ngpxNEG+1:ngpx,1:ngpa-ngpaNEG),xi,yri,lfi)
! 	
! ELSE IF(xi>=x(ngpxNEG+1) .and. yi>max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))) THEN	!in upper parallelogram
! 	lytop = max((xi-x(1))/(1.0-mturcg),maxval(dgrid)*(xi-abl(ij,ify,izy)))
! 	lybot = max((xi-x(ngpxNEG+1))/(1.0-mturcg),maxval(dgrid)*(xi))
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,nx)-y(ngpa-ngpaNEG,nx))/((x(ngpxNEG)-x(1))/(1.0-mturcg))
! 	CALL BiPosNegInterp1 (ngpx-ngpxNEG,x(ngpxNEG+1:ngpx),ngpaNEG+1,yr2,f(ngpxNEG+1:ngpx,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 	
! ELSE IF(xi<x(ngpxNEG+1) .and. yi>maxval(dgrid)*(xi)) THEN	!in top part of lower left triangle
! 	lytop = max((xi-x(1))/(1.0-mturcg),maxval(dgrid)*(xi-abl(ij,ify,izy)))
! 	lybot = max(maxval(dgrid)*(xi),0.0)
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yr2 = (y(ngpa-ngpaNEG:ngpa,ngpxNEG+1)-max(maxval(dgrid)*x(ngpxNEG+1),0.0))/(max((x(ngpxNEG+1)-x(1))/(1.0-mturcg),maxval(dgrid)*(x(ngpxNEG+1)-abl(ij,ify,izy))) - max(maxval(dgrid)*x(ngpxNEG+1),0.0))
! 	CALL BiPosNegInterp1 (ngpxNEG+1,x(1:ngpxNEG+1),ngpaNEG+1,yr2,f(1:ngpxNEG+1,ngpa-ngpaNEG:ngpa),xi,yri,lfi)
! 
! ELSE IF(xi<x(ngpxNEG+1) .and. yi<=maxval(dgrid)*(xi)) THEN	!in bottom part of lower left triangle: only happens with deductions
! 	lytop = maxval(dgrid)*xi
! 	lybot = 0.0
! 	yri = (yi-lybot)/(lytop-lybot)
! 	yr1 = y(1:ngpa-ngpaNEG,ngpxNEG+1)/(maxval(dgrid)*x(ngpxNEG+1))
! 	CALL BiPosNegInterp1 (ngpxNEG+1,x(1:ngpxNEG+1),ngpa-ngpaNEG,yr1,f(1:ngpxNEG+1,1:ngpa-ngpaNEG),xi,yri,lfi)
! 
! END IF
! 
! fi = lfi
! 
! END SUBROUTINE BiPosNegInterp1RatioAX
! !-----------------------------------------------
! 
! SUBROUTINE BiPosNegInterp1RatioAXret (nx,x,ny,y,f,xi,yi,fi)
! 
! IMPLICIT NONE
! INTEGER, INTENT(in)		:: nx,ny
! REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
! REAL(long), INTENT(out)	:: fi
! REAL(long)				:: yri,yr(ny),lfi
! 
! IF(xi==x(1)) THEN
! 	write(*,*) 'cant call BiPosInterp1Ratio in closed interval'
! 	pause
! END IF
! IF(NoRatioInterp==1) THEN
! 	CALL BiPosNegInterp1NR(nx,x,ny,y,f,xi,yi,lfi)
! 	fi = lfi
! 	return
! END IF
! 
! yri = yi/max(maxval(dgrid)*xi,xi-x(1))
! 
! yr = y(:,nx)/(x(nx)-x(1))
! CALL BiPosNegInterp1 (nx,x,ny,yr,f,xi,yri,lfi)
! fi = lfi
! 
! 
! END SUBROUTINE BiPosNegInterp1RatioAXret
! !-----------------------------------------------
SUBROUTINE TransformBiLinInterp1Ratio (p,nx,x,ny,y,f,xi,yi,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: p,x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)				:: yri,yr(ny),lfi

IF(xi==x(1)) THEN
	write(*,*) 'cant call BiLinInterp1Ratio in closed interval'
	pause
END IF
IF(NoRatioInterp==1) THEN
	CALL TransformBiLinInterp1NR(p,nx,x,ny,y,f,xi,yi,lfi)
	fi = lfi
	return
END IF

yri = yi/(xi-x(1))
yr = y(:,nx)/(x(nx)-x(1))
CALL TransformBiLinInterp1 (p,nx,x,ny,yr,f,xi,yri,lfi)
fi = lfi

END SUBROUTINE TransformBiLinInterp1Ratio
!-----------------------------------------------

SUBROUTINE BiLinInterp1NR (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!allows for non-rectangular grid, by allowing y grid to be function of x
!extrapolates out of range
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yA,yB,yC,yD,fLL,fHH,fLH,fHL,y1,y2,f1,f2,xratio
INTEGER					:: xlocL,xlocH,ylocD, ylocC,ylocA,ylocB

xlocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF
xlocH = xlocL+1

ylocD = MAXLOC(y(:,xlocL),1,MASK=yi>y(:,xlocL))
ylocC = MAXLOC(y(:,xlocH),1,MASK=yi>y(:,xlocH))


IF (yi<=y(1,xlocL)) THEN
	ylocD = 1
END IF
IF (yi<=y(1,xlocH)) THEN
	ylocC = 1
END IF

IF (yLocD>=ny) THEN 
	yLocD = ny-1
END IF

IF (yLocC>=ny) THEN 
	yLocC = ny-1
END IF

ylocA = ylocD+1
ylocB = ylocC+1

xL  = x(xlocL)
xH  = x(xlocH)

yD  = y(ylocD,xlocL)
yC  = y(ylocC,xlocH)
yA  = y(ylocA,xlocL)
yB  = y(ylocB,xlocH)

fLL = f(xlocL,ylocD)
fLH = f(xlocL,ylocA)
fHL = f(xlocH,ylocC)
fHH = f(xlocH,ylocB)

! term1 = (fHH/xH)*(xi-xL) + fLH
! term2 = (yB/xH)*(xi-xL) + yA
! term3 = (yi-yD)-(yC/xH)*(xi-xL)
! term4 = (fHL/xH)*(xi-xL) + fLL
! fi = (term1/term2)*term3 + term4

xratio = (xi-xL)/(xH-xL)

y1 = (yC-yD)*xratio + yD
y2 = (yB-yA)*xratio + yA
f1 = (fHL-fLL)*xratio + fLL
f2 = (fHH-fLH)*xratio + fLH

fi = (yi-y1)*(f2-f1)/(y2-y1) + f1



END SUBROUTINE BiLinInterp1NR
!-----------------------------------------------

SUBROUTINE BiPosNegInterp1NR (nx,x,ny,y,f,xi,yi,fi)

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: fLL,fHH,fLH,fHL
INTEGER					:: xlocL,xlocH,ylocD, ylocC,ylocA,ylocB

xlocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	xlocL = 1
	xlocH = 1
ELSE IF (xLocL>=nx) THEN 
	xLocL = nx
	xlocH = nx
ELSE
	xlocH = xlocL+1	
END IF

ylocD = MAXLOC(y(:,xlocL),1,MASK=yi>y(:,xlocL))
ylocC = MAXLOC(y(:,xlocH),1,MASK=yi>y(:,xlocH))


IF (yi<=y(1,xlocL)) THEN
	ylocD = 1
	ylocA = 1
ELSE IF (yLocD>=ny) THEN 
	yLocD = ny
	ylocA = ny
ELSE
	ylocA = ylocD+1
END IF


IF (yi<=y(1,xlocH)) THEN
	ylocC = 1
	ylocB = 1
ELSE IF (yLocC>=ny) THEN 
	yLocC = ny
	ylocB = ny
ELSE
	ylocB = ylocC+1
END IF



fLL = f(xlocL,ylocD)
fLH = f(xlocL,ylocA)
fHL = f(xlocH,ylocC)
fHH = f(xlocH,ylocB)

if (fLL>=0.0) fLL  = 1.0
if (fLL<0.0) fLL  = 0.0

if (fLH>=0.0) fHL  = 1.0
if (fLH<0.0) fLH  = 0.0

if (fHL>=0.0) fHL  = 1.0
if (fHL<0.0) fHL  = 0.0

if (fHH>=0.0) fHH  = 1.0
if (fHH<0.0) fHH  = 0.0

if(fLL*fHH*fLH*fHL>0.0) then
	fi = 1.0
else
	fi = -1.0
end if


END SUBROUTINE BiPosNegInterp1NR
!-----------------------------------------------

SUBROUTINE TransformBiLinInterp1NR (p,nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!allows for non-rectangular grid, by allowing y grid to be function of x
!extrapolates out of range
!interpolates the transformed function raised to the power p
IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: p,x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yA,yB,yC,yD,fLL,fHH,fLH,fHL,y1,y2,f1,f2,xratio
INTEGER					:: xlocL,xlocH,ylocD, ylocC,ylocA,ylocB

xlocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF
xlocH = xlocL+1

ylocD = MAXLOC(y(:,xlocL),1,MASK=yi>y(:,xlocL))
ylocC = MAXLOC(y(:,xlocH),1,MASK=yi>y(:,xlocH))


IF (yi<=y(1,xlocL)) THEN
	ylocD = 1
END IF
IF (yi<=y(1,xlocH)) THEN
	ylocC = 1
END IF

IF (yLocD>=ny) THEN 
	yLocD = ny-1
END IF

IF (yLocC>=ny) THEN 
	yLocC = ny-1
END IF

ylocA = ylocD+1
ylocB = ylocC+1

xL  = x(xlocL)
xH  = x(xlocH)

yD  = y(ylocD,xlocL)
yC  = y(ylocC,xlocH)
yA  = y(ylocA,xlocL)
yB  = y(ylocB,xlocH)

fLL = abs(f(xlocL,ylocD))**p
fLH = abs(f(xlocL,ylocA))**p
fHL = abs(f(xlocH,ylocC))**p
fHH = abs(f(xlocH,ylocB))**p



xratio = (xi-xL)/(xH-xL)

y1 = (yC-yD)*xratio + yD
y2 = (yB-yA)*xratio + yA
f1 = (fHL-fLL)*xratio + fLL
f2 = (fHH-fLH)*xratio + fLH

fi = (yi-y1)*(f2-f1)/(y2-y1) + f1

IF(fi>0.0) THEN
	fi = fi**(1.0/p)
	if(f(nx,ny)<0.0)fi=-fi
	return
END IF

!if fi<=0.0 then take log transform first
fLL = log(fLL)
fLH = log(fLH)
fHL = log(fHL)
fHH = log(fHH)
f1 = (fHL-fLL)*xratio + fLL
f2 = (fHH-fLH)*xratio + fLH
fi = (yi-y1)*(f2-f1)/(y2-y1) + f1
fi = exp(fi)
fi = fi**(1.0/p)
if(f(nx,ny)<0.0)fi=-fi
return



END SUBROUTINE TransformBiLinInterp1NR

!-----------------------------------------------

SUBROUTINE ExpTransformBiLinInterp1NR (nx,x,ny,y,f,xi,yi,fi)
!this does linear interpolation of f(x,y) at points (xi,yi)
!requires x and y to be sorted in ascending order
!allows for non-rectangular grid, by allowing y grid to be function of x
!extrapolates out of range
!interpolates the transformed function by taking exponential

IMPLICIT NONE
INTEGER, INTENT(in)		:: nx,ny
REAL(long), INTENT(in)	:: x(:),y(:,:),f(:,:),xi,yi
REAL(long), INTENT(out)	:: fi
REAL(long)	            :: xL,xH,yA,yB,yC,yD,fLL,fHH,fLH,fHL,y1,y2,f1,f2,xratio
INTEGER					:: xlocL,xlocH,ylocD, ylocC,ylocA,ylocB

xlocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	xlocL = 1
END IF

IF (xLocL>=nx) THEN 
	xLocL = nx-1
END IF
xlocH = xlocL+1

ylocD = MAXLOC(y(:,xlocL),1,MASK=yi>y(:,xlocL))
ylocC = MAXLOC(y(:,xlocH),1,MASK=yi>y(:,xlocH))


IF (yi<=y(1,xlocL)) THEN
	ylocD = 1
END IF
IF (yi<=y(1,xlocH)) THEN
	ylocC = 1
END IF

IF (yLocD>=ny) THEN 
	yLocD = ny-1
END IF

IF (yLocC>=ny) THEN 
	yLocC = ny-1
END IF

ylocA = ylocD+1
ylocB = ylocC+1

xL  = x(xlocL)
xH  = x(xlocH)

yD  = y(ylocD,xlocL)
yC  = y(ylocC,xlocH)
yA  = y(ylocA,xlocL)
yB  = y(ylocB,xlocH)

fLL = exp(f(xlocL,ylocD))
fLH = exp(f(xlocL,ylocA))
fHL = exp(f(xlocH,ylocC))
fHH = exp(f(xlocH,ylocB))

xratio = (xi-xL)/(xH-xL)

y1 = (yC-yD)*xratio + yD
y2 = (yB-yA)*xratio + yA
f1 = (fHL-fLL)*xratio + fLL
f2 = (fHH-fLH)*xratio + fLH

fi = (yi-y1)*(f2-f1)/(y2-y1) + f1

IF(fi<0.0 .and. y2<yi) THEN
	!yi is in the right range, but the first linear interp took it out of range. use nearest point on boundary
	fi = f2
END IF

IF(fi<0.0 .and. yi<y1) THEN
	!yi is in the right range, but the first linear interp took it out of range. use nearest point on boundary
	fi = f1
END IF

IF(fi<0.0) THEN
	if (Display==1) write(*,*) 'neg fi'
END IF

fi = log(fi)


END SUBROUTINE ExpTransformBiLinInterp1NR
!-----------------------------------------------

SUBROUTINE FindLinProb1 (xi,x,y,p)
!this takes in xi, and finds two points in either side of in in x
!and returns the indices of them y and associated probabilities p
IMPLICIT NONE
REAL(8), INTENT(in)	    :: x(:),xi
REAL(8), INTENT(out)	:: p(2)
INTEGER, INTENT(out)    :: y(2)

INTEGER					:: locL,n

n = size(x)
IF (n==1) THEN
	p(1) = 1.0
	p(2) = 0.0
	y(1) = 1
	y(2) = 1
	return
END IF

LocL = MAXLOC(x,1,MASK=xi>x)

IF (xi<=x(1)) THEN
	y(1) = 1
	y(2) = 2
	p(1) = 1.0
	p(2) = 0.0
ELSE IF (LocL>=n) THEN 
	LocL = n-1
    y(1) = n-1
    y(2) = n
    p(1) = 0.0
    p(2) = 1.0
ELSE IF (x(LocL+1) == x(LocL)) THEN
	y(1) = LocL
    y(2) = LocL+1
	p(1) = 0.5
	p(2) = 0.5
ELSE
    y(1) = LocL
    y(2) = LocL+1
    p(2) = (xi - x(LocL))/real(x(LocL+1)-x(LocL))
    p(1) = 1.0 - p(2)
END IF

END SUBROUTINE FindLinProb1

!----------------------------------------------
SUBROUTINE WriteVector(f,iL,iH,mat)

INTEGER, INTENT(IN)			:: f,iL,iH
REAL(long),INTENT(in)		:: mat(iL:iH)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') 1
lstring = '('//trim(lstring) // 'F16.6)'
DO i1=iL,iH
	WRITE(f,lstring) mat(i1)
END DO

CLOSE(f)

END SUBROUTINE WriteVector
!----------------------------------------------
SUBROUTINE WriteVectorInteger(f,iL,iH,mat)

INTEGER, INTENT(IN)			:: f,iL,iH
INTEGER, INTENT(in)		:: mat(iL:iH)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') 1
lstring = '('//trim(lstring) // 'I16)'
DO i1=iL,iH
	WRITE(f,lstring) mat(i1)
END DO

CLOSE(f)

END SUBROUTINE WriteVectorInteger
!----------------------------------------------
SUBROUTINE WriteMatrix(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'F16.6)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrix
!----------------------------------------------
SUBROUTINE WriteMatrix3(f,n1,n2,n3,mat)

INTEGER, INTENT(IN)			:: f,n1,n2,n3
REAL(long),INTENT(in)		:: mat(n1,n2,n3)
CHARACTER		::lstring*80
INTEGER			::i3,i2

WRITE(UNIT=lstring, FMT='(I5)') n1
lstring = '('//trim(lstring) // 'F16.6)'
DO i3=1,n3
    DO i2=1,n2
    	WRITE(f,lstring) (mat(:,i2,i3))
    END DO
END DO

CLOSE(f)

END SUBROUTINE WriteMatrix3
!----------------------------------------------
SUBROUTINE WriteMatrixLH(f,iL,iH,n2,mat)
!same as write matrix but allows the indices of the first dimension to be different from 1:n1
INTEGER, INTENT(IN)			:: f,n2,iL,iH
REAL(long),INTENT(in)		:: mat(iL:iH,1:n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'F16.6)'
DO i1=iL,iH
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixLH
!----------------------------------------------
SUBROUTINE WriteMatrixLong(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'F20.14)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixLong
!----------------------------------------------
SUBROUTINE WriteMatrixExpon(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'E16.8)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixExpon
!----------------------------------------------
SUBROUTINE WriteMatrixCSVExpon(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(E16.8,","),E16.8)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSVExpon
!----------------------------------------------
SUBROUTINE WriteMatrixInteger(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
INTEGER,INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2
lstring = '('//trim(lstring) // 'I16)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixInteger
!----------------------------------------------
SUBROUTINE WriteMatrixCSV(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
REAL(long),INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(F16.6,","),F16.6)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSV
!----------------------------------------------
SUBROUTINE WriteMatrixCSVInteger(f,n1,n2,mat)

INTEGER, INTENT(IN)			:: f,n1,n2
INTEGER,INTENT(in)		:: mat(n1,n2)
CHARACTER		::lstring*80
INTEGER			::i1

WRITE(UNIT=lstring, FMT='(I5)') n2-1
lstring = '('//trim(lstring) // '(I16,","),I16)'
DO i1=1,n1
	WRITE(f,lstring) (mat(i1,:))
END DO

CLOSE(f)

END SUBROUTINE WriteMatrixCSVInteger
!----------------------------------------------
SUBROUTINE RandomDiscrete(Nout,Xout,Nin,Pin)
!generates Nout random draws from the integers 1 to Nin
!using probabilities in Pin
IMPLICIT NONE
INTEGER, INTENT(in)			:: Nout,Nin
INTEGER,INTENT(out)		:: Xout(:)
REAL(long), INTENT(in)  :: Pin(:)

INTEGER			::i1,i2
REAL(long)      :: lran(Nout)

!IF(sum(Pin) .ne. 1.0) write(*,*) 'error in RandomDiscrete: Pin doesnt sum to 1.0'

CALL RANDOM_NUMBER(lran)

Xout(:) = 0
DO i1 = 1,Nout
    IF ( lran(i1) .le. Pin(1) ) THEN
        Xout(i1) = 1
    ELSE
        i2 = 2
        DO WHILE (i2 .le. Nin)
            IF ( (lran(i1) .le. SUM(Pin(1:i2)) ).and. (lran(i1) > SUM(Pin(1:i2-1)) ) ) THEN
                Xout(i1) = i2
                i2 = Nin+1
            ELSE
                i2 = i2+1
            END IF
        END DO
    END IF
END DO

END SUBROUTINE RandomDiscrete

!--------------------------------------------------------------
SUBROUTINE RandomDiscrete1(Xout,Nin,Pin)
!generates Nout random draws from the integers 1 to Nin
!using probabilities in Pin
IMPLICIT NONE
INTEGER, INTENT(in)			:: Nin
INTEGER,INTENT(out)		:: Xout
REAL(long), INTENT(in)  ::Pin(:)

INTEGER			::i2
REAL(long)      :: lran

!IF(sum(Pin) .ne. 1.0) write(*,*) 'error in RandomDiscrete: Pin doesnt sum to 1.0'

CALL RANDOM_NUMBER(lran)

Xout = 0
IF ( lran .le. Pin(1) ) THEN
    Xout = 1
ELSE
    i2 = 2
    DO WHILE (i2 .le. Nin)
        IF ( (lran .le. SUM(Pin(1:i2)) ).and. (lran > SUM(Pin(1:i2-1)) ) ) THEN
            Xout = i2
            i2 = Nin+1
        ELSE
            i2 = i2+1
        END IF
    END DO
END IF


END SUBROUTINE RandomDiscrete1

!----------------------------------------------
SUBROUTINE DiscreteDist(Nout,Xout,Nin,Pin,lran)
!takes in random numbers and a prob dist over the integers 1 to Nin
!and returns the corresponding values
!Created 11/22/09 - ERS

INTEGER,    INTENT(out) :: Xout(:)
INTEGER,    INTENT(in)  :: Nin,Nout
REAL(8),    INTENT(in)  :: Pin(:)
REAL(long), INTENT(in)  :: lran(:)
INTEGER			        :: i1,i2

Xout(:) = 0
DO i1 = 1,Nout
    IF ( lran(i1) .le. Pin(1) ) THEN
        Xout(i1) = 1
    ELSE
        i2 = 2
        DO WHILE (i2 .le. Nin)
            IF ( (lran(i1) .le. SUM(Pin(1:i2)) ).and. (lran(i1) > SUM(Pin(1:i2-1)) ) ) THEN
                Xout(i1) = i2
                i2 = Nin+1
            ELSE
                i2 = i2+1
            END IF
        END DO
    END IF
END DO

END SUBROUTINE DiscreteDist

!-----------------------------------------------
SUBROUTINE DiscreteDist1(Xout,Nin,Pin,lran)
!takes in a random number and a prob dist over the integers 1 to Nin
!and returns the corresponding value

INTEGER, INTENT(in)     :: Nin
INTEGER,INTENT(out)     :: Xout
REAL(8), INTENT(in)     :: Pin(:),lran
INTEGER                 :: i2

Xout = 0
IF ( lran .le. Pin(1) ) THEN
   Xout = 1
ELSE
   i2 = 2
   DO WHILE (i2 .le. Nin)
       IF ( (lran .le. SUM(Pin(1:i2)) ).and. (lran > SUM(Pin(1:i2-1)) ) ) THEN
           Xout = i2
           i2 = Nin+1
       ELSE
           i2 = i2+1
       END IF
   END DO
END IF

END SUBROUTINE DiscreteDist1

!-----------------------------------------------
!--------------------------------------------------------------
RECURSIVE SUBROUTINE quick_sort(list, order)

! Quick sort routine from:
! Brainerd, W.S., Goldberg, C.H. & Adams, J.C. (1990) "Programmer's Guide to
! Fortran 90", McGraw-Hill  ISBN 0-07-000248-7, pages 149-150.
! Modified by Alan Miller to include an associated integer array which gives
! the positions of the elements in the original order.

IMPLICIT NONE
REAL(long), DIMENSION (:), INTENT(IN OUT)  :: list
INTEGER, DIMENSION (:), INTENT(OUT)  :: order

! Local variable
INTEGER :: i

DO i = 1, SIZE(list)
  order(i) = i
END DO

CALL quick_sort_1(1, SIZE(list))

CONTAINS

RECURSIVE SUBROUTINE quick_sort_1(left_end, right_end)

INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL(long)                :: reference, temp
INTEGER, PARAMETER  :: max_simple_sort_size = 6

IF (right_end < left_end + max_simple_sort_size) THEN
  ! Use interchange sort for small lists
  CALL interchange_sort(left_end, right_end)

ELSE
  ! Use partition ("quick") sort
  reference = list((left_end + right_end)/2)
  i = left_end - 1; j = right_end + 1

  DO
    ! Scan list from left end until element >= reference is found
    DO
      i = i + 1
      IF (list(i) >= reference) EXIT
    END DO
    ! Scan list from right end until element <= reference is found
    DO
      j = j - 1
      IF (list(j) <= reference) EXIT
    END DO


    IF (i < j) THEN
      ! Swap two out-of-order elements
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    ELSE IF (i == j) THEN
      i = i + 1
      EXIT
    ELSE
      EXIT
    END IF
  END DO

  IF (left_end < j) CALL quick_sort_1(left_end, j)
  IF (i < right_end) CALL quick_sort_1(i, right_end)
END IF

END SUBROUTINE quick_sort_1


SUBROUTINE interchange_sort(left_end, right_end)
IMPLICIT NONE
INTEGER, INTENT(IN) :: left_end, right_end

!     Local variables
INTEGER             :: i, j, itemp
REAL                :: temp

DO i = left_end, right_end - 1
  DO j = i+1, right_end
    IF (list(i) > list(j)) THEN
      temp = list(i); list(i) = list(j); list(j) = temp
      itemp = order(i); order(i) = order(j); order(j) = itemp
    END IF
  END DO
END DO

END SUBROUTINE interchange_sort

END SUBROUTINE quick_sort
!-----------------------------------------------------------------------
SUBROUTINE Percentile(n,x,p,q)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: x(n),p
REAL(long), INTENT(out)	:: q
REAL(long)				:: lx(n),li,lx1,lx2
INTEGER					:: lxord(n),i1,i2

lx = x
li = (n+1)*p
i1 = dint(li)
i2 = i1+1
CALL quick_sort(lx,lxord)
lx1 = lx(i1)
lx2 = lx(i2)

q  = lx1  + (li -real(i1) )*((lx2 -lx1 )/real(i2 -i1 ))

END SUBROUTINE Percentile

!-----------------------------------------------------------------------
SUBROUTINE Rouwenhorst(n,p,q,R)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(long), INTENT(in)	:: p,q
REAL(long), INTENT(out)	:: R(n,n)
REAL(long)				:: X(n,n), Y1(n,n), Y2(n,n), Y3(n,n), Y4(n,n)
INTEGER					:: j,i

X(1,1) = p
X(1,2) = 1-p
X(2,1) = 1-q
X(2,2) = q

IF(n>2) THEN
DO j = 2,n-1
	Y1(:,:) = 0.0
	Y2(:,:) = 0.0
	Y3(:,:) = 0.0
	Y4(:,:) = 0.0
	Y1(1:j,1:j) = X(1:j,1:j)
	Y2(1:j,2:j+1) = X(1:j,1:j)
	Y3(2:j+1,1:j) = X(1:j,1:j)
	Y4(2:j+1,2:j+1) = X(1:j,1:j)
	X = p*Y1 + (1-p)*Y2 + (1-q)*Y3 + q*Y4
	DO i = 2,j
		X(i,:) = X(i,:)/2.0
	END DO
END DO

END IF
R = X

END SUBROUTINE Rouwenhorst
!-----------------------------------------------------------------------
SUBROUTINE OLS(n,k,x,y,beta)

!x is n x k
!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: x(n,k),y(n)
REAL(8), INTENT(out)	:: beta(k)
REAL(8)					:: lxx(k,k),lxxinv(k,k)
INTEGER					:: ierr

lxx = matmul(transpose(x),x)
CALL InvertMatrix(lxx, lxxinv, k, ierr)
beta = matmul(lxxinv,matmul(transpose(x),y))

END SUBROUTINE OLS

!-----------------------------------------------------------------------
SUBROUTINE IVREG(n,k,m,x,y,z,beta)

!x is n x k
!z is n x m, m>=k
!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k,m
REAL(8), INTENT(in)		:: x(n,k),y(n),z(n,m)
REAL(8), INTENT(out)	:: beta(k)
!REAL(8)					:: lzz(m,m),lzzinv(m,m),
REAL(8)					:: lzx(k,k),lzxinv(k,k)
!REAL(8)					:: lPz(n,n),lPzx(n,k),lxPzx(k,k),lxPzxinv(k,k),lPzy(n)
INTEGER					:: ierr


IF(m==k) THEN
	lzx = matmul(transpose(z),x)
	CALL InvertMatrix(lzx, lzxinv, k, ierr)
	beta = matmul(lzxinv,matmul(transpose(z),y))
	
!ELSE IF(m>k) THEN

!	lzz = matmul(transpose(z),z)
!	CALL InvertMatrix(lzz, lzzinv, m, ierr)
!	lPz = matmul(matmul(z,lzzinv),transpose(z))
!	lPzx = matmul(lPz,x)
!	lxPzx = matmul(transpose(x),lPzx)
!	CALL InvertMatrix(lxPzx, lxPzxinv, k, ierr)
!	lPzy = matmul(lPz,y)
!	beta = matmul(lxPzxinv,matmul(transpose(x),lPzy))

ELSE
	write(*,*) 'IVREG requires m>=k'
	return
END IF


END SUBROUTINE IVREG

!-----------------------------------------------------------------------
SUBROUTINE OLSglobal(n,k,y,beta)
!same as OLS but the X variables are stored in a global n x k array, X_reg

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(k)
REAL(8)					:: lxx(k,k),lxxinv(k,k)
INTEGER					:: ierr

lxx = matmul(transpose(X_reg),X_reg)
CALL InvertMatrix(lxx, lxxinv, k, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))

END SUBROUTINE OLSglobal

!-----------------------------------------------------------------------
SUBROUTINE OLSGlob2(n,k,y,beta)
!same as OLSGlobal but the X variables are stored in a global n x k array, X1

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(k)
REAL(8)					:: lxx(k,k),lxxinv(k,k)
INTEGER					:: ierr

lxx = matmul(transpose(X_reg),X_reg)
lxx(k,k)=1.0d0
CALL InvertMatrix(lxx, lxxinv, k, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))

END SUBROUTINE OLSGlob2




!-----------------------------------------------------------------------
SUBROUTINE OLSGlob3(n,y,beta)
!same as OLSGlobal but the X variables are stored in a global n x k array, X1

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(5)
REAL(8)					:: lxx(5,5),lxxinv(5,5)
INTEGER					:: ierr

X_reg(:,4)=X_reg(:,5)
X_reg(:,5)=0.0d0
lxx = matmul(transpose(X_reg),X_reg)
lxx(5,5)=1.0d0
CALL InvertMatrix(lxx, lxxinv, 5, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))
X_reg(:,5)=X_reg(:,4)

END SUBROUTINE OLSGlob3


!-----------------------------------------------------------------------
SUBROUTINE OLSGlob3_2(n,y,beta)
!same as OLSGlobal but the X variables are stored in a global n x k array, X1

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(5)
REAL(8)					:: lxx(5,5),lxxinv(5,5)
INTEGER					:: ierr

X_reg(:,4)=0.0d0
X_reg(:,5)=0.0d0
lxx = matmul(transpose(X_reg),X_reg)
lxx(4,4)=1.0d0
lxx(5,5)=1.0d0
CALL InvertMatrix(lxx, lxxinv, 5, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))

END SUBROUTINE OLSGlob3_2


!-----------------------------------------------------------------------

SUBROUTINE OLSGlob4(n,k,y,beta)
!same as OLSGlobal but the X variables are stored in a global n x k array, X1

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(5)
REAL(8)					:: lxx(5,5),lxxinv(5,5),lk
INTEGER					:: ierr

lk = k
X_reg(:,2)=X_reg(:,5)
X_reg(:,5)=0.0d0
lxx = matmul(transpose(X_reg),X_reg)
lxx(5,5)=1.0d0
CALL InvertMatrix(lxx, lxxinv, 5, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))
X_reg(:,5)=X_reg(:,2)

END SUBROUTINE OLSGlob4
!-----------------------------------------------------------------------

SUBROUTINE OLSGlob4_2(n,y,beta)
!same as OLSGlobal but the X variables are stored in a global n x k array, X1

!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(5)
REAL(8)					:: lxx(5,5),lxxinv(5,5)
INTEGER					:: ierr

X_reg(:,2)=X_reg(:,3)
X_reg(:,3)=X_reg(:,4)
X_reg(:,4)=0.0d0
lxx = matmul(transpose(X_reg),X_reg)
lxx(5,5)=1.0d0
lxx(4,4)=1.0d0
CALL InvertMatrix(lxx, lxxinv, 5, ierr)
beta = matmul(lxxinv,matmul(transpose(X_reg),y))
X_reg(:,4)=X_reg(:,3)
X_reg(:,3)=X_reg(:,2)

END SUBROUTINE OLSGlob4_2



!-----------------------------------------------------------------------
SUBROUTINE IVREGglobal(n,k,y,beta)
!same as IVREG but the X variables are stored in a global n x k array, X_reg
!same as IVREG but the Z variables are stored in a global n x k array, Z_reg
!can only handle k = m

!x is n x k
!z is n x m, m>=k
!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(k)
REAL(8)					:: lzx(k,k),lzxinv(k,k)
INTEGER					:: ierr


lzx = matmul(transpose(Z_reg),X_reg)
CALL InvertMatrix(lzx, lzxinv, k, ierr)
beta = matmul(lzxinv,matmul(transpose(Z_reg),y))
	


END SUBROUTINE IVREGglobal

!-----------------------------------------------
!-----------------------------------------------------------------------
SUBROUTINE IVREGGlob2(n,k,y,beta)
!same as IVREG but the X variables are stored in a global n x k array, X_reg
!same as IVREG but the Z variables are stored in a global n x k array, Z_reg
!can only handle k = m

!x is n x k
!z is n x m, m>=k
!y is n x 1
!beta is k x 1
IMPLICIT NONE
INTEGER, INTENT(in)		:: n,k
REAL(8), INTENT(in)		:: y(n)
REAL(8), INTENT(out)	:: beta(k)
REAL(8)					:: lzx(k,k),lzxinv(k,k)
INTEGER					:: ierr


lzx = matmul(transpose(Z_reg),X_reg)
lzx(k,k)=1
CALL InvertMatrix(lzx, lzxinv, k, ierr)
beta = matmul(lzxinv,matmul(transpose(Z_reg),y))
	


END SUBROUTINE IVREGGlob2

!-----------------------------------------------





SUBROUTINE FindLeftCross (n1,x1,y1,n2,x2,y2,xout)
!for two vectors with y1(1)>y2(1), it finds first point x where y2(x)>=x2(x)
IMPLICIT NONE
INTEGER, INTENT(in)		:: n1,n2
REAL(8), INTENT(in)	:: x1(n1),y1(n1),x2(n2),y2(n2)
REAL(8), INTENT(out)	:: xout
REAL(8)	            :: y2atx1(n1), y1atx2(n2),x(n1+n2),ydiff(n1+n2),ydiffsort(n1+n2),xL,xH,yL,yH
INTEGER					:: i,order(n1+n2)

!first check for no left cross point
! IF (y1(1)<=y2(1)) THEN
! 	write(*,*) 'err in FindLeftCross: y2>y1 at start'
! 	return
! END IF

!THERE MUST BE A MORE EFFICIENT WAY!
!interpolate y2 at each point in n1 to find where cross
CALL LinInterp (n2,x2,y2,n1,x1,y2atx1)
!interpolate y1 at each point in n2 to find where cross
CALL LinInterp (n1,x1,y1,n2,x2,y1atx2)

!combines the two vectors
DO i = 1,n1+n2
	x(1:n1) = x1
	ydiff(1:n1) = y1-y2atx1
	x(n1+1:n1+n2) = x2
	ydiff(n1+1:n1+n2) = y1atx2-y2
END DO

CALL quick_sort(x, order)
DO i =1,n1+n2
	ydiffsort(i) = ydiff(order(i))
END DO

DO i = 1,n1+n2
	IF (ydiffsort(i)<=0.0) GOTO 10
END DO
IF(i == n1+n2+1) THEN
	!write(*,*)'problem in FindLeftCross: no crossing'
	!for all reasonable cash levels adjusting is better, perhaps if assets are really high
	xout = 999.99	
	return
END IF

10 xH = ydiffsort(i)
yH = x(i)

IF(i>1) THEN
	xL = ydiffsort(i-1)
	yL = x(i-1)
ELSE
	write(*,*)'prob in FindLeftCross: ydiff<=0 at first point'
	!need to extrapolate
	xL = ydiffsort(1)
	yL = x(1)
	
	xH = ydiffsort(2)
	yH = x(2)
	!xout = -100.0
	
	!return
END IF

xout  = yL   -xL *((yH -yL )/(xH -xL ))

END SUBROUTINE FindLeftCross

!-----------------------------------------------

SUBROUTINE PowerSpacedGrid (n,k,low,high,y)

!gives a grid spaced between low and high based on the unit interval with a function x^(1/k)
!k = 1 is linear, k = 0 is L-shaped

IMPLICIT NONE

INTEGER, INTENT(in)	:: n
REAL(8), INTENT(in)	:: k,low,high
REAL(8), INTENT(out) :: y(:)
INTEGER				:: i
REAL(8)				:: x(n),z(n)

IF(n<2) THEN
	write(*,*) 'n must be at least 2 to make grids'
	return
END IF

IF (n==2) THEN
	y(1) = low
	y(2) = high
	return
END IF

x(1) = 0.0
x(n) = 1.0
DO i = 2,n-1
	x(i) = (i-1)/real(n-1)
END DO

z = x**(1.0/k)

y = low + (high-low)*z


END SUBROUTINE PowerSpacedGrid










END MODULE Procedures
