SUBROUTINE RebateRegressions

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER		:: i,in,it,ij,startq,endq,itoffset,n_obs_reb,n_k_reb
REAL(8), DIMENSION(4+allquarters)		:: lregcoef
real(8)     :: percval(2)
CHARACTER	:: lstring*80


!use quarters 0,1,2 post announcement as t+1
!that way we cover all possible periods in which rebates could be receieved

write(*,*) 'Constructing matrices for regressions'

IF (allquarters==1) THEN
    startq=0
    endq=2
    itoffset=1
    n_obs_reb=3*(nsimrebateA+nsimrebateB)
    n_k_reb=5
ELSE
    startq=(3-announcequ)
    endq=1+startq
    itoffset=1-startq
    n_obs_reb=2*(nsimrebateA+nsimrebateB)
    n_k_reb=4

ENDIF

!set up regression matrices but only use thos receieving rebate
!$OMP PARALLEL DO PRIVATE(it,i)
DO in = 1,nsimrebateA+nsimrebateB
	DO it = startq,endq
		i = (it-1+itoffset)*(nsimrebateA+nsimrebateB) + in
		!non-durable consumption
		dCtp1(i) =  simtrans(it)%csim(in) - simtrans(it-1)%csim(in)
		dlogCtp1(i) = log(simtrans(it)%csim(in)) - log(simtrans(it-1)%csim(in))
		

		!quarter dummies
		quarter_reg(i,:) = 0.0
		quarter_reg(i,it+itoffset) = 1.0
		
		!age
		age_reg(i)	=simtrans(it)%jsim(in)

		! liquid wealth
		liq_reg(i) = simtrans(it)%lsim(in)
		
		! earnings
		inc_reg(i)  =simtrans(it)%ysim(in)
		
		if(age_reg(i) <=Jwork) then
    		lpl_reg(i) = liq_reg(i)+ 0.5*(inc_reg(i) - simtrans(it)%taxsim(in))
        else
            lpl_reg(i) = liq_reg(i)+ 0.5*(simtrans(it)%psim(in) - simtrans(it)%taxsim(in))
		endif
		
		
		!rebate
		rebamt_reg(i) = simtrans(it)%rebsim(in)
		IF(rebamt_reg(i)>0.0) THEN
			rebind_reg(i) = 1.0
		ELSE
			rebind_Reg(i) = 0.0
		END IF
				
	END DO
END DO 
!$OMP END PARALLEL DO

IF(RebateRegressionsInStata==1) THEN

	OPEN(3, FILE = trim(OutputDir) // 'dCtp1_reg.txt', STATUS = 'replace'); CALL WriteMatrixLong(3,n_obs_reb,1,dCtp1)
	OPEN(3, FILE = trim(OutputDir) // 'dlogCtp1_reg.txt', STATUS = 'replace'); CALL WriteMatrixLong(3,n_obs_reb,1,dlogCtp1)
	OPEN(3, FILE = trim(OutputDir) // 'age_reg.txt', STATUS = 'replace'); CALL WriteMatrixInteger(3,n_obs_reb,1,INT(age_reg))
	OPEN(3, FILE = trim(OutputDir) // 'liq_reg.txt', STATUS = 'replace'); CALL WriteMatrix(3,n_obs_reb,1,liq_reg)
	OPEN(3, FILE = trim(OutputDir) // 'inc_reg.txt', STATUS = 'replace'); CALL WriteMatrix(3,n_obs_reb,1,inc_reg)
	OPEN(3, FILE = trim(OutputDir) // 'lpl_reg.txt', STATUS = 'replace'); CALL WriteMatrix(3,n_obs_reb,1,lpl_reg)
	OPEN(3, FILE = trim(OutputDir) // 'rebamt_reg.txt', STATUS = 'replace'); CALL WriteMatrix(3,n_obs_reb,1,rebamt_reg)
	DO it = 1, 2+allquarters
		WRITE(UNIT=lstring, FMT='(I1)') it
		OPEN(3, FILE = trim(OutputDir) // 'quarter'// trim(lstring) //'_reg.txt', STATUS = 'replace'); CALL WriteMatrixInteger(3,n_obs_reb,1,INT(quarter_reg(:,it)))
	END DO

ELSE
	WRITE(*,*) 'Please set RebateRegressionsInStata = 1 in Parameters.f90'
END IF



END SUBROUTINE RebateRegressions
