SUBROUTINE SetParameters

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

real(8)::rhopow,blimfrac,r_real_free_annual,junk
integer::rebc

!TARGET CAPITAL INCOME RATIO

ALLOCATE(initliq(ngpzy+ngpfylev-1))
ALLOCATE(initill(ngpzy+ngpfylev-1))
ALLOCATE(initnw(ngpzy+ngpfylev-1))
ALLOCATE(initliqdist(ngpzy+ngpfylev-1,2))
ALLOCATE(initilldist(ngpzy+ngpfylev-1,2))
ALLOCATE(initnwdist(ngpzy+ngpfylev-1,2))


InfiniteFixedCostModel = 0
targetUsualKY 			=  12.0
targetMeanIlliquid 		= 200000.0d0/(DataAvAnnualEarns/real(Numeraire))	
targetMedianIlliquid 	= 54600.0d0/(DataAvAnnualEarns/real(Numeraire))	
targetP75Illiquid 		= 235000.0d0/(DataAvAnnualEarns/real(Numeraire))

rebatepct = 0.5d0

IF(InfiniteFixedCostModel==1) targetMedianIlliquid 	= 62440.0d0/(DataAvAnnualEarns/real(Numeraire))	

!EARNINGS PROCESS
SELECT CASE (WhichStochModel)

	CASE(-1) !Uemp only 
		Vepsy 	= 0.0d0
		Vfey	= 0.0d0 !var fix effect
		Vzy0	= 0.0d0 !initial var persistent
		Vetay  	= 0.0d0 !0.0 !var pers shocks
		rhoy  	= 0.0d0 !0.99d0 !0.99  
		lambday = 0.0	!probability of switch at quarterly level: only active if StaySwitchModelForzy=1
		rholevslo = 0.0d0
		Vphy	= 0.0d0

	CASE(0) 
		Vepsy 	= 0.0
		Vfey	= 0.2723 !var fix effect
		Vzy0	= 0.0d0 !initial var persistent
		Vetay  	= 0.0d0 !0.0 !var pers shocks
		rhoy  	= 0.999d0 !0.99d0 !0.99  
		lambday = 0.0	!probability of switch at quarterly level: only active if StaySwitchModelForzy=1
		rholevslo = 0.0d0
		Vphy	= 0.00002199574


	CASE(1) !AR only
		Vepsy   = 0.0d0 !(0.005d0)**2
		Vfey    = 0.0
		Vzy0    = 0.18d0
		rhoy	= 1.0d0
		lambday = 1.0d0/1.0d0   !probability of switch at quarterly level: only active if StaySwitchModelForzy=1
		IF( rhoy .LT. 1.0d0 ) THEN
			rhopow  = rhoy**(2.0d0*(real(Jwork)-1.0d0))
            Vetay   = (Jwork*0.003d0+(1.0d0-rhoy**(2.0d0*real(Jwork)))*Vzy0)*(1.0d0-rhoy**(2.0d0))/(1.0d0-rhopow)
		ELSE
			Vetay   = 0.003d0
		ENDIF

		rholevslo = 0.0d0
		Vphy	= 0.0d0

	CASE(2) !FE + AR(1) 
		Vepsy 	= 0.0
		Vfey	= 0.1105
		Vzy0	= 0.0 !initial var persistent
		Vetay  	= 0.00959 !var pers shocks
		rhoy  	= 0.99d0
		lambday = 0.0	!probability of switch at quarterly level: only active if StaySwitchModelForzy=1

	CASE(3) !FE + AR(1) + IID
		Vepsy 	= 0.34
		Vfey	= 0.11
		Vzy0	= 0.0 !initial var persistent
		Vetay  	= 0.01 !var pers shocks
		rhoy  	= 0.9902 
		lambday = 0.0	!probability of switch at quarterly level: only active if StaySwitchModelForzy=1

	
END SELECT
    

!UNEMPLOYMENT
initialunemp =  0.15 !initial unemp rate
uemprepl = 0.5	!replacement rate of last earnings when unemployed: set to 1 if no unemp shocks so that borrowlims are correct
uemplosspers = 0.10 ! persistent earnings loss upon re-employment
uemplosstrans = 0.0 ! transitory earnings loss upon re-employment: to achieve total earnings loss on impact of 15%
IF(NoUnempShocks==1) initialunemp = 0.0
IF(NoUnempShocks==1) uemplosspers = 0.0
IF(NoUnempShocks==1) uemplosstrans = 0.0
IF(NoUnempShocks==1) uemprepl = 1.0

!REBATE STUFF
pcrebatespend = 0.0		!when run with both rebate surprises, this forces them to spend this fraction in the period they get it.
allquarters = 1
rebtype = 0

!PREFERENCE PARMETERS
gamC 	= 4.0 
sigC    = 0.6667 
servflow = 0.01 
ndweight = 0.85
liqflow = 0.0

!DISCOUNT FACTOR (QUARTERLY)
bet = 0.986

!BORROWING LIMIT: SET TO VERY LARGE NO. FOR NBL
sameborrowlim = 0 !All agents have same borrowing limit
blimfrac = 0.18d0
borrowlim = real(Borrowing)*blimfrac/4.0  !multiple of persistent component of quarterly earnings (pos no. here) 
fixcost = 1000.0/(DataAvAnnualEarns/real(Numeraire)) !as frac of average quarterly earnings
varcost = 0.0	!as percentage of actual quarterly earnings
utilcost = 0.0	!utility cost: + log(1-utilcost), so with log utility is interpreted as fraction of consumption

!INITIAL WEALTH
posinitwealth = 1
sameinitwealth = 0

!RETURNS
nomretliq = 0.033	!annual nominal return on liquid assets (normalization)
inflation = 0.04	!anuual inflation rate
nomretill = 0.0684	!annual nominal return on illiquuid assets
nomborrow = 0.15	!annual nominal interest rate on borrowing

!GOVERNMENT PARAMETERS
 ctax = 0.072
liqtax = 0.2319	!tax rate on nominal liquid returns
illtax = 0.0786	!tax rate on nominal illiquid returns
taxurcg = 0.0	!fraction of tax on illiquid returns paid out of liquid wealth.

!GOVT BUDGET CONSTRAINT
nomretgov = 0.05
qgov = ((1.0+inflation)/(1.0+nomretgov)) ** 0.25

!RECESSION PARAMETERS
recdepth = 0.03	!total percentage decline
recpclim = 1.0 	!concentrate recession below this percentile, set to 1.0 for uniform

!REAL BOND PRICES (QUARTERLY)
qfree = ((1.0+inflation)/(1.0+nomretliq*(1.0-liqtax))) ** 0.25
qill = ((1.0+inflation)/(1.0+nomretill*(1.0-illtax*(1.0-taxurcg)))) ** 0.25
qborr = ((1.0+inflation)/(1.0+nomborrow)) ** 0.25
mturcg = illtax*taxurcg*((1.0+nomretill)**0.25 - 1.0) / ((1.0+nomretill*(1.0-illtax*(1.0-taxurcg))) ** 0.25)
r_real_free_annual = (1.0+nomretliq*(1.0-liqtax))/(1.0+inflation)-1.0d0
print*,'Real liq int rate: ',r_real_free_annual
print*,'Real ill int rate: ',(1.0+nomretill*(1.0-illtax))/(1.0+inflation)-1.0d0
bondparam = 0.0	!higher numbers mean interest rate increases quicker
maxdeduction = 0.5 !fraction of earnings (no deductions during unemployment)
maxdedret	= 0.05 !fraction of illiquid assets to be withdrawn every quarter 



!READ IN PARAMETERS FROM TEXT FILE
IF (ReadParameters==1) THEN
	OPEN(1, FILE = InputDir // trim(InputParamFile))
	READ(1,*) OutputDir
	READ(1,*) OutputDirSims
	READ(1,*) bet
	READ(1,*) gamC
	READ(1,*) sigC
	READ(1,*) fixcost
	READ(1,*) varcost
	READ(1,*) inflation
	READ(1,*) nomretill
	READ(1,*) nomborrow
	READ(1,*) taxurcg	
	READ(1,*) targetUsualKY
	READ(1,*) maxdeduction
	READ(1,*) maxdedret
	READ(1,*) rebateamt
	READ(1,*) utilcost
	READ(1,*) recpclim
	READ(1,*) servflow
	READ(1,*) ndweight
	READ(1,*) rebatepct
	READ(1,*) recdepth
	CLOSE(1)

	nomretgov = 0.05 !Nominaral borrowing rate for government
	
	qfree = ((1.0+inflation)/(1.0+nomretliq*(1.0-liqtax))) ** 0.25
	qill = ((1.0+inflation)/(1.0+nomretill*(1.0-illtax*(1.0-taxurcg)))) ** 0.25
	qborr = ((1.0+inflation)/(1.0+nomborrow)) ** 0.25
	qgov = ((1.0+inflation)/(1.0+nomretgov)) ** 0.25

        
	r_real_free_annual = (1.0+nomretliq*(1.0-liqtax))/(1.0+inflation)-1.0d0
	print*,'Real liq int rate: ',r_real_free_annual
	print*,'Real ill int rate: ',(1.0+nomretill*(1.0-illtax))/(1.0+inflation)-1.0d0

	IF(sameborrowlim) THEN	
		borrowlim = blimfrac
	ELSE
		borrowlim = blimfrac*real(Borrowing)*4.0d0*0.18d0/0.25d0
	ENDIF



	IF (InfiniteFixedCostModel==1 .or. fixcost<0.0) THEN
		InfiniteFixedCostModel=1
		write(*,*) 'Infinite Fixed Cost Model'
		fixcost = 10000000000.0
		nomretliq = 0.064
		liqtax = 0.17
		qfree = ((1.0+inflation)/(1.0+nomretliq*(1.0-liqtax))) ** 0.25
		qill = 1.0
		r_real_free_annual=1.0d0/((1.0+inflation)/(1.0+nomretliq*(1.0-liqtax)))-1.0d0
		qborr = ((1.0+inflation)/(1.0+nomborrow)) ** 0.25
        print*,'Rate inf fixed cost: ',r_real_free_annual
		targetMedianIlliquid 	= 62440.0d0/(DataAvAnnualEarns/real(Numeraire))	
		
	ELSE IF(fixcost==0.0) THEN
		nomretill=0.064
		illtax=0.17d
		qill = ((1.0+inflation)/(1.0+nomretill*(1.0-illtax))) ** 0.25
		qfree = 1.0d0
		qborr = ((1.0+inflation)/(1.0+nomborrow)) ** 0.25
	END IF
	
	mturcg = illtax*taxurcg*((1.0+nomretill)**0.25 - 1.0) / ((1.0+nomretill*(1.0-illtax*(1.0-taxurcg))) ** 0.25)
	fixcost = fixcost/(DataAvAnnualEarns/real(Numeraire))


ELSE
	OutputDir 		= ParamOutputDir
	OutputDirSims 	= ParamOutputDirSims
END IF

ALLOCATE(rebgrid(1),rebdist(1))
rebgrid = rebateamt; rebdist=1.0d0
nrebgrid=1
rebateamt = rebateamt/(DataAvAnnualEarns/real(Numeraire))
rebgrid = rebgrid/(DataAvAnnualEarns/real(Numeraire))



END SUBROUTINE SetParameters
