SUBROUTINE SimulateTaxesTransition

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER		:: iseed(2),isize

INTEGER		:: in,ij,ijret,it,iyr,iqu,ip,ify,izy,iznext(2),iu,ltaxsys,insim,iW,iR,tempdraw
REAL(8)	    :: lliqmin,lsavmin,lpynext(2)
REAL(8), EXTERNAL	:: FnTax, FnBondPrice, FnInvQA

IF(Display==1) write(*,*) 'Simulating taxes transition'

!set random number seed
iseed(1) = 5432
iseed(2) = 6543
isize = 2
CALL RANDOM_SEED(size = isize)
CALL RANDOM_SEED(put = iseed)   

BushDebt=0.0d0

IF(SameRandomNumbersRebate == 0) THEN
	insim = nsimtrans
ELSE
	insim = nsimrebateA
END IF


!fill in steady state simulation, last period before announcement, by randomly drawing ids and assigning ages per poulation
IF(DeterministicModel==0) THEN
	DO in = 1,insim
		idsimI(in) = mod(in,nsim)+1
		CALL RandomDiscrete1(simtrans(-1)%jsim(in), Jtot, popdist)
	END DO
ELSE
	DO in = 1,insim
		idsimI(in) = 1	!deterministic model so only 1 steady state simulation
		simtrans(-1)%jsim(in) = in
	END DO
END IF

IF(SameRandomNumbersRebate == 1) THEN

	idsimI(nsimrebateA+1:2*nsimrebateA)=idsimI(1:nsimrebateA)
	idsimI(2*nsimrebateA+1:nsimtrans)=idsimI(1:nsimrebateA)

END IF



!draw all random numbers so can do in parallel
write(*,*) 'drawing random numbers for simulations'
CALL RANDOM_NUMBER(dierand)
CALL RANDOM_NUMBER(uerand)
CALL RANDOM_NUMBER(zyrand)
CALL RANDOM_NUMBER(eyrand)
CALL RANDOM_NUMBER(initliqrand)
CALL RANDOM_NUMBER(initillrand)

!$OMP PARALLEL DO PRIVATE(it)
DO in = 1,insim 

	!fixed state variables
	fysimtransI(in) = fysimI(idsimI(in))

	!state variables at beginning of transition
	simtrans(-1)%asim(in) = asim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%lsim(in) = lsim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%psim(in) = psim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%psimI(in) = psimI(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%dedsimI(in) = dedsimI(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%adjsim(in) = adjsim(idsimI(in),simtrans(-1)%jsim(in))
	
	
	IF(simtrans(-1)%jsim(in) <=Jwork) THEN
		simtrans(-1)%ysim(in) = ysim(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%ypssim(in) = ypssim(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%eysimI(in) = eysimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%zysimI(in) = zysimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%usimI(in) = usimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%usim(in) = usim(idsimI(in),simtrans(-1)%jsim(in))
		
	ELSE
		simtrans(-1)%ysim(in) = 0.0
		simtrans(-1)%ypssim(in) = 0.0
		simtrans(-1)%eysimI(in) = 0
		simtrans(-1)%zysimI(in) = 0
		simtrans(-1)%usimI(in) = 0
		simtrans(-1)%usim(in) = 0.0
	END IF

	!decision variables in period before announcement
	simtrans(-1)%csim(in) = csim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%ssim(in) = ssim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%taxsim(in) = taxsim(idsimI(in),simtrans(-1)%jsim(in))
	
	
	!assets and durables at time of announcement
	IF(simtrans(-1)%jsim(in)==Jtot) THEN
	   IF(ngpzy .eq. 21) THEN
		CALL DiscreteDist1(tempdraw,2,initilldist(zysimI(idsimI(in),1),:),initillrand(in,0))
                simtrans(0)%asim(in)=(real(tempdraw)-1.0d0)*initill(zysimI(idsimI(in),1))
                CALL DiscreteDist1(tempdraw,2,initliqdist(zysimI(idsimI(in),1),:),initliqrand(in,0))
                simtrans(0)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(zysimI(idsimI(in),1))
            ELSE

                CALL DiscreteDist1(tempdraw,2,initilldist(fylevsimI(idsimI(in)),:),initillrand(in,0))
		simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
		CALL DiscreteDist1(tempdraw,2,initliqdist(fylevsimI(idsimI(in)),:),initliqrand(in,0))
	        simtrans(0)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))  

            ENDIF  
		simtrans(0)%dedsimI(in) = 1		
	ELSE
		simtrans(0)%asim(in) = asim(idsimI(in),simtrans(-1)%jsim(in)+1)
		simtrans(0)%lsim(in) = lsim(idsimI(in),simtrans(-1)%jsim(in)+1)
		simtrans(0)%dedsimI(in) = dedsimI(idsimI(in),simtrans(-1)%jsim(in)+1)		
	END IF
END DO
!$OMP END PARALLEL DO



DO it = 0,Ttransition +Tadditionalsim
	IF (Display==1) write(*,*) 'Filling in random numbers for transition at time ',it,' of ',Ttransition+Tadditionalsim

	!$OMP PARALLEL DO PRIVATE(ij,ijret,iznext,lpynext)
	DO in = 1,insim	
		
		!age
		IF(simtrans(it-1)%jsim(in)==Jtot) THEN	!die with prob 1
			simtrans(it-1)%diesimI(in) = 1
			simtrans(it)%jsim(in) = 1
		ELSE IF(simtrans(it-1)%jsim(in)<=Jwork) THEN !working age dont die
			simtrans(it-1)%diesimI(in) = 0
			simtrans(it)%jsim(in) = simtrans(it-1)%jsim(in) + 1
		ELSE !retired, might die
			CALL DiscreteDist1(simtrans(it-1)%diesimI(in), 2, (/surprob(simtrans(it-1)%jsim(in)-Jwork),1.0-surprob(simtrans(it-1)%jsim(in)-Jwork)/),dierand(in,it))
			simtrans(it-1)%diesimI(in) = simtrans(it-1)%diesimI(in)-1
			IF(simtrans(it-1)%diesimI(in)==0) simtrans(it)%jsim(in) = simtrans(it-1)%jsim(in) + 1
			IF(simtrans(it-1)%diesimI(in)==1) simtrans(it)%jsim(in) = 1
		END IF
		ij = simtrans(it)%jsim(in)
		ijret = ij-Jwork
		
		SELECT CASE(ij)
		
			CASE(1)	!newly born
				CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/1.0-initialunemp,initialunemp/),uerand(in,it))
				simtrans(it)%usim(in)	= ugrid(simtrans(it)%usimI(in))
				CALL DiscreteDist1(simtrans(it)%zysimI(in), ngpzy, zydist(1,:),zyrand(in,it))
				IF(SimulateAtMeanTrans==0 .and. ngpey>1) THEN
					CALL DiscreteDist1(simtrans(it)%eysimI(in), ngpey, eydist,eyrand(in,it))
    			ELSE
					simtrans(it)%eysimI(in)= (ngpey+1)/2
				END IF

				simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
				IF(simtrans(it)%usimI(in)==1) THEN
					simtrans(it)%ysim(in) = ygrid(ij,fysimtransI(in),simtrans(it)%zysimI(in),simtrans(it)%eysimI(in))
				ELSE IF(simtrans(it)%usimI(in)==2) THEN
					simtrans(it)%ysim(in) =  uemprepl*simtrans(it)%ypssim(in)
			    END IF
			    simtrans(it)%psimI(in) = 1
			    simtrans(it)%psim(in) = 0.0
			IF(ngpzy .eq. 21) THEN
				CALL DiscreteDist1(tempdraw,2,initilldist(simtrans(it)%zysimI(in),:),initillrand(in,it))
				simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(simtrans(it)%zysimI(in))
				CALL DiscreteDist1(tempdraw,2,initliqdist(simtrans(it)%zysimI(in),:),initliqrand(in,it))
				simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(simtrans(it)%zysimI(in))
			ELSE
				CALL DiscreteDist1(tempdraw,2,initilldist(fylevsimI(idsimI(in)),:),initillrand(in,it))
				simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
				CALL DiscreteDist1(tempdraw,2,initliqdist(fylevsimI(idsimI(in)),:),initliqrand(in,it))
				simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))
			ENDIF
			CASE(2:Jwork)	!working age
	    		
				SELECT CASE(simtrans(it-1)%usimI(in))	
					CASE(1)	!emp last period
			        	CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/1.0-jobloss(ij-1),jobloss(ij-1) /),uerand(in,it))
						simtrans(it)%usim(in) = ugrid(simtrans(it)%usimI(in))
			
						IF(simtrans(it)%usimI(in)==1) THEN	!emp this period too
							CALL DiscreteDist1(simtrans(it)%zysimI(in), ngpzy, zytrans(ij-1,simtrans(it-1)%zysimI(in),:),zyrand(in,it))
							IF(SimulateAtMeanTrans==0 .and. ngpey>1) THEN
								CALL DiscreteDist1(simtrans(it)%eysimI(in), ngpey, eydist,eyrand(in,it))
							ELSE
								simtrans(it)%eysimI(in)= (ngpey+1)/2							
							END IF
				        	simtrans(it)%ysim(in) = ygrid(ij,fysimtransI(in),simtrans(it)%zysimI(in),simtrans(it)%eysimI(in))
					    	simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
							
						ELSE IF(simtrans(it)%usimI(in)==2) THEN	!lost job
					        simtrans(it)%zysimI(in) = simtrans(it-1)%zysimI(in)
							simtrans(it)%eysimI(in) = 0
					    	simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
							simtrans(it)%ysim(in) 	= uemprepl*simtrans(it)%ypssim(in)
							
						END IF
			
					CASE(2) !unemp last period
						CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/jobfind(ij-1),1.0-jobfind(ij-1) /),uerand(in,it))
						simtrans(it)%usim(in) = ugrid(simtrans(it)%usimI(in))
				
						IF(simtrans(it)%usimI(in)==1) THEN	!found job
							IF(ngpzy>1) THEN
								CALL FindLinProb1 ((1.0-uemplosspers)*ypsgrid(ij,fysimtransI(in),simtrans(it-1)%zysimI(in)),ypsgrid(ij,fysimtransI(in),:),iznext,lpynext)
						        CALL DiscreteDist1(simtrans(it)%zysimI(in), 2, lpynext,zyrand(in,it))
								simtrans(it)%zysimI(in) = iznext(simtrans(it)%zysimI(in))
							ELSE
								simtrans(it)%zysimI(in) = 1
							END IF
					        simtrans(it)%eysimI(in) = 0
					        simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
					        simtrans(it)%ysim(in) = simtrans(it)%ypssim(in) * (1.0-uemplosstrans)
	
						ELSE IF(simtrans(it)%usimI(in)==2) THEN	!remain unemployed
					        simtrans(it)%zysimI(in) = simtrans(it-1)%zysimI(in)
							simtrans(it)%eysimI(in) = simtrans(it-1)%eysimI(in)
						    simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
					        simtrans(it)%ysim(in) = uemprepl*simtrans(it)%ypssim(in)
							
						END IF

				END SELECT
	
			    simtrans(it)%psimI(in) = 1
			    simtrans(it)%psim(in) = 0.0

			CASE(Jwork+1)	!first retirement year: draw medical stuff
				simtrans(it)%usimI(in) = 0
				simtrans(it)%usim(in) = 0.0
				simtrans(it)%ysim(in) = 0.0_8
				simtrans(it)%ypssim(in) = 0.0_8
				simtrans(it)%psimI(in) = (simtrans(it-1)%zysimI(in)-1)*ngpfy+fysimtransI(in)
				simtrans(it)%psim(in) = pgrid(ij,simtrans(it)%psimI(in))
						
			CASE(Jwork+2:Jtot)	!retirement age
				simtrans(it)%usimI(in) = 0
				simtrans(it)%usim(in) = 0.0
        		simtrans(it)%ysim(in) = 0.0_8
	    		simtrans(it)%ypssim(in) = 0.0_8
				simtrans(it)%psimI(in) = simtrans(it-1)%psimI(in)
			    simtrans(it)%psim(in) = pgrid(ij,simtrans(it)%psimI(in))
		
		END SELECT
        

        
    END DO    
	!$OMP END PARALLEL DO
	
END DO


!set up rebate types and copy random numbers if using same random numbers for rebate types
!rebate type
DO in = 1,nsimtrans
	IF(in<=nsimrebateA) rebatetypeI(in) = 1
	IF(in<=nsimrebateA+nsimrebateB .and. in>nsimrebateA) rebatetypeI(in) = 2
	IF(in>nsimrebateA+nsimrebateB) rebatetypeI(in) = 0
END DO
IF(SameRandomNumbersRebate == 1) THEN
	!$OMP PARALLEL DO
	DO in = 1,nsimrebateA
		simtrans(:)%diesimI(in+nsimrebateA) = simtrans(:)%diesimI(in)
		simtrans(:)%jsim(in+nsimrebateA) = simtrans(:)%jsim(in)
		simtrans(:)%usimI(in+nsimrebateA) = simtrans(:)%usimI(in)
		simtrans(:)%usim(in+nsimrebateA) = simtrans(:)%usim(in)
		simtrans(:)%ysim(in+nsimrebateA) = simtrans(:)%ysim(in)
		simtrans(:)%ypssim(in+nsimrebateA) = simtrans(:)%ypssim(in)
		simtrans(:)%psimI(in+nsimrebateA) = simtrans(:)%psimI(in)
		simtrans(:)%psim(in+nsimrebateA) = simtrans(:)%psim(in)
		fysimtransI(in+nsimrebateA) = fysimtransI(in)
		simtrans(:)%eysimI(in+nsimrebateA) = simtrans(:)%eysimI(in)
		simtrans(:)%zysimI(in+nsimrebateA) = simtrans(:)%zysimI(in)
		
		simtrans(:)%asim(in+nsimrebateA) = simtrans(:)%asim(in) 
		simtrans(:)%lsim(in+nsimrebateA) = simtrans(:)%lsim(in) 
		simtrans(:)%dedsimI(in+nsimrebateA) = simtrans(:)%dedsimI(in) 
		simtrans(-1)%csim(in+nsimrebateA) = simtrans(-1)%csim(in) 
		simtrans(-1)%ssim(in+nsimrebateA) = simtrans(-1)%ssim(in) 
		simtrans(-1)%xsim(in+nsimrebateA) = simtrans(-1)%xsim(in) 
		simtrans(-1)%taxsim(in+nsimrebateA) = simtrans(-1)%taxsim(in) 
		simtrans(-1)%adjsim(in+nsimrebateA) = simtrans(-1)%adjsim(in) 
	END DO
	!$OMP END PARALLEL DO

	!$OMP PARALLEL DO
	DO in = nsimrebateA+1,2*nsimrebateA
		simtrans(:)%diesimI(in+nsimrebateA) = simtrans(:)%diesimI(in)
		simtrans(:)%jsim(in+nsimrebateA) = simtrans(:)%jsim(in)
		simtrans(:)%usimI(in+nsimrebateA) = simtrans(:)%usimI(in)
		simtrans(:)%usim(in+nsimrebateA) = simtrans(:)%usim(in)
		simtrans(:)%ysim(in+nsimrebateA) = simtrans(:)%ysim(in)
		simtrans(:)%ypssim(in+nsimrebateA) = simtrans(:)%ypssim(in)
		simtrans(:)%psimI(in+nsimrebateA) = simtrans(:)%psimI(in)
		simtrans(:)%psim(in+nsimrebateA) = simtrans(:)%psim(in)
		fysimtransI(in+nsimrebateA) = fysimtransI(in)
		simtrans(:)%eysimI(in+nsimrebateA) = simtrans(:)%eysimI(in)
		simtrans(:)%zysimI(in+nsimrebateA) = simtrans(:)%zysimI(in)
	
		simtrans(:)%asim(in+nsimrebateA) = simtrans(:)%asim(in) 
		simtrans(:)%lsim(in+nsimrebateA) = simtrans(:)%lsim(in) 
		simtrans(:)%dedsimI(in+nsimrebateA) = simtrans(:)%dedsimI(in) 
		simtrans(-1)%csim(in+nsimrebateA) = simtrans(-1)%csim(in) 
		simtrans(-1)%ssim(in+nsimrebateA) = simtrans(-1)%ssim(in) 
		simtrans(-1)%xsim(in+nsimrebateA) = simtrans(-1)%xsim(in) 
		simtrans(-1)%taxsim(in+nsimrebateA) = simtrans(-1)%taxsim(in) 
		simtrans(-1)%adjsim(in+nsimrebateA) = simtrans(-1)%adjsim(in) 
	END DO
	!$OMP END PARALLEL DO
	
END IF


DO it = 0,Ttransition+Tadditionalsim
	write(*,*) 'Simulating transition at time ',it,' of ',Ttransition+Tadditionalsim
	
	DO in = 1,nsimtrans

		IF(ModelRecession==1 .and. SolveTransition==1) THEN
			IF(it<=recndrop-1) THEN	!-1 since time=0 is period of surprise
				IF(simtrans(it)%ysim(in) .le. recylim) simtrans(it)%ysim(in) = (1.0-(it+1)*recdepthcond/real(recndrop))*simtrans(it)%ysim(in)
				IF(simtrans(it)%ypssim(in) .le. recylim) simtrans(it)%ypssim(in) = (1.0-(it+1)*recdepthcond/real(recndrop))*simtrans(it)%ypssim(in)
				
			ELSE IF(it<=recngrow+recndrop-1) THEN
				IF(simtrans(it)%ysim(in) .le. recylim) simtrans(it)%ysim(in) = (1.0-recdepthcond + (it+1-recndrop)*recdepthcond/real(recngrow))*simtrans(it)%ysim(in)
				IF(simtrans(it)%ypssim(in) .le. recylim) simtrans(it)%ypssim(in) = (1.0-recdepthcond + (it+1-recndrop)*recdepthcond/real(recngrow))*simtrans(it)%ypssim(in)
			END IF				
		END IF
		
		ij = simtrans(it)%jsim(in)
		ijret = ij-Jwork
		ip = simtrans(it)%psimI(in)
		ify = fysimtransI(in)
		izy = simtrans(it)%zysimI(in)
		iu	= simtrans(it)%usimI(in)

		IF(ij<=Jwork) iW = Wind(ify,izy)
		IF(ij>Jwork) iR = Rind(ip)
		
		IF(ij <Jwork) lliqmin = abl(ij+1,ify,izy)
		IF(ij>=Jwork) lliqmin = 0.0
		lsavmin = lliqmin*FnBondPrice(lliqmin,0.0_8)
		
		!work out tax system
		IF (NoChangeTaxSystem==1 .or. SolveTransition==0) THEN
			ltaxsys = WhichTaxSystem
		ELSE
			IF (it<=4-announcequ) THEN !in 2001
				iyr = 2001
				iqu = announcequ + it
			ELSE
				iyr = 2001 + (announcequ+it-1-mod(announcequ+it-1,4))/4
				iqu = mod(announcequ+it-1,4) + 1
			END IF

			SELECT CASE(iyr)
				CASE(2001)
					ltaxsys = 1
				CASE(2002)
					ltaxsys = 2
				CASE(2003)
					ltaxsys = 3
				CASE(2004)
					ltaxsys = 4
				CASE(2005)
					ltaxsys = 5
				CASE(2006:2010)
					ltaxsys = 6
				CASE(2011:)
					IF (sunset==1) ltaxsys = 1
					IF (sunset==0) ltaxsys = 6
			END SELECT
		END IF
		
		!assign rebate amounts
		simtrans(it)%rebsim(in) = 0.0	!stuff below modifies decisions rules and rebate amounts if rebatetypeI = 1 or 2.

		IF(iyr .lt. 2011) BushDebt = BushDebt+(FnTax(1,simtrans(it)%ysim(in),1,1,0)-FnTax(ltaxsys,simtrans(it)%ysim(in),1,1,0))/real(insim)

		
	END DO
	BushDebt=BushDebt/qgov
END DO

write(*,*) 'Bush Debt: ', BushDebt

END SUBROUTINE SimulateTaxesTransition
