SUBROUTINE SimulateTransition

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

INTEGER		:: iseed(2),isize,jL,jH,indexval

INTEGER		:: in,ij,ijret,it,iyr,iqu,ip,ify,izy,iznext(2),iu,ltaxsys,insim,iW,iR,id,idedchoice(1),tempdraw,n5,count1,count2,count3,countj,ijtemp
REAL(8)	    :: lxA,lxN,lass,lliqmin,lsavmin,lsavmax,lpynext(2),lvalded(ngpd),lvaldedret(ngpdret),ltaxincr,tempincval
REAL(8), EXTERNAL	:: FnTax, FnBondPrice, FnInvQA

IF(Display==1) write(*,*) 'Simulating model transition'

n5=5

!IF(BushInc==0) THEN

!set random number seed
iseed(1) = 5432
iseed(2) = 6543
isize = 2
CALL RANDOM_SEED(size = isize)
CALL RANDOM_SEED(put = iseed)   


IF(SameRandomNumbersRebate == 0) THEN
	insim = nsimtrans
ELSE
	insim = nsimrebateA
END IF


!fill in steady state simulation, last period before announcement, by randomly drawing ids and assigning ages per poulation
IF(DeterministicModel==0) THEN
	DO in = 1,insim
		idsimI(in) = mod(in,nsim)+1
		CALL RandomDiscrete1(simtrans(-1)%jsim(in), Jtot, popdist)
	END DO
ELSE
	DO in = 1,insim
		idsimI(in) = 1	!deterministic model so only 1 steady state simulation
		simtrans(-1)%jsim(in) = in
	END DO
END IF


!draw all random numbers so can do in parallel
write(*,*) 'drawing random numbers for simulations'
CALL RANDOM_NUMBER(dierand)
CALL RANDOM_NUMBER(uerand)
CALL RANDOM_NUMBER(zyrand)
CALL RANDOM_NUMBER(eyrand)
CALL RANDOM_NUMBER(initliqrand)
CALL RANDOM_NUMBER(initillrand)


   CALL RandomDiscrete(insim, rebsimI(1:in), nrebgrid, rebdist)

	IF(SameRandomNumbersRebate == 1) THEN
    	rebsimI(nsimrebateA+1:2*nsimrebateA)=rebsimI(1:nsimrebateA)
        rebsimI(2*nsimrebateA+1:nsimtrans)=rebsimI(1:nsimrebateA)
	ENDIF

IF(SameRandomNumbersRebate == 1) THEN

	idsimI(nsimrebateA+1:2*nsimrebateA)=idsimI(1:nsimrebateA)
	idsimI(2*nsimrebateA+1:nsimtrans)=idsimI(1:nsimrebateA)

END IF



!$OMP PARALLEL DO PRIVATE(it,tempdraw)
DO in = 1,insim 
	
	!fixed state variables
	fysimtransI(in) = fysimI(idsimI(in))

	!state variables at beginning of transition
	simtrans(-1)%asim(in) = asim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%lsim(in) = lsim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%psim(in) = psim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%psimI(in) = psimI(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%dedsimI(in) = dedsimI(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%adjsim(in) = adjsim(idsimI(in),simtrans(-1)%jsim(in))
	
	
	IF(simtrans(-1)%jsim(in) <=Jwork) THEN
		simtrans(-1)%ysim(in) = ysim(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%ypssim(in) = ypssim(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%eysimI(in) = eysimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%zysimI(in) = zysimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%usimI(in) = usimI(idsimI(in),simtrans(-1)%jsim(in))
		simtrans(-1)%usim(in) = usim(idsimI(in),simtrans(-1)%jsim(in))
		
	ELSE
		simtrans(-1)%ysim(in) = 0.0
		simtrans(-1)%ypssim(in) = 0.0
		simtrans(-1)%eysimI(in) = 0
		simtrans(-1)%zysimI(in) = 0
		simtrans(-1)%usimI(in) = 0
		simtrans(-1)%usim(in) = 0.0
	END IF

	!decision variables in period before announcement
	simtrans(-1)%csim(in) = ctotsim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%ssim(in) = ssim(idsimI(in),simtrans(-1)%jsim(in))
	simtrans(-1)%taxsim(in) = taxsim(idsimI(in),simtrans(-1)%jsim(in))
	
	
	!assets and durables at time of announcement
	IF(simtrans(-1)%jsim(in)==Jtot) THEN
		IF(NoInitialWealth==1) THEN
			simtrans(0)%asim(in) = 0.0
			simtrans(0)%lsim(in) = 0.0
		ELSE	
			IF(ngpzy .eq. 21) THEN
				CALL DiscreteDist1(tempdraw,2,initilldist(zysimI(idsimI(in),1),:),initillrand(in,0))
	            simtrans(0)%asim(in)=(real(tempdraw)-1.0d0)*initill(zysimI(idsimI(in),1))
	            CALL DiscreteDist1(tempdraw,2,initliqdist(zysimI(idsimI(in),1),:),initliqrand(in,0))
	            simtrans(0)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(zysimI(idsimI(in),1))
			ELSE
				CALL DiscreteDist1(tempdraw,2,initilldist(fylevsimI(idsimI(in)),:),initillrand(in,0))
				simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
				CALL DiscreteDist1(tempdraw,2,initliqdist(fylevsimI(idsimI(in)),:),initliqrand(in,0))
		        simtrans(0)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))  
			END IF  
		END IF
		simtrans(0)%dedsimI(in) = 1	
		
	ELSE
		simtrans(0)%asim(in) = asim(idsimI(in),simtrans(-1)%jsim(in)+1)
		simtrans(0)%lsim(in) = lsim(idsimI(in),simtrans(-1)%jsim(in)+1)
		simtrans(0)%dedsimI(in) = dedsimI(idsimI(in),simtrans(-1)%jsim(in)+1)		
	END IF
END DO
!$OMP END PARALLEL DO



DO it = 0,Ttransition +Tadditionalsim
	IF (Display==1) write(*,*) 'Filling in random numbers for transition at time ',it,' of ',Ttransition+Tadditionalsim

	!$OMP PARALLEL DO PRIVATE(ij,ijret,iznext,lpynext,tempdraw)
	DO in = 1,insim	
		
		!age
		IF(simtrans(it-1)%jsim(in)==Jtot) THEN	!die with prob 1
			simtrans(it-1)%diesimI(in) = 1
			simtrans(it)%jsim(in) = 1
		ELSE IF(simtrans(it-1)%jsim(in)<=Jwork) THEN !working age dont die
			simtrans(it-1)%diesimI(in) = 0
			simtrans(it)%jsim(in) = simtrans(it-1)%jsim(in) + 1
		ELSE !retired, might die
			CALL DiscreteDist1(simtrans(it-1)%diesimI(in), 2, (/surprob(simtrans(it-1)%jsim(in)-Jwork),1.0-surprob(simtrans(it-1)%jsim(in)-Jwork)/),dierand(in,it))
			simtrans(it-1)%diesimI(in) = simtrans(it-1)%diesimI(in)-1
			IF(simtrans(it-1)%diesimI(in)==0) simtrans(it)%jsim(in) = simtrans(it-1)%jsim(in) + 1
			IF(simtrans(it-1)%diesimI(in)==1) simtrans(it)%jsim(in) = 1
		END IF
		ij = simtrans(it)%jsim(in)
		ijret = ij-Jwork
		
		SELECT CASE(ij)
		
			CASE(1)	!newly born
				CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/1.0-initialunemp,initialunemp/),uerand(in,it))
				simtrans(it)%usim(in)	= ugrid(simtrans(it)%usimI(in))
				CALL DiscreteDist1(simtrans(it)%zysimI(in), ngpzy, zydist(1,:),zyrand(in,it))
				IF(SimulateAtMeanTrans==0 .and. ngpey>1) THEN
					CALL DiscreteDist1(simtrans(it)%eysimI(in), ngpey, eydist,eyrand(in,it))
    			ELSE
					simtrans(it)%eysimI(in)= (ngpey+1)/2
				END IF

				simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
				IF(simtrans(it)%usimI(in)==1) THEN
					simtrans(it)%ysim(in) = ygrid(ij,fysimtransI(in),simtrans(it)%zysimI(in),simtrans(it)%eysimI(in))
				ELSE IF(simtrans(it)%usimI(in)==2) THEN
					simtrans(it)%ysim(in) =  uemprepl*simtrans(it)%ypssim(in)
			    END IF
			    simtrans(it)%psimI(in) = 1
			    simtrans(it)%psim(in) = 0.0
			
				IF(NoInitialWealth==1) THEN
					simtrans(it)%asim(in) = 0.0
					simtrans(it)%lsim(in) = 0.0
				ELSE
					IF(ngpzy .eq. 21) THEN
						CALL DiscreteDist1(tempdraw,2,initilldist(simtrans(it)%zysimI(in),:),initillrand(in,it))
						simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(simtrans(it)%zysimI(in))
						CALL DiscreteDist1(tempdraw,2,initliqdist(simtrans(it)%zysimI(in),:),initliqrand(in,it))
						simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(simtrans(it)%zysimI(in))
					ELSE
						CALL DiscreteDist1(tempdraw,2,initilldist(fylevsimI(idsimI(in)),:),initillrand(in,it))
						simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
						CALL DiscreteDist1(tempdraw,2,initliqdist(fylevsimI(idsimI(in)),:),initliqrand(in,it))
						simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))
					END IF
				END IF
				
			CASE(2:Jwork)	!working age
	    		
				SELECT CASE(simtrans(it-1)%usimI(in))	
					CASE(1)	!emp last period
			        	CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/1.0-jobloss(ij-1),jobloss(ij-1) /),uerand(in,it))
						simtrans(it)%usim(in) = ugrid(simtrans(it)%usimI(in))
			
						IF(simtrans(it)%usimI(in)==1) THEN	!emp this period too
							CALL DiscreteDist1(simtrans(it)%zysimI(in), ngpzy, zytrans(ij-1,simtrans(it-1)%zysimI(in),:),zyrand(in,it))
							IF(SimulateAtMeanTrans==0 .and. ngpey>1) THEN
								CALL DiscreteDist1(simtrans(it)%eysimI(in), ngpey, eydist,eyrand(in,it))
							ELSE
								simtrans(it)%eysimI(in)= (ngpey+1)/2							
							END IF
				        	simtrans(it)%ysim(in) = ygrid(ij,fysimtransI(in),simtrans(it)%zysimI(in),simtrans(it)%eysimI(in))
					    	simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
							
						ELSE IF(simtrans(it)%usimI(in)==2) THEN	!lost job
					        simtrans(it)%zysimI(in) = simtrans(it-1)%zysimI(in)
							simtrans(it)%eysimI(in) = 0
					    	simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
							simtrans(it)%ysim(in) 	= uemprepl*simtrans(it)%ypssim(in)
							
						END IF
			
					CASE(2) !unemp last period
						CALL DiscreteDist1(simtrans(it)%usimI(in), ngpu, (/jobfind(ij-1),1.0-jobfind(ij-1) /),uerand(in,it))
						simtrans(it)%usim(in) = ugrid(simtrans(it)%usimI(in))
				
						IF(simtrans(it)%usimI(in)==1) THEN	!found job
							IF(ngpzy>1) THEN
								CALL FindLinProb1 ((1.0-uemplosspers)*ypsgrid(ij,fysimtransI(in),simtrans(it-1)%zysimI(in)),ypsgrid(ij,fysimtransI(in),:),iznext,lpynext)
						        CALL DiscreteDist1(simtrans(it)%zysimI(in), 2, lpynext,zyrand(in,it))
								simtrans(it)%zysimI(in) = iznext(simtrans(it)%zysimI(in))
							ELSE
								simtrans(it)%zysimI(in) = 1
							END IF
					        simtrans(it)%eysimI(in) = 0
					        simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
					        simtrans(it)%ysim(in) = simtrans(it)%ypssim(in) * (1.0-uemplosstrans)
	
						ELSE IF(simtrans(it)%usimI(in)==2) THEN	!remain unemployed
					        simtrans(it)%zysimI(in) = simtrans(it-1)%zysimI(in)
							simtrans(it)%eysimI(in) = simtrans(it-1)%eysimI(in)
						    simtrans(it)%ypssim(in) = ypsgrid(ij,fysimtransI(in),simtrans(it)%zysimI(in))
					        simtrans(it)%ysim(in) = uemprepl*simtrans(it)%ypssim(in)
							
						END IF

				END SELECT
	
			    simtrans(it)%psimI(in) = 1
			    simtrans(it)%psim(in) = 0.0

			CASE(Jwork+1)	!first retirement year: draw medical stuff
				simtrans(it)%usimI(in) = 0
				simtrans(it)%usim(in) = 0.0
				simtrans(it)%ysim(in) = 0.0_8
				simtrans(it)%ypssim(in) = 0.0_8
				simtrans(it)%psimI(in) = (simtrans(it-1)%zysimI(in)-1)*ngpfy+fysimtransI(in)
				simtrans(it)%psim(in) = pgrid(ij,simtrans(it)%psimI(in))
						
			CASE(Jwork+2:Jtot)	!retirement age
				simtrans(it)%usimI(in) = 0
				simtrans(it)%usim(in) = 0.0
        		simtrans(it)%ysim(in) = 0.0_8
	    		simtrans(it)%ypssim(in) = 0.0_8
				simtrans(it)%psimI(in) = simtrans(it-1)%psimI(in)
			    simtrans(it)%psim(in) = pgrid(ij,simtrans(it)%psimI(in))
		
		END SELECT
        

        
    END DO    
	!$OMP END PARALLEL DO
	
END DO


!set up rebate types and copy random numbers if using same random numbers for rebate types
!rebate type
DO in = 1,nsimtrans
	IF(in<=nsimrebateA) rebatetypeI(in) = 1
	IF(in<=nsimrebateA+nsimrebateB .and. in>nsimrebateA) rebatetypeI(in) = 2
	IF(in>nsimrebateA+nsimrebateB) rebatetypeI(in) = 0
END DO
IF(SameRandomNumbersRebate == 1) THEN
	!$OMP PARALLEL DO
	DO in = 1,nsimrebateA
		simtrans(:)%diesimI(in+nsimrebateA) = simtrans(:)%diesimI(in)
		simtrans(:)%jsim(in+nsimrebateA) = simtrans(:)%jsim(in)
		simtrans(:)%usimI(in+nsimrebateA) = simtrans(:)%usimI(in)
		simtrans(:)%usim(in+nsimrebateA) = simtrans(:)%usim(in)
		simtrans(:)%ysim(in+nsimrebateA) = simtrans(:)%ysim(in)
		simtrans(:)%ypssim(in+nsimrebateA) = simtrans(:)%ypssim(in)
		simtrans(:)%psimI(in+nsimrebateA) = simtrans(:)%psimI(in)
		simtrans(:)%psim(in+nsimrebateA) = simtrans(:)%psim(in)
		fysimtransI(in+nsimrebateA) = fysimtransI(in)
		simtrans(:)%eysimI(in+nsimrebateA) = simtrans(:)%eysimI(in)
		simtrans(:)%zysimI(in+nsimrebateA) = simtrans(:)%zysimI(in)
		
		simtrans(:)%asim(in+nsimrebateA) = simtrans(:)%asim(in) 
		simtrans(:)%lsim(in+nsimrebateA) = simtrans(:)%lsim(in) 
		simtrans(:)%dedsimI(in+nsimrebateA) = simtrans(:)%dedsimI(in) 
		simtrans(-1)%csim(in+nsimrebateA) = simtrans(-1)%csim(in) 
		simtrans(-1)%ssim(in+nsimrebateA) = simtrans(-1)%ssim(in) 
		simtrans(-1)%xsim(in+nsimrebateA) = simtrans(-1)%xsim(in) 
		simtrans(-1)%taxsim(in+nsimrebateA) = simtrans(-1)%taxsim(in) 
		simtrans(-1)%adjsim(in+nsimrebateA) = simtrans(-1)%adjsim(in) 
	END DO
	!$OMP END PARALLEL DO

	!$OMP PARALLEL DO
	DO in = nsimrebateA+1,2*nsimrebateA
		simtrans(:)%diesimI(in+nsimrebateA) = simtrans(:)%diesimI(in)
		simtrans(:)%jsim(in+nsimrebateA) = simtrans(:)%jsim(in)
		simtrans(:)%usimI(in+nsimrebateA) = simtrans(:)%usimI(in)
		simtrans(:)%usim(in+nsimrebateA) = simtrans(:)%usim(in)
		simtrans(:)%ysim(in+nsimrebateA) = simtrans(:)%ysim(in)
		simtrans(:)%ypssim(in+nsimrebateA) = simtrans(:)%ypssim(in)
		simtrans(:)%psimI(in+nsimrebateA) = simtrans(:)%psimI(in)
		simtrans(:)%psim(in+nsimrebateA) = simtrans(:)%psim(in)
		fysimtransI(in+nsimrebateA) = fysimtransI(in)
		simtrans(:)%eysimI(in+nsimrebateA) = simtrans(:)%eysimI(in)
		simtrans(:)%zysimI(in+nsimrebateA) = simtrans(:)%zysimI(in)
	
		simtrans(:)%asim(in+nsimrebateA) = simtrans(:)%asim(in) 
		simtrans(:)%lsim(in+nsimrebateA) = simtrans(:)%lsim(in) 
		simtrans(:)%dedsimI(in+nsimrebateA) = simtrans(:)%dedsimI(in) 
		simtrans(-1)%csim(in+nsimrebateA) = simtrans(-1)%csim(in) 
		simtrans(-1)%ssim(in+nsimrebateA) = simtrans(-1)%ssim(in) 
		simtrans(-1)%xsim(in+nsimrebateA) = simtrans(-1)%xsim(in) 
		simtrans(-1)%taxsim(in+nsimrebateA) = simtrans(-1)%taxsim(in) 
		simtrans(-1)%adjsim(in+nsimrebateA) = simtrans(-1)%adjsim(in) 
	END DO
	!$OMP END PARALLEL DO
	
END IF

!set rebateamt to average so that it works fine with tax increases
rebateamt = DOT_PRODUCT(rebgrid,rebdist)

DO it = 0,Ttransition+Tadditionalsim
	write(*,*) 'Simulating transition at time ',it,' of ',Ttransition+Tadditionalsim
	IF(associated(gdecsim)) nullify(gdecsim)
	

	
	!$OMP PARALLEL DO PRIVATE(ij,ijret,iW,iR,iyr,iqu,ip,ify,izy,iu,ltaxsys,ltaxincr,lxA,LxN,lass,lliqmin,lsavmin,lsavmax,lvalded,lvaldedret,id,idedchoice,tempdraw) COPYIN(gdecsim)
	DO in = 1,nsimtrans

		IF(SolveTransition==0) THEN
			gdecsim => decInitialSS(1)
		ELSE IF (SolveTransition==1) THEN
			IF(it<=Ttransition) THEN
				gdecsim => dectrans(it)
			ELSE
				IF(sunset==1 .and. GovtBudgetConstraint==0) gdecsim => decInitialSS(1)
				IF(sunset==0 .or. GovtBudgetConstraint==1) gdecsim => decFinalSS(1)
			END IF
		END IF

		IF(GovtBudgetConstraint==1) THEN
			IF(BushInc==1) THEN
				IF(it>Ttransition) ltaxincr = (0.5*rebateamt*(1.0+qgov)*(1.0-qgov)/(qgov**npergovbc)+(1.0d0-qgov)*qgov*BushDebt/(qgov**(npergovbc-40)))/taxbase
			ELSE
				IF(it>Ttransition) ltaxincr = 0.5*rebateamt*(1.0+qgov)*(1.0-qgov)/((qgov**npergovbc)*taxbase)
			ENDIF
			IF(it<=Ttransition) ltaxincr = 0.0
		ELSE
			ltaxincr = 0.0
		END IF

		IF(ModelRecession==1 .and. SolveTransition==1) THEN
			IF(it<=recndrop-1) THEN	!-1 since time=0 is period of surprise
				IF(simtrans(it)%ysim(in) .le. recylim) simtrans(it)%ysim(in) = (1.0-(it+1)*recdepthcond/real(recndrop))*simtrans(it)%ysim(in)
				IF(simtrans(it)%ypssim(in) .le. recylim) simtrans(it)%ypssim(in) = (1.0-(it+1)*recdepthcond/real(recndrop))*simtrans(it)%ypssim(in)
! 				simtrans(it)%psim(in) = (1.0-(it+1)*recdepth/real(recndrop))*simtrans(it)%psim(in)
				
			ELSE IF(it<=recngrow+recndrop-1) THEN
				IF(simtrans(it)%ysim(in) .le. recylim) simtrans(it)%ysim(in) = (1.0-recdepthcond + (it+1-recndrop)*recdepthcond/real(recngrow))*simtrans(it)%ysim(in)
				IF(simtrans(it)%ypssim(in) .le. recylim) simtrans(it)%ypssim(in) = (1.0-recdepthcond + (it+1-recndrop)*recdepthcond/real(recngrow))*simtrans(it)%ypssim(in)
! 				simtrans(it)%psim(in) = (1.0-(it+1)*recdepth/real(recndrop))*simtrans(it)%psim(in)
			END IF				
		END IF
		
		ij = simtrans(it)%jsim(in)
		ijret = ij-Jwork
		ip = simtrans(it)%psimI(in)
		ify = fysimtransI(in)
		izy = simtrans(it)%zysimI(in)
		iu	= simtrans(it)%usimI(in)

		IF(ij<=Jwork) iW = Wind(ify,izy)
		IF(ij>Jwork) iR = Rind(ip)
		
		IF(ij <Jwork) lliqmin = abl(ij+1,ify,izy)
		IF(ij>=Jwork) lliqmin = 0.0
		lsavmin = lliqmin*FnBondPrice(lliqmin,0.0_8)
		
		!work out tax system
		IF (NoChangeTaxSystem==1 .or. SolveTransition==0) THEN
			ltaxsys = WhichTaxSystem
		ELSE
			IF (it<=4-announcequ) THEN !in 2001
				iyr = 2001
				iqu = announcequ + it
			ELSE
				iyr = 2001 + (announcequ+it-1-mod(announcequ+it-1,4))/4
				iqu = mod(announcequ+it-1,4) + 1
			END IF

			SELECT CASE(iyr)
				CASE(2001)
					ltaxsys = 1
				CASE(2002)
					ltaxsys = 2
				CASE(2003)
					ltaxsys = 3
				CASE(2004)
					ltaxsys = 4
				CASE(2005)
					ltaxsys = 5
				CASE(2006:2010)
					ltaxsys = 6
				CASE(2011:)
					IF (sunset==1) ltaxsys = 1
					IF (sunset==0) ltaxsys = 6
			END SELECT
		END IF
		
		!assign rebate amounts
		simtrans(it)%rebsim(in) = 0.0	!stuff below modifies decisions rules and rebate amounts if rebatetypeI = 1 or 2.
		SELECT CASE(announcequ)
		
			CASE(3) !announcement in quarter 3
				IF(it==0 .and. rebatetypeI(in)==1) simtrans(it)%rebsim(in) = rebgrid(rebsimI(in)) !Group A
				IF(it==1 .and. rebatetypeI(in)==2) simtrans(it)%rebsim(in) = rebgrid(rebsimI(in)) !Group B

			CASE(2) !announcement in quarter 2
				IF(it==1 .and. rebatetypeI(in)==1) simtrans(it)%rebsim(in) = rebgrid(rebsimI(in)) !Group A
				IF(it==2 .and. rebatetypeI(in)==2) simtrans(it)%rebsim(in) = rebgrid(rebsimI(in)) !Group B
			 		
		END SELECT
		
 !               IF(DISPLAY==1) write(*,*) 'ok to assign rebate amounts'
		!work out rebate decision rules
		IF (BothRebateSurprises==0) THEN	!else no chance in decision rules
                      indexval=rebsimI(in)
			SELECT CASE(announcequ)
		
				CASE(3) !announcement in quarter 3
					IF (rebatetypeI(in)==2 .and. it==0) THEN !Group B
						NULLIFY(gdecsim)
						gdecsim => decrebateB(0,indexval)
					END IF
		 
				CASE(2) !announcement in quarter 2
					IF(rebatetypeI(in)==1 .and. it==0) THEN !Group A
						NULLIFY(gdecsim)
						gdecsim => decrebateA(0,indexval)
					END IF
				
					IF (rebatetypeI(in)==2 .and. it<=1) THEN !Group B
						NULLIFY(gdecsim)
						gdecsim => decrebateB(it,indexval)
					END IF
			 		
			END SELECT
		END IF 
		
!                IF(DISPLAY==1) write(*,*) 'ok to assign gdecsim'
!                print*,simtrans(it)%rebsim(in),rebsimI(in)
		!fill in decisions
		SELECT CASE(ij)
		
			CASE(1:Jwork)	!working age
			
			 	IF(simtrans(it)%jsim(in)==1) THEN	!new born
					
					IF(NoInitialWealth==1) THEN
						simtrans(it)%asim(in) = 0.0
						simtrans(it)%lsim(in) = 0.0
					ELSE	
						IF(in .le. nsimrebateA) THEN
							IF(ngpzy .eq. 21) THEN
								CALL DiscreteDist1(tempdraw,2,initilldist(simtrans(it)%zysimI(in),:),initillrand(in,it))
								simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(simtrans(it)%zysimI(in))
								simtrans(it)%asim(in+nsimrebateA)=(real(tempdraw)-1.0d0)*initill(simtrans(it)%zysimI(in))
								simtrans(it)%asim(in+2*nsimrebateA)=(real(tempdraw)-1.0d0)*initill(simtrans(it)%zysimI(in))

								CALL DiscreteDist1(tempdraw,2,initliqdist(simtrans(it)%zysimI(in),:),initliqrand(in,it))
								simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(simtrans(it)%zysimI(in))
								simtrans(it)%lsim(in+nsimrebateA)=(real(tempdraw)-1.0d0)*initliq(simtrans(it)%zysimI(in))
								simtrans(it)%lsim(in+nsimrebateA*2)=(real(tempdraw)-1.0d0)*initliq(simtrans(it)%zysimI(in))

							ELSE
								CALL DiscreteDist1(tempdraw,2,initilldist(fylevsimI(idsimI(in)),:),initillrand(in,it))
								simtrans(it)%asim(in)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
								simtrans(it)%asim(in+nsimrebateA)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))
								simtrans(it)%asim(in+nsimrebateA*2)=(real(tempdraw)-1.0d0)*initill(fylevsimI(idsimI(in)))

								CALL DiscreteDist1(tempdraw,2,initliqdist(fylevsimI(idsimI(in)),:),initliqrand(in,it))
								simtrans(it)%lsim(in)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))
								simtrans(it)%lsim(in+nsimrebateA)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))
								simtrans(it)%lsim(in+nsimrebateA*2)=(real(tempdraw)-1.0d0)*initliq(fylevsimI(idsimI(in)))

							END IF
						END IF
					END IF

					simtrans(it)%dedsimI(in) = 1
				END IF

				lass = simtrans(it)%asim(in)
				simtrans(it)%taxsim(in) = FnTax(ltaxsys,simtrans(it)%ysim(in),1,INT(simtrans(it)%usim(in)),ltaxincr)+ mturcg*lass
				lxN = simtrans(it)%lsim(in) + simtrans(it)%ysim(in) - simtrans(it)%taxsim(in) - dgrid(simtrans(it)%dedsimI(in))*(simtrans(it)%ypssim(in)-simtrans(it)%taxsim(in)) + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
				lxA = simtrans(it)%lsim(in) + simtrans(it)%ysim(in) - simtrans(it)%taxsim(in) + simtrans(it)%asim(in)  + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
! 				IF(iu==1) lass = lass + dgrid(simtrans(it)%dedsimI(in))*simtrans(it)%ypssim(in) 
				

				!adjustment decision 
				IF(UsePosNegDvalInterp==0) CALL BiLinInterp1RatioXA(ij,ify,izy,ngpxN,xNgrid(ij,:,:,ify,izy,simtrans(it)%dedsimI(in)),ngpa,agrid(ij,:,ify,izy),gdecsim%dval(ij,:,:,simtrans(it)%dedsimI(in),iW,iu),lxN,lass,simtrans(it)%dVsim(in))			


				IF(simtrans(it)%dVsim(in)>=0.0) THEN!ADJUST
					simtrans(it)%adjsim(in) = 1.0
					simtrans(it)%xsim(in) = lxA

					lsavmax = lxA -fixcost - varcost*simtrans(it)%ypssim(in)*(1.0-ugrid(iu)*(1.0-uemprepl))- cmin*(1.0+ctax)
					IF (lxA==xAgrid(ij,1,ify,izy)) THEN
						idedchoice = maxloc(gdecsim%valA(ij,1,:,iW,iu))
						simtrans(it+1)%dedsimI(in) = idedchoice(1)
						simtrans(it)%Vsim(in)	= gdecsim%valA(ij,1,simtrans(it+1)%dedsimI(in),iW,iu)
						simtrans(it)%ssim(in)	= gdecsim%savA(ij,1,simtrans(it+1)%dedsimI(in),iW,iu)
						simtrans(it+1)%asim(in) = gdecsim%assA(ij,1,simtrans(it+1)%dedsimI(in),iW,iu)
					ELSE


						DO id = 1,ngpd
							CALL LinInterp1(ngpxA,xAgrid(ij,:,ify,izy),gdecsim%valA(ij,:,id,iW,iu),lxA,lvalded(id))
						END DO
						idedchoice = maxloc(lvalded)
						simtrans(it+1)%dedsimI(in) = idedchoice(1)
						simtrans(it)%Vsim(in) = lvalded(simtrans(it+1)%dedsimI(in))
						CALL LinInterp1(ngpxA,xAgrid(ij,:,ify,izy),gdecsim%savA(ij,:,simtrans(it+1)%dedsimI(in),iW,iu),lxA,simtrans(it)%ssim(in))
						CALL LinInterp1(ngpxA,xAgrid(ij,:,ify,izy),gdecsim%assA(ij,:,simtrans(it+1)%dedsimI(in),iW,iu),lxA,simtrans(it+1)%asim(in))

					END IF

					IF(simtrans(it)%ssim(in)>lsavmax .or. simtrans(it)%ssim(in)<lsavmin .or. simtrans(it+1)%asim(in)<0.0) THEN
						IF(Display==2) write(*,*) 'interpolation problem in simulations (working)'
						simtrans(it)%ssim(in) = max(min(simtrans(it)%ssim(in),lsavmax),lsavmin)
						simtrans(it+1)%asim(in) = max(simtrans(it+1)%asim(in),0.0)
					END IF
					simtrans(it+1)%lsim(in) = FnInvQA(simtrans(it)%ssim(in)-qill*simtrans(it+1)%asim(in),0.0_8)

					simtrans(it)%csim(in) = (lxA-fixcost-varcost*simtrans(it)%ypssim(in)*(1.0-ugrid(iu)*(1.0-uemprepl))-simtrans(it)%ssim(in))/(1.0+ctax)

					IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
						write(*,*) 'bad consumption (ADJ):',in,ij,idsimI(in),fylevsimI(idsimI(in)),initliq(fylevsimI(idsimI(in))),initill(fylevsimI(idsimI(in)))
					END IF

			
				ELSE !do not adjust

					simtrans(it)%adjsim(in) = 0.0
					simtrans(it)%xsim(in) = lxN
			
					IF(ij<Jwork) simtrans(it+1)%dedsimI(in) = simtrans(it)%dedsimI(in) 
					IF(ij==Jwork) simtrans(it+1)%dedsimI(in) = 1
			
					simtrans(it+1)%asim(in) = (lass +dgrid(simtrans(it)%dedsimI(in))*(simtrans(it)%ypssim(in)-simtrans(it)%taxsim(in)))/qill
					lsavmax = lxN - cmin*(1.0+ctax)

					CALL BiLinInterp1RatioXA(ij,ify,izy,ngpxN,xNgrid(ij,:,:,ify,izy,simtrans(it)%dedsimI(in)),ngpa,agrid(ij,:,ify,izy),gdecsim%valN(ij,:,:,simtrans(it)%dedsimI(in),iW,iu),lxN,lass,simtrans(it)%Vsim(in))
					CALL BiLinInterp1RatioXA(ij,ify,izy,ngpxN,xNgrid(ij,:,:,ify,izy,simtrans(it)%dedsimI(in)),ngpa,agrid(ij,:,ify,izy),gdecsim%savN(ij,:,:,simtrans(it)%dedsimI(in),iW,iu),lxN,lass,simtrans(it)%ssim(in))

					IF(simtrans(it)%ssim(in)>lsavmax .or. simtrans(it)%ssim(in)<lsavmin) THEN
						IF(Display==2) write(*,*) 'interpolation problem in simulations (working NA)'
						simtrans(it)%ssim(in) = max(min(simtrans(it)%ssim(in),lsavmax),lsavmin)
					END IF
					simtrans(it+1)%lsim(in) = FnInvQA(simtrans(it)%ssim(in),0.0_8)
					simtrans(it)%csim(in) = (lxN-simtrans(it)%ssim(in))/(1.0+ctax)

					IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
						if (Display==1) write(*,*) 'bad consumption (NO ADJ):',in,ij
					END IF

				END IF

				IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
				    if (Display==1) write(*,*) 'bad consumption (SimulateTransition):',in,ij,simtrans(it)%xsim(in),simtrans(it)%asim(in),simtrans(it)%csim(in),simtrans(it)%ssim(in),simtrans(it+1)%asim(in),simtrans(it)%ysim(in),simtrans(it)%taxsim(in),simtrans(it)%usim(in),simtrans(it+1)%lsim(in),simtrans(it)%lsim(in)
					simtrans(it)%csim(in) = cmin		!temporary fix, need to find bug here
				END IF

				!adjust for rounding errors
				IF(abs(simtrans(it+1)%asim(in))<1.0e-5) simtrans(it+1)%asim(in) = 0.0
				IF(abs(simtrans(it+1)%lsim(in))<1.0e-5) simtrans(it+1)%lsim(in) = 0.0
			
			CASE(Jwork+1:Jtot-1)	!retirement age	

				lass = simtrans(it)%asim(in)
				simtrans(it)%taxsim(in) = FnTax(ltaxsys,simtrans(it)%psim(in),0,0,ltaxincr)+ mturcg*lass
				lxN = simtrans(it)%lsim(in) + simtrans(it)%psim(in) - simtrans(it)%taxsim(in) + dgridret(simtrans(it)%dedsimI(in))*simtrans(it)%asim(in) + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
				lxA = simtrans(it)%lsim(in) + simtrans(it)%psim(in) - simtrans(it)%taxsim(in) + simtrans(it)%asim(in) + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
! 				lass = lass*(1.0-dgridret(simtrans(it)%dedsimI(in))) 
		


				IF(UsePosNegDvalInterp==0) CALL BiLinInterp1RatioXAret(ngpxNret,xNgridret(ijret,:,:,ip,simtrans(it)%dedsimI(in)),ngparet,agridret(ijret,:,ip),gdecsim%dvalret(ijret,:,:,simtrans(it)%dedsimI(in),iR),lxN,lass,simtrans(it)%dVsim(in))			



				IF(simtrans(it)%dVsim(in)>=0.0) THEN !adjust


					simtrans(it)%adjsim(in) = 1.0
					lsavmax = lxA -fixcost -varcost*simtrans(it)%psim(in) - cmin*(1.0+ctax)
					simtrans(it)%xsim(in) = lxA

					IF (lxA==xAgridret(ijret,1,ip)) THEN
						idedchoice = maxloc(gdecsim%valAret(ijret,1,:,iR))
						simtrans(it+1)%dedsimI(in) = idedchoice(1)
						simtrans(it)%Vsim(in)	= gdecsim%valAret(ijret,1,simtrans(it+1)%dedsimI(in),iR)
						simtrans(it)%ssim(in)	= gdecsim%savAret(ijret,1,simtrans(it+1)%dedsimI(in),iR)
						simtrans(it+1)%asim(in) = gdecsim%assAret(ijret,1,simtrans(it+1)%dedsimI(in),iR)

					ELSE		
				
						DO id = 1,ngpdret
							CALL LinInterp1(ngpxAret,xAgridret(ijret,:,ip),gdecsim%valAret(ijret,:,id,iR),lxA,lvaldedret(id))
						END DO
						idedchoice = maxloc(lvaldedret)
						simtrans(it+1)%dedsimI(in) = idedchoice(1)
						simtrans(it)%Vsim(in) = lvaldedret(simtrans(it+1)%dedsimI(in))
				
						CALL LinInterp1(ngpxAret,xAgridret(ijret,:,ip),gdecsim%savAret(ijret,:,simtrans(it+1)%dedsimI(in),iR),lxA,simtrans(it)%ssim(in))
						CALL LinInterp1(ngpxAret,xAgridret(ijret,:,ip),gdecsim%assAret(ijret,:,simtrans(it+1)%dedsimI(in),iR),lxA,simtrans(it+1)%asim(in))
				
					END IF
	
					!adjust for interpolating outside constraints
					IF(simtrans(it)%ssim(in)>lsavmax .or. simtrans(it)%ssim(in)<lsavmin .or. simtrans(it+1)%asim(in)<0.0) THEN
						IF(Display==2) write(*,*) 'interpolation problem in simulations (retirement)'
						simtrans(it)%ssim(in) = max(min(simtrans(it)%ssim(in),lsavmax),lsavmin)
						simtrans(it+1)%asim(in) = max(simtrans(it+1)%asim(in),0.0)
					END IF
					simtrans(it+1)%lsim(in) = FnInvQA(simtrans(it)%ssim(in)-qill*simtrans(it+1)%asim(in),0.0_8)
					!make sure doesnt leave a small amount in the bank due to interpolation:
					IF(simtrans(it+1)%asim(in)/(qill**(Jret-ijret)) <= fixcost + varcost*simtrans(it)%psim(in)) simtrans(it+1)%asim(in) = 0.0

					simtrans(it)%csim(in) = (lxA-fixcost-varcost*simtrans(it)%psim(in)-simtrans(it)%ssim(in))/(1.0+ctax)

					IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
					    if (Display==1) write(*,*) 'bad consumption (ADJ):',in,ij
					END IF


				ELSE !do not adjust

					simtrans(it)%adjsim(in) = 0.0
					simtrans(it+1)%asim(in) = lass*(1.0-dgridret(simtrans(it)%dedsimI(in))) /qill
					simtrans(it)%xsim(in) = lxN
					simtrans(it+1)%dedsimI(in) = simtrans(it)%dedsimI(in)
					lsavmax = lxN - cmin*(1.0+ctax)

					CALL BiLinInterp1RatioXAret(ngpxNret,xNgridret(ijret,:,:,ip,simtrans(it)%dedsimI(in)),ngparet,agridret(ijret,:,ip),gdecsim%valNret(ijret,:,:,simtrans(it)%dedsimI(in),iR),lxN,lass,simtrans(it)%Vsim(in))
					CALL BiLinInterp1RatioXAret(ngpxNret,xNgridret(ijret,:,:,ip,simtrans(it)%dedsimI(in)),ngparet,agridret(ijret,:,ip),gdecsim%savNret(ijret,:,:,simtrans(it)%dedsimI(in),iR),lxN,lass,simtrans(it)%ssim(in))			
					simtrans(it)%ssim(in) = max(min(simtrans(it)%ssim(in),lsavmax),lsavmin)

					simtrans(it+1)%lsim(in) = FnInvQA(simtrans(it)%ssim(in),0.0_8)
					simtrans(it)%csim(in) = (lxN-simtrans(it)%ssim(in))/(1.0+ctax)

					IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
						if (Display==1) write(*,*) 'bad consumption (NO ADJ):',in,ij
					END IF

				END IF


			   	IF(isnan(simtrans(it)%csim(in)) .or. simtrans(it)%csim(in)<=0.0) THEN
					if (Display==1) write(*,*) 'bad consumption (SimulateTransition retired):',in,ij,simtrans(it)%xsim(in),simtrans(it)%csim(in),simtrans(it)%adjsim(in),simtrans(it)%asim(in),simtrans(it)%lsim(in),simtrans(it)%dVsim(in),simtrans(it+1)%asim(in),simtrans(it+1)%lsim(in)
					simtrans(it)%csim(in) = cmin		!temporary fix, need to find bug here
			    END IF

				!adjust for rounding errors
				IF(abs(simtrans(it+1)%asim(in))<1.0e-5) simtrans(it+1)%asim(in) = 0.0
				IF(abs(simtrans(it+1)%lsim(in))<1.0e-5) simtrans(it+1)%lsim(in) = 0.0
			
			CASE(Jtot)	!last period, next asim and lsim are re-initialized: NOTE PROBLEM SINCE DOESNT DO ADJUSTMENT HERE..
				simtrans(it)%taxsim(in) = FnTax(ltaxsys,simtrans(it)%psim(in),0,0,ltaxincr)+ mturcg*simtrans(it)%asim(in)
				lxN = simtrans(it)%lsim(in) + simtrans(it)%psim(in) - simtrans(it)%taxsim(in) + dgridret(simtrans(it)%dedsimI(in))*simtrans(it)%asim(in) + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
				lxA = simtrans(it)%lsim(in) + simtrans(it)%psim(in) - simtrans(it)%taxsim(in) + simtrans(it)%asim(in) + simtrans(it)%rebsim(in)*(1.0-pcrebatespend)
				
				IF(lxA-fixcost-varcost*simtrans(it)%psim(in) > lxN) THEN
					simtrans(it)%csim(in) = (lxA-fixcost-varcost*simtrans(it)%psim(in)) / (1.0+ctax)
					simtrans(it)%adjsim(in) = 1.0					
				ELSE
					simtrans(it)%csim(in) = max(lxN,cmin) / (1.0+ctax)
					simtrans(it)%adjsim(in) = 0.0									
				END IF
				simtrans(it+1)%asim(in) = 0.0
				simtrans(it+1)%lsim(in) = 0.0			
				simtrans(it+1)%ssim(in) = 0.0			
				simtrans(it+1)%dedsimI(in) = 1				

		END SELECT

		!adjust for forced spending
		simtrans(it)%csim(in)   = simtrans(it)%csim(in) + pcrebatespend*simtrans(it)%rebsim(in) / (1.0+ctax) 
		
	END DO
	!$OMP END PARALLEL DO	

END DO

!adjust for housing
DO it = -1,Ttransition+Tadditionalsim
 simtrans(it)%ctotsim(:) = simtrans(it)%csim(:)
 simtrans(it)%csim(:) = ndweight*(simtrans(it)%ctotsim(:) + servflow*simtrans(it+1)%asim(:) + liqflow*MAX(simtrans(it+1)%lsim(:),0.0))
 simtrans(it)%flowsim(:) = (1.0d0-ndweight)*(simtrans(it)%ctotsim(:) + servflow*simtrans(it+1)%asim(:) + liqflow*MAX(simtrans(it+1)%lsim(:),0.0))
END DO

Simcstr5=0
Simcstr5noill=0

DO in=2*nsimrebateA+1,3*nsimrebateA
   ijtemp=simtrans(0)%jsim(in)
   IF(ijtemp .LE. Jwork-1) THEN
      IF( simtrans(0)%lsim(in) .EQ. abl(ijtemp,fysimtransI(in),simtrans(0)%zysimI(in)) .AND. &
           simtrans(1)%lsim(in) .EQ. abl(ijtemp+1,fysimtransI(in),simtrans(1)%zysimI(in)) .OR. &
           simtrans(0)%lsim(in) .EQ. 0.0d0 .AND. simtrans(1)%lsim(in) .EQ. 0.0d0 ) THEN
         IF(simtrans(0)%adjsim(in) .EQ. 0.0d0) THEN
            Simcstr5(in)=1
            Simcstr5(in-nsimrebateA)=1
            Simcstr5(in-2*nsimrebateA)=1
            IF(simtrans(1)%asim(in) .eq. 0.0d0) THEN
               Simcstr5noill(in)=1
               Simcstr5noill(in-nsimrebateA)=1
               Simcstr5noill(in-2*nsimrebateA)=1
            ENDIF

         ENDIF
      ENDIF
   ELSEIF(ijtemp .LT. Jtot) THEN
      IF( simtrans(0)%lsim(in) .EQ. 0.0d0 .and. simtrans(1)%lsim(in) .EQ. 0.0d0) THEN
         IF(simtrans(0)%adjsim(in) .EQ. 0.0d0) THEN
            Simcstr5(in)=1
            Simcstr5(in-nsimrebateA)=1
            Simcstr5(in-2*nsimrebateA)=1
            IF(simtrans(1)%asim(in) .EQ. 0.0d0) THEN
               Simcstr5noill(in)=1
               Simcstr5noill(in-nsimrebateA)=1
               Simcstr5noill(in-2*nsimrebateA)=1
            ENDIF
         ENDIF
      ENDIF
   ENDIF
         




ENDDO





END SUBROUTINE SimulateTransition
