# delimit;
set more 1;
clear;
cap clear matrix;
scalar drop _all;

set logtype text;

global VarList c y p l reb;
global Nsim 60000; *set to 600000 for full model;
global Tpath 6; *set to 14 for full model;
global trunclist 2.0;

use Simulations_long.dta, clear;


*organize data;
*grp=0: *counterfactual;
*grp=1: *grp A: early receivers;
*grp=2: *grp B: early receivers;

gen grp=0 if id>=2*$Nsim/3+1 &id<=$Nsim;
replace grp=1 if id>=1 &id<=$Nsim/3;
replace grp=2 if id>=$Nsim/3+1 &id<=2*$Nsim/3;

rename id oldid;
gen id = mod(oldid,200000);


*create some variables;
rename reb rebamt;
gen age2 = age^2;

sort id grp t;
by id grp: gen dCtp1 = c-c[_n-1];
tab t, gen(quarter);


*simulation specifc rebate coefficients;
bys t: egen meanreb = mean(rebamt);

sort id grp t;
by id: egen temp1 = mean(dCtp1) if rebamt>0 & meanreb>0 & grp>0;
by id: egen temp1a = max(temp1);

by id: egen temp0 = mean(dCtp1) if rebamt==0 & meanreb>0 & grp>0;
by id: egen temp0a = max(temp0);

qui sum rebamt if rebamt>0;
gen rebcoef_sim = (temp1a-temp0a)/r(mean);

sum rebcoef_sim;
gen streb= (rebcoef_sim-r(mean))/r(sd);

compress;
save RebateRegressionsDatasetOrganized.dta, replace;


use RebateRegressionsDatasetOrganized.dta, clear;

sum rebamt if rebamt>0;
scalar scmeanreb = r(mean);

*create true marginal propensities;
sort t id grp;
by t id: gen mpc = (c[2]-c[1])/rebamt[2] if t==2;
sum mpc if t==2 & grp==1;
scalar sc_mpc = r(mean);
by t id: gen mpcnews = (c[3]-c[1])/rebamt[2] if t==2;
sum mpcnews if t==2 & grp==1;
scalar sc_mpcnews = r(mean);

*create fraction of constrained;
sum simCstr5 if grp==0 & t==1;
scalar sc_cstr5 = r(mean);
sum simCstr5noill if grp==0 & t==1;
scalar sc_cstr5_poor = r(mean);
scalar sc_cstr5_wealthy = sc_cstr5 - sc_cstr5_poor;

foreach i of numlist $trunclist{;
	local i1 = `i'*10;

	*create true marginal propensities;
	sum mpc if t==2 & grp==1 & (streb>-`i' & streb<`i');
	scalar sc_mpc_trunc`i1' = r(mean);
	sum mpcnews if t==2 & grp==1 & (streb>-`i' & streb<`i');
	scalar sc_mpcnews_trunc`i1' = r(mean);

	*create fraction of constrained;
	sum simCstr5 if grp==0 & t==1 & (streb>-`i' & streb<`i');
	scalar sc_cstr5_trunc`i1' = r(mean);
	sum simCstr5noill if grp==0 & t==1 & (streb>-`i' & streb<`i');
	scalar sc_cstr5_poor_trunc`i1' = r(mean);
	scalar sc_cstr5_wealthy_trunc`i1' = sc_cstr5_trunc`i1' - sc_cstr5_poor_trunc`i1';

	*create true marginal propensities by constrained;
	sum mpc if t==2 & grp==1 & (streb>-`i' & streb<`i') & simCstr5==0;
	scalar sc_mpc_notcstr_trunc`i1' = r(mean);
	sum mpc if t==2 & grp==1 & (streb>-`i' & streb<`i') & simCstr5==1;
	scalar sc_mpc_cstr_trunc`i1' = r(mean);
	sum mpc if t==2 & grp==1 & (streb>-`i' & streb<`i') & simCstr5noill==1;
	scalar sc_mpc_cstr_poor_trunc`i1' = r(mean);
	sum mpc if t==2 & grp==1 & (streb>-`i' & streb<`i') & simCstr5==1 & simCstr5noill==0;
	scalar sc_mpc_cstr_wealthy_trunc`i1' = r(mean);
	
	*rebate coefficient decomposition;
	sum dCtp1 if t==2 & grp==1 & (streb>-`i' & streb<`i');
	scalar sc_dCgr1_t1_trunc`i1' = r(mean)/scmeanreb;
	sum dCtp1 if t==2 & grp==2 & (streb>-`i' & streb<`i');
	scalar sc_dCgr2_t1_trunc`i1' = r(mean)/scmeanreb;
	sum dCtp1 if t==3 & grp==1 & (streb>-`i' & streb<`i');
	scalar sc_dCgr1_t2_trunc`i1' = r(mean)/scmeanreb;
	sum dCtp1 if t==3 & grp==2 & (streb>-`i' & streb<`i');
	scalar sc_dCgr2_t2_trunc`i1' = r(mean)/scmeanreb;
	

};


*create true marginal propensities by constrained;
sum mpc if t==2 & grp==1 & simCstr5==0;
scalar sc_mpc_notcstr = r(mean);
sum mpc if t==2 & grp==1 & simCstr5==1;
scalar sc_mpc_cstr = r(mean);
sum mpc if t==2 & grp==1 & simCstr5noill==1;
scalar sc_mpc_cstr_poor = r(mean);
sum mpc if t==2 & grp==1 & simCstr5==1 & simCstr5noill==0;
scalar sc_mpc_cstr_wealthy = r(mean);

*rebate coefficient decomposition;
sum rebamt if rebamt>0;
scalar scmeanreb = r(mean);
sum dCtp1 if t==2 & grp==1;
scalar sc_dCgr1_t1 = r(mean)/scmeanreb;
sum dCtp1 if t==2 & grp==2;
scalar sc_dCgr2_t1 = r(mean)/scmeanreb;
sum dCtp1 if t==3 & grp==1;
scalar sc_dCgr1_t2 = r(mean)/scmeanreb;
sum dCtp1 if t==3 & grp==2;
scalar sc_dCgr2_t2 = r(mean)/scmeanreb;

reg dCtp1 rebamt quarter* age* if meanreb>0 & grp>0;
scalar N_ols = e(N);

foreach i of numlist $trunclist{;
	local i1 = `i'*10;

	di "***OLS WITH TRUNCATION ON STANDARDIZED REBATE COEFFICIENTS (`i')****";
	reg dCtp1 rebamt quarter* age* if (streb>-`i' & streb<`i') & meanreb>0 & grp>0;
	scalar reb_truncsim_st`i1' = _b[rebamt];
	scalar N_truncsim_st`i1' = e(N);
	scalar pcdrop_truncsim_st`i1' = 1-N_truncsim_st`i1'/N_ols;
};

scalar list;

cap log close;
qui log using rebate_coefficients_log.txt, replace;
scalar list;
qui log close;

preserve;
keep in 1;

foreach i of numlist $trunclist{;
	local i1 = `i'*10;

	local sclist reb_truncsim_st`i1' pcdrop_truncsim_st`i1'
				 sc_dCgr1_t1_trunc`i1' sc_dCgr2_t1_trunc`i1' sc_dCgr1_t2_trunc`i1' sc_dCgr2_t2_trunc`i1'
				 sc_mpc_trunc`i1' sc_mpcnews_trunc`i1';
	foreach s in `sclist'{;
		gen `s' = `s';
	};
	outsheet `sclist' using rebate_coefficients_list_trunc`i1'.txt, replace nonames;


	local sclist sc_mpc_notcstr_trunc`i1' sc_mpc_cstr_trunc`i1'
				 sc_mpc_cstr_wealthy_trunc`i1' sc_mpc_cstr_poor_trunc`i1'				 
				 sc_cstr5_trunc`i1' sc_cstr5_wealthy_trunc`i1' sc_cstr5_poor_trunc`i1';
	foreach s in `sclist'{;
		gen `s' = `s';
	};
	outsheet `sclist' using mpc_constrained_list_trunc`i1'.txt, replace nonames;
};
restore;




*aggregate transition path;
gen c_CF = c if grp==0;
gen c_grA = c if grp==1;
gen c_grB = c if grp==2;
gen c_TOT = c if grp==1 |grp==2;
gen reb_CF = rebamt if grp==0;
gen reb_grA = rebamt if grp==1;
gen reb_grB = rebamt if grp==2;
gen reb_TOT = rebamt if grp==1 |grp==2;

sum rebamt if rebamt>0;
gen totreb = r(mean);

foreach i of numlist $trunclist{;
	local i1 = `i'*10;

	preserve;
	keep if (streb>-`i' & streb<`i');
	collapse c_* reb_* totreb, by(t);
	outsheet using counterfactual_transition_path_trunc`i1'.txt, replace;
	restore;
};


