# delimit;
set more 1;

clear;
cap clear matrix;
set logtype text;

global SaveDir ; *location to save figures goes here;
global SimsDir ; *location of fortran output goes here;
global streb_threshold 2.0;

cd $SimsDir;

use RebateRegressionsDatasetOrganized.dta, clear;

sum rebcoef_sim;
gen astreb= abs(streb);
drop if astreb>=$streb_threshold;

gen rebcoef_trunc = rebcoef_sim;
replace rebcoef_trunc =1 if rebcoef_trunc>1;
replace rebcoef_trunc =0 if rebcoef_trunc<0;

keep if grp==1 & t==1;

qui sum rebcoef_trunc if rebcoef_trunc>0;
scalar meanpos_rebcoef_trunc =r(mean);
local mprc = meanpos_rebcoef_trunc;

*Figure 7a;
hist rebcoef_trunc, bin(20) frac
	xtitle("Rebate Coefficient", size(large))
	ytitle("Fraction of Households", size(large))
	color(red) lcolor(black) scheme(s1color);
graph export $SaveDir/fig_7a.eps, replace;


*Figure 7b;
gen inc = y if age<=152;
replace inc = p if age >152;

preserve;
local nq=5;
local nqinv = 100/`nq'; 
xtile rcpctile = rebcoef_sim, nq(`nq');
replace inc = inc*4*53000;

collapse (median) medinc= inc (mean) avinc = inc if age<=152, by(rcpctile) fast;
replace rcpctile = rcpctile*`nqinv';

twoway bar medinc rcpctile, barw(`nqinv')
	xtitle("Percentile of Rebate Coefficient Distribution", size(large)) ytitle("Median Earnings", size(large))
	color(red) lcolor(black) scheme(s1color) yscale(range(0)) ylabel(0(20000)100000);
graph export $SaveDir/fig_7b.eps, replace;
restore;
